@@ -29,8 +29,7 @@
  *
  *
  *    Usage:
- *     (NT): AppendTransform.dll <filename>
- *     (Solaris): append-transform.so <filename>
+ *      append-transform.so <filename>
  *
  *              <filename> is the name of the file containing the
  *              text to be appended
@@ -88,8 +87,8 @@ handle_transform(TSCont contp)
   TSVConn output_conn;
   TSVIO write_vio;
   MyData *data;
-  int towrite;
-  int avail;
+  int64_t towrite;
+  int64_t avail;
 
   /* Get the output connection where we'll write data to. */
   output_conn = TSTransformOutputVConnGet(contp);
@@ -268,11 +267,7 @@ transformable(TSHttpTxn txnp)
     }
 
     value = TSMimeHdrFieldValueStringGet(bufp, hdr_loc, field_loc, 0, &val_length);
-#ifndef _WIN32
     if (value && (strncasecmp(value, "text/html", sizeof("text/html") - 1) == 0)) {
-#else
-    if (value && (strnicmp(value, "text/html", sizeof("text/html") - 1) == 0)) {
-#endif
       ASSERT_SUCCESS(TSHandleMLocRelease(bufp, hdr_loc, field_loc));
       ASSERT_SUCCESS(TSHandleMLocRelease(bufp, TS_NULL_MLOC, hdr_loc));
 
