@@ -27,8 +27,7 @@
  *
  *
  *	Usage:
- * 	(NT): NullTransform.dll
- *	(Solaris): null-transform.so
+ *	  null-transform.so
  *
  *
  */
@@ -104,7 +103,7 @@ handle_transform(TSCont contp)
     data = my_data_alloc();
     data->output_buffer = TSIOBufferCreate();
     data->output_reader = TSIOBufferReaderAlloc(data->output_buffer);
-    TSDebug("null-transform", "\tWriting %d bytes on VConn", TSVIONBytesGet(input_vio));
+    TSDebug("null-transform", "\tWriting %" PRId64 " bytes on VConn", TSVIONBytesGet(input_vio));
     //data->output_vio = TSVConnWrite(output_conn, contp, data->output_reader, INT32_MAX);
     data->output_vio = TSVConnWrite(output_conn, contp, data->output_reader, INT64_MAX);
     // data->output_vio = TSVConnWrite(output_conn, contp, data->output_reader, TSVIONBytesGet(input_vio));
