@@ -356,6 +356,7 @@ struct CacheVC: public CacheVConnection
   virtual void set_http_info(CacheHTTPInfo *info);
   virtual void get_http_info(CacheHTTPInfo ** info);
 #endif
+  virtual bool is_pread_capable();
   virtual bool set_pin_in_cache(time_t time_pin);
   virtual time_t get_pin_in_cache();
   virtual bool set_disk_io_priority(int priority);
@@ -755,6 +756,17 @@ CacheVC::get_frag_table() {
   return 0;
 }
 
+TS_INLINE bool
+CacheVC::is_pread_capable() {
+/* "fix" for Range related crashers: */
+#if defined(RANGES_DONT_WORK_VERY_WELL_ON_3_2_X)
+  ink_debug_assert(od);
+  return od->vector.count() <= 1;
+#else
+  return false;
+#endif
+}
+
 TS_INLINE int
 Vol::close_write(CacheVC *cont)
 {
