@@ -289,7 +289,7 @@ RamCacheCLFUSEntry *RamCacheCLFUS::destroy(RamCacheCLFUSEntry *e) {
   if (!e->flag_bits.lru) {
     objects--;
     bytes -= e->size + ENTRY_OVERHEAD;
-    CACHE_SUM_DYN_STAT_THREAD(cache_ram_cache_bytes_stat, -e->size);
+    CACHE_SUM_DYN_STAT_THREAD(cache_ram_cache_bytes_stat, -(int64_t)e->size);
     e->data = NULL;
   } else
     history--;
@@ -510,7 +510,7 @@ int RamCacheCLFUS::put(INK_MD5 *key, IOBufferData *data, uint32_t len, bool copy
       return 0;
     }
     bytes -= victim->size + ENTRY_OVERHEAD;
-    CACHE_SUM_DYN_STAT_THREAD(cache_ram_cache_bytes_stat, -victim->size);
+    CACHE_SUM_DYN_STAT_THREAD(cache_ram_cache_bytes_stat, -(int64_t)victim->size);
     victims.enqueue(victim);
     if (victim == compressed)
       compressed = 0;
