@@ -33,6 +33,7 @@
 /*************************************************************************/
 int cluster_port_number = DEFAULT_CLUSTER_PORT_NUMBER;
 int cache_clustering_enabled = 0;
+int num_of_cluster_threads = DEFAULT_NUMBER_OF_CLUSTER_THREADS;
 
 ClusterProcessor clusterProcessor;
 RecRawStatBlock *cluster_rsb = NULL;
@@ -51,7 +52,7 @@ ClusterProcessor::~ClusterProcessor()
 }
 
 int
-ClusterProcessor::internal_invoke_remote(ClusterMachine * m, int cluster_fn,
+ClusterProcessor::internal_invoke_remote(ClusterHandler *ch, int cluster_fn,
                                          void *data, int len, int options, void *cmsg)
 {
   EThread *thread = this_ethread();
@@ -66,11 +67,10 @@ ClusterProcessor::internal_invoke_remote(ClusterMachine * m, int cluster_fn,
   bool malloced = (cluster_fn == CLUSTER_FUNCTION_MALLOCED);
   OutgoingControl *c;
 
-  ClusterHandler *ch = m->clusterHandler;
   if (!ch || (!malloced && !((unsigned int) cluster_fn < (uint32_t) SIZE_clusterFunction))) {
     // Invalid message or node is down, free message data
     if (malloced) {
-      xfree(data);
+      ats_free(data);
     }
     if (cmsg) {
       invoke_remote_data_args *args = (invoke_remote_data_args *)
@@ -93,7 +93,6 @@ ClusterProcessor::internal_invoke_remote(ClusterMachine * m, int cluster_fn,
     c = OutgoingControl::alloc();
   }
   CLUSTER_INCREMENT_DYN_STAT(CLUSTER_CTRL_MSGS_SENT_STAT);
-  c->m = m;
   c->submit_time = ink_get_hrtime();
 
   if (malloced) {
@@ -135,7 +134,9 @@ ClusterProcessor::internal_invoke_remote(ClusterMachine * m, int cluster_fn,
 
       MUTEX_TRY_LOCK(lock, ch->mutex, tt);
       if (!lock) {
-        return 1;
+		if(ch->thread && ch->thread->signal_hook)
+		  ch->thread->signal_hook(ch->thread);
+		return 1;
       }
       if (steal)
         ch->steal_thread(tt);
@@ -143,19 +144,19 @@ ClusterProcessor::internal_invoke_remote(ClusterMachine * m, int cluster_fn,
     }
   } else {
     c->mutex = ch->mutex;
-    eventProcessor.schedule_imm(c);
+    eventProcessor.schedule_imm_signal(c);
     return 0;
   }
 }
 
 int
-ClusterProcessor::invoke_remote(ClusterMachine * m, int cluster_fn, void *data, int len, int options)
+ClusterProcessor::invoke_remote(ClusterHandler *ch, int cluster_fn, void *data, int len, int options)
 {
-  return internal_invoke_remote(m, cluster_fn, data, len, options, (void *) NULL);
+  return internal_invoke_remote(ch, cluster_fn, data, len, options, (void *) NULL);
 }
 
 int
-ClusterProcessor::invoke_remote_data(ClusterMachine * m, int cluster_fn,
+ClusterProcessor::invoke_remote_data(ClusterHandler *ch, int cluster_fn,
                                      void *data, int data_len,
                                      IOBufferBlock * buf,
                                      int dest_channel, ClusterVCToken * token,
@@ -163,7 +164,7 @@ ClusterProcessor::invoke_remote_data(ClusterMachine * m, int cluster_fn,
 {
   if (!buf) {
     // No buffer data, translate this into a invoke_remote() request
-    return internal_invoke_remote(m, cluster_fn, data, data_len, options, (void *) NULL);
+    return internal_invoke_remote(ch, cluster_fn, data, data_len, options, (void *) NULL);
   }
   ink_assert(data);
   ink_assert(data_len);
@@ -194,7 +195,7 @@ ClusterProcessor::invoke_remote_data(ClusterMachine * m, int cluster_fn,
   *(int32_t *) chdr->data = -1;   // always -1 for compound message
   memcpy(chdr->data + sizeof(int32_t), (char *) &mh, sizeof(mh));
 
-  return internal_invoke_remote(m, cluster_fn, data, data_len, options, (void *) chdr);
+  return internal_invoke_remote(ch, cluster_fn, data, data_len, options, (void *) chdr);
 }
 
 void
@@ -236,7 +237,7 @@ ClusterProcessor::open_local(Continuation * cont, ClusterMachine * m, ClusterVCT
   bool immediate = ((options & CLUSTER_OPT_IMMEDIATE) ? true : false);
   bool allow_immediate = ((options & CLUSTER_OPT_ALLOW_IMMEDIATE) ? true : false);
 
-  ClusterHandler *ch = m->clusterHandler;
+  ClusterHandler *ch = ((CacheContinuation *)cont)->ch;
   if (!ch)
     return NULL;
   EThread *t = ch->thread;
@@ -249,8 +250,9 @@ ClusterProcessor::open_local(Continuation * cont, ClusterMachine * m, ClusterVCT
   vc->new_connect_read = (options & CLUSTER_OPT_CONN_READ ? 1 : 0);
   vc->start_time = ink_get_hrtime();
   vc->last_activity_time = vc->start_time;
-  vc->machine = m;
+  vc->ch = ch;
   vc->token.alloc();
+  vc->token.ch_id = ch->id;
   token = vc->token;
 #ifdef CLUSTER_THREAD_STEALING
   CLUSTER_INCREMENT_DYN_STAT(CLUSTER_CONNECTIONS_OPENNED_STAT);
@@ -264,6 +266,8 @@ ClusterProcessor::open_local(Continuation * cont, ClusterMachine * m, ClusterVCT
     }
     vc->action_ = cont;
     ink_atomiclist_push(&ch->external_incoming_open_local, (void *) vc);
+	if(ch->thread && ch->thread->signal_hook)
+	  ch->thread->signal_hook(ch->thread);
     return CLUSTER_DELAYED_OPEN;
 
 #ifdef CLUSTER_THREAD_STEALING
@@ -301,7 +305,9 @@ ClusterProcessor::connect_local(Continuation * cont, ClusterVCToken * token, int
 #endif
   if (!m)
     return NULL;
-  ClusterHandler *ch = m->clusterHandler;
+  if (token->ch_id >= (uint32_t)m->num_connections)
+    return NULL;
+  ClusterHandler *ch = m->clusterHandlers[token->ch_id];
   if (!ch)
     return NULL;
   EThread *t = ch->thread;
@@ -314,7 +320,7 @@ ClusterProcessor::connect_local(Continuation * cont, ClusterVCToken * token, int
   vc->new_connect_read = (options & CLUSTER_OPT_CONN_READ ? 1 : 0);
   vc->start_time = ink_get_hrtime();
   vc->last_activity_time = vc->start_time;
-  vc->machine = m;
+  vc->ch = ch;
   vc->token = *token;
   vc->channel = channel;
 #ifdef CLUSTER_THREAD_STEALING
@@ -329,7 +335,7 @@ ClusterProcessor::connect_local(Continuation * cont, ClusterVCToken * token, int
     }
     vc->mutex = ch->mutex;
     vc->action_ = cont;
-    ch->thread->schedule_imm(vc);
+    ch->thread->schedule_imm_signal(vc);
     return CLUSTER_DELAYED_OPEN;
 #ifdef CLUSTER_THREAD_STEALING
   } else {
@@ -349,11 +355,7 @@ ClusterProcessor::connect_local(Continuation * cont, ClusterVCToken * token, int
 
 bool ClusterProcessor::disable_remote_cluster_ops(ClusterMachine * m)
 {
-  if (!m)
-    return false;
-
-  ClusterHandler *
-    ch = m->clusterHandler;
+  ClusterHandler *ch = m->pop_ClusterHandler(1);
   if (ch) {
     return ch->disable_remote_cluster_ops;
   } else {
@@ -378,6 +380,8 @@ int CacheClusterMonitorIntervalSecs = 1;
 int cluster_send_buffer_size = 0;
 int cluster_receive_buffer_size = 0;
 unsigned long cluster_sockopt_flags = 0;
+unsigned long cluster_packet_mark = 0;
+unsigned long cluster_packet_tos = 0;
 
 int RPC_only_CacheCluster = 0;
 #endif
@@ -667,12 +671,16 @@ ClusterProcessor::init()
   else {
     IOCORE_ReadConfigInteger(cluster_port, "proxy.config.cluster.cluster_port");
   }
+  if (num_of_cluster_threads == DEFAULT_NUMBER_OF_CLUSTER_THREADS)
+    IOCORE_ReadConfigInteger(num_of_cluster_threads, "proxy.config.cluster.threads");
 
   IOCORE_EstablishStaticConfigInt32(CacheClusterMonitorEnabled, "proxy.config.cluster.enable_monitor");
   IOCORE_EstablishStaticConfigInt32(CacheClusterMonitorIntervalSecs, "proxy.config.cluster.monitor_interval_secs");
   IOCORE_ReadConfigInteger(cluster_receive_buffer_size, "proxy.config.cluster.receive_buffer_size");
   IOCORE_ReadConfigInteger(cluster_send_buffer_size, "proxy.config.cluster.send_buffer_size");
   IOCORE_ReadConfigInteger(cluster_sockopt_flags, "proxy.config.cluster.sock_option_flag");
+  IOCORE_ReadConfigInteger(cluster_packet_mark, "proxy.config.cluster.sock_packet_mark");
+  IOCORE_ReadConfigInteger(cluster_packet_tos, "proxy.config.cluster.sock_packet_tos");
   IOCORE_EstablishStaticConfigInt32(RPC_only_CacheCluster, "proxy.config.cluster.rpc_cache_cluster");
 
   int cluster_type = 0;
@@ -723,7 +731,7 @@ ClusterProcessor::start()
 #endif
   if (cache_clustering_enabled && (cacheProcessor.IsCacheEnabled() == CACHE_INITIALIZED)) {
 
-    ET_CLUSTER = eventProcessor.spawn_event_threads(1, "ET_CLUSTER");
+    ET_CLUSTER = eventProcessor.spawn_event_threads(num_of_cluster_threads, "ET_CLUSTER");
     for (int i = 0; i < eventProcessor.n_threads_for_type[ET_CLUSTER]; i++) {
       initialize_thread_for_net(eventProcessor.eventthread[ET_CLUSTER][i], i);
     }
@@ -742,20 +750,21 @@ ClusterProcessor::start()
 }
 
 void
-ClusterProcessor::connect(char *hostname)
+ClusterProcessor::connect(char *hostname, int16_t id)
 {
   //
   // Construct a cluster link to the given machine
   //
   ClusterHandler *ch = NEW(new ClusterHandler);
   SET_CONTINUATION_HANDLER(ch, (ClusterContHandler) & ClusterHandler::connectClusterEvent);
-  ch->hostname = xstrdup(hostname);
+  ch->hostname = ats_strdup(hostname);
   ch->connector = true;
+  ch->id = id;
   eventProcessor.schedule_imm(ch, ET_CLUSTER);
 }
 
 void
-ClusterProcessor::connect(unsigned int ip, int port, bool delay)
+ClusterProcessor::connect(unsigned int ip, int port, int16_t id, bool delay)
 {
   //
   // Construct a cluster link to the given machine
@@ -765,6 +774,7 @@ ClusterProcessor::connect(unsigned int ip, int port, bool delay)
   ch->ip = ip;
   ch->port = port;
   ch->connector = true;
+  ch->id = id;
   if (delay)
     eventProcessor.schedule_in(ch, CLUSTER_MEMBER_DELAY, ET_CLUSTER);
   else
@@ -803,7 +813,7 @@ ClusterProcessor::send_machine_list(ClusterMachine * m)
     //////////////////////////////////////////////////////////////
     ink_release_assert(!"send_machine_list() bad msg version");
   }
-  invoke_remote(m, MACHINE_LIST_CLUSTER_FUNCTION, data, len);
+  invoke_remote(m->pop_ClusterHandler(), MACHINE_LIST_CLUSTER_FUNCTION, data, len);
 }
 
 void
