@@ -162,12 +162,11 @@ TS_INLINE bool SplitDNSConfig::isSplitDNSEnabled()
    -------------------------------------------------------------- */
 struct DNSServer
 {
-  unsigned int x_server_ip[MAXNS];
+  IpEndpoint x_server_ip[MAXNS];
   char x_dns_ip_line[MAXDNAME * 2];
 
   char x_def_domain[MAXDNAME];
   char x_domain_srch_list[MAXDNAME];
-  int  x_dns_server_port[MAXNS];
 
   DNSHandler *x_dnsH;
 
@@ -175,7 +174,6 @@ struct DNSServer
   : x_dnsH(NULL)
   {
     memset(x_server_ip, 0, sizeof(x_server_ip));
-    memset(x_dns_server_port, 0, sizeof(x_dns_server_port));
 
     memset(x_def_domain, 0, MAXDNAME);
     memset(x_domain_srch_list, 0, MAXDNAME);
@@ -199,8 +197,8 @@ public:
 
   const char *get_host();
 
-  ip_addr_t get_ip();
-  ip_addr_t get_client_ip();
+  sockaddr const* get_ip(); // unused required virtual method.
+  sockaddr const* get_client_ip(); // unused required virtual method.
 
   const char *m_pHost;
 };
@@ -221,7 +219,7 @@ TS_INLINE DNSRequestData::DNSRequestData()
 TS_INLINE char *
 DNSRequestData::get_string()
 {
-  return xstrdup((char *) m_pHost);
+  return ats_strdup((char *) m_pHost);
 }
 
 
@@ -238,18 +236,18 @@ DNSRequestData::get_host()
 /* --------------------------------------------------------------
    DNSRequestData::get_ip()
    -------------------------------------------------------------- */
-TS_INLINE ip_addr_t DNSRequestData::get_ip()
+TS_INLINE sockaddr const* DNSRequestData::get_ip()
 {
-  return (ip_addr_t) 0;
+  return NULL;
 }
 
 
 /* --------------------------------------------------------------
    DNSRequestData::get_client_ip()
    -------------------------------------------------------------- */
-TS_INLINE ip_addr_t DNSRequestData::get_client_ip()
+TS_INLINE sockaddr const* DNSRequestData::get_client_ip()
 {
-  return (ip_addr_t) 0;
+  return NULL;
 }
 
 /* --------------------------------------------------------------
