@@ -29,7 +29,6 @@
 
 #define MAX_LOCK_TIME	HRTIME_MSECONDS(200)
 #define THREAD_MUTEX_THREAD_HOLDING	(-1024*1024)
-#define HANDLER_NAME(_c) _c?_c->handler_name:(char*)NULL
 
 class EThread;
 typedef EThread *EThreadPtr;
@@ -65,10 +64,9 @@ extern void lock_taken(const char *file, int line, const char *handler);
   allow you to lock/unlock the underlying mutex object.
 
 */
-class ProxyMutex:public RefCountObj
+class ProxyMutex: public RefCountObj
 {
 public:
-
   /**
     Underlying mutex object.
 
@@ -87,7 +85,7 @@ public:
 
   */
   volatile EThreadPtr thread_holding;
-
+  
   int nthread_holding;
 
 #ifdef DEBUG
@@ -118,7 +116,7 @@ public:
     new_ProxyMutex function, which provides a faster allocation.
 
   */
-    ProxyMutex()
+  ProxyMutex()
   {
     thread_holding = NULL;
     nthread_holding = 0;
@@ -172,14 +170,15 @@ extern inkcoreapi ClassAllocator<ProxyMutex> mutexAllocator;
 class ProxyMutexPtr
 {
 public:
-
   /**
     Constructor of the ProxyMutexPtr class. You can provide a pointer
     to a ProxyMutex object or set it at a later time with the
     assignment operator.
 
   */
-ProxyMutexPtr(ProxyMutex * ptr = 0):m_ptr(ptr) {
+  ProxyMutexPtr(ProxyMutex * ptr = 0)
+    : m_ptr(ptr)
+  {
     if (m_ptr)
       REF_COUNT_OBJ_REFCOUNT_INC(m_ptr);
   }
@@ -194,7 +193,8 @@ ProxyMutexPtr(ProxyMutex * ptr = 0):m_ptr(ptr) {
     @param src Constant ref to the ProxyMutexPtr to initialize from.
 
   */
-  ProxyMutexPtr(const ProxyMutexPtr & src):m_ptr(src.m_ptr)
+  ProxyMutexPtr(const ProxyMutexPtr & src)
+    : m_ptr(src.m_ptr)
   {
     if (m_ptr)
       REF_COUNT_OBJ_REFCOUNT_INC(m_ptr);
@@ -206,7 +206,8 @@ ProxyMutexPtr(ProxyMutex * ptr = 0):m_ptr(ptr) {
     reference to the object (reference count equal to zero).
 
   */
-  ~ProxyMutexPtr() {
+  ~ProxyMutexPtr()
+  {
     if (m_ptr && !m_ptr->refcount_dec())
       m_ptr->free();
   }
@@ -222,7 +223,8 @@ ProxyMutexPtr(ProxyMutex * ptr = 0):m_ptr(ptr) {
     @param p The reference to the ProxyMutex object.
 
   */
-  ProxyMutexPtr & operator =(ProxyMutex * p) {
+  ProxyMutexPtr & operator =(ProxyMutex * p)
+  {
     ProxyMutex *temp_ptr = m_ptr;
     if (m_ptr == p)
       return (*this);
@@ -246,7 +248,8 @@ ProxyMutexPtr(ProxyMutex * ptr = 0):m_ptr(ptr) {
     @param src The ProxyMutexPtr to get the ProxyMutex reference from.
 
   */
-  ProxyMutexPtr & operator =(const ProxyMutexPtr & src) {
+  ProxyMutexPtr & operator =(const ProxyMutexPtr & src)
+  {
     return (operator =(src.m_ptr));
   }
 
@@ -281,7 +284,7 @@ ProxyMutexPtr(ProxyMutex * ptr = 0):m_ptr(ptr) {
   */
   operator  ProxyMutex *() const
   {
-    return (m_ptr);
+    return m_ptr;
   }
 
   /**
@@ -294,7 +297,7 @@ ProxyMutexPtr(ProxyMutex * ptr = 0):m_ptr(ptr) {
   */
   ProxyMutex *operator ->() const
   {
-    return (m_ptr);
+    return m_ptr;
   }
 
   /**
@@ -574,19 +577,14 @@ struct MutexLock
     m.clear();
   }
 
-  ~MutexLock() {
+  ~MutexLock()
+  {
     if (m)
       Mutex_unlock(m, m->thread_holding);
   }
 
-  int operator!()
-  {
-    return false;
-  }
-  operator  bool()
-  {
-    return true;
-  }
+  int operator!() const { return false; }
+  operator bool() { return true; }
 };
 
 struct MutexTryLock
@@ -624,7 +622,8 @@ struct MutexTryLock
         m = am;
   }
 
-  ~MutexTryLock() {
+  ~MutexTryLock()
+  {
     if (m.m_ptr)
       Mutex_unlock(m.m_ptr, m.m_ptr->thread_holding);
   }
@@ -638,14 +637,8 @@ struct MutexTryLock
     lock_acquired = false;
   }
 
-  int operator!()
-  {
-    return !lock_acquired;
-  }
-  operator  bool()
-  {
-    return lock_acquired;
-  }
+  int operator!() const { return !lock_acquired; }
+  operator bool() const { return lock_acquired; }
 };
 
 inline void
@@ -738,7 +731,6 @@ MutexTryLock _l(__FILE__,__LINE__,(char*)NULL,_m,_t)
 #    define MUTEX_TRY_LOCK_SPIN(_l,_m,_t,_sc) \
 MutexTryLock _l(__FILE__,__LINE__,(char*)NULL,_m,_t,_sc)
 
-#    ifdef PURIFY
 
 /**
   Attempts to acquire the lock to the ProxyMutex.
@@ -754,12 +746,9 @@ MutexTryLock _l(__FILE__,__LINE__,(char*)NULL,_m,_t,_sc)
   @param _c Continuation whose mutex will be attempted to lock.
 
 */
-#      define MUTEX_TRY_LOCK_FOR(_l,_m,_t,_c) \
-MutexTryLock _l(__FILE__,__LINE__,(char *)NULL,_m,_t)
-#    else //PURIFY
-#      define MUTEX_TRY_LOCK_FOR(_l,_m,_t,_c) \
-MutexTryLock _l(__FILE__,__LINE__,HANDLER_NAME(_c),_m,_t)
-#    endif //PURIFY
+
+#    define MUTEX_TRY_LOCK_FOR(_l,_m,_t,_c) \
+MutexTryLock _l(__FILE__,__LINE__,NULL,_m,_t)
 #  else //DEBUG
 #    define MUTEX_TRY_LOCK(_l,_m,_t) MutexTryLock _l(_m,_t)
 #    define MUTEX_TRY_LOCK_SPIN(_l,_m,_t,_sc) MutexTryLock _l(_m,_t,_sc)
@@ -785,17 +774,10 @@ MutexTryLock _l(__FILE__,__LINE__,HANDLER_NAME(_c),_m,_t)
 #ifdef DEBUG
 #  define MUTEX_TAKE_TRY_LOCK(_m,_t) \
 Mutex_trylock(__FILE__,__LINE__,(char*)NULL,_m,_t)
-#  ifdef PURIFY
-#    define MUTEX_TAKE_TRY_LOCK_FOR(_m,_t,_c) \
-Mutex_trylock(__FILE__,__LINE__,(char *)NULL,_m,_t)
-#    define MUTEX_TAKE_TRY_LOCK_FOR_SPIN(_m,_t,_c,_sc) \
-Mutex_trylock_spin(__FILE__,__LINE__,(char *)NULL,_m,_t,_sc)
-#  else
-#    define MUTEX_TAKE_TRY_LOCK_FOR(_m,_t,_c) \
+#  define MUTEX_TAKE_TRY_LOCK_FOR(_m,_t,_c) \
 Mutex_trylock(__FILE__,__LINE__,(char*)NULL,_m,_t)
-#    define MUTEX_TAKE_TRY_LOCK_FOR_SPIN(_m,_t,_c,_sc) \
-Mutex_trylock_spin(__FILE__,__LINE__,HANDLER_NAME(_c),_m,_t,_sc)
-#  endif
+#  define MUTEX_TAKE_TRY_LOCK_FOR_SPIN(_m,_t,_c,_sc) \
+Mutex_trylock_spin(__FILE__,__LINE__,NULL,_m,_t,_sc)
 #else
 #  define MUTEX_TAKE_TRY_LOCK(_m,_t) Mutex_trylock(_m,_t)
 #  define MUTEX_TAKE_TRY_LOCK_FOR(_m,_t,_c) Mutex_trylock(_m,_t)
@@ -808,13 +790,8 @@ Mutex_trylock_spin(_m,_t,_sc)
 Mutex_lock(__FILE__,__LINE__,(char*)NULL,_m,_t)
 #  define MUTEX_SET_AND_TAKE_LOCK(_s,_m,_t)\
 _s.set_and_take(__FILE__,__LINE__,(char*)NULL,_m,_t)
-#  ifdef PURIFY
-#    define MUTEX_TAKE_LOCK_FOR(_m,_t,_c) \
-Mutex_lock(__FILE__,__LINE__,(char *)NULL,_m,_t)
-#  else
-#    define MUTEX_TAKE_LOCK_FOR(_m,_t,_c) \
-Mutex_lock(__FILE__,__LINE__,HANDLER_NAME(_c),_m,_t)
-#  endif //PURIFY
+#  define MUTEX_TAKE_LOCK_FOR(_m,_t,_c) \
+Mutex_lock(__FILE__,__LINE__,NULL,_m,_t)
 #else
 #  define MUTEX_TAKE_LOCK(_m,_t) Mutex_lock(_m,_t)
 #  define MUTEX_SET_AND_TAKE_LOCK(_s,_m,_t)_s.set_and_take(_m,_t)
