@@ -36,7 +36,6 @@
 #define EVENT_SRV_IP_REMOVED             (SRV_EVENT_EVENTS_START+1)
 #define EVENT_SRV_GET_RESPONSE           (SRV_EVENT_EVENTS_START+2)
 
-#define PATH_NAME_MAX 511
 #define HOST_DB_MAX_ROUND_ROBIN_INFO         16
 
 //
@@ -119,21 +118,18 @@ struct HostDBRoundRobin;
 
 struct HostDBInfo
 {
-  // Public Interface
-  unsigned int &ip()  {
-    return data.ip;
-  }
+  /** Internal IP address data.
+      This is at least large enough to hold an IPv6 address.
+  */
+  sockaddr* ip()  { return &data.ip.sa; }
+  sockaddr const* ip() const { return &data.ip.sa; }
 
   char *hostname();
   char *srvname();
   HostDBRoundRobin *rr();
 
   /** Indicate that the HostDBInfo is BAD and should be deleted. */
-  void bad()
-  {
-    full = 0;
-  }
-
+  void bad() { full = 0; }
 
   /** Check the HostDBInfo or selected RR entry of a HostDBInfo is ok. */
   int ok(bool byname, HostDBInfo * rr = NULL) {
@@ -145,7 +141,7 @@ struct HostDBInfo
     } else if (byname) {
       if (reverse_dns)
         goto Lbad;
-      if (!ip())
+      if (!ats_is_ip(ip()))
         goto Lbad;
     } else {
       if (!reverse_dns)
@@ -220,27 +216,25 @@ struct HostDBInfo
     database. Any new user fields must be added to this function.
 
   */
-  void set_from(HostDBInfo & info)
+  void set_from(HostDBInfo const& that)
   {
-    ip() = info.ip();
-    ip_timestamp = info.ip_timestamp;
-    ip_timeout_interval = info.ip_timeout_interval;
-    round_robin = info.round_robin;
-    reverse_dns = info.reverse_dns;
-    app.allotment.application1 = info.app.allotment.application1;
-    app.allotment.application2 = info.app.allotment.application2;
+    memcpy(&data, &that.data, sizeof data);
+    ip_timestamp = that.ip_timestamp;
+    ip_timeout_interval = that.ip_timeout_interval;
+    round_robin = that.round_robin;
+    reverse_dns = that.reverse_dns;
+    app.allotment.application1 = that.app.allotment.application1;
+    app.allotment.application2 = that.app.allotment.application2;
   }
 
 
   //
   // Private
   //
-  union
-  {
-    unsigned int ip;
-    int hostname_offset;
-    // int srv_host_offset;
-    uint64_t dummy_pad;
+
+  union {
+    IpEndpoint ip; ///< IP address / port data.
+    int hostname_offset; ///< Some hostname thing.
   } data;
 
   unsigned int srv_weight:16;
@@ -266,10 +260,8 @@ struct HostDBInfo
 
   uint64_t md5_high;
 
-  sockaddr_in6 ip6;
-
-  bool failed() { return !ip(); }
-  void set_failed() { ip() = 0;  }
+  bool failed() { return !ats_is_ip(ip()); }
+  void set_failed() { ats_ip_invalidate(ip());  }
 
   void set_deleted() { deleted = 1; }
   bool is_deleted() const { return deleted; }
@@ -302,7 +294,7 @@ struct HostDBInfo
 
   void reset()
   {
-    ip() = 0;
+    ats_ip_invalidate(ip());
     app.allotment.application1 = 0;
     app.allotment.application2 = 0;
     backed = 0;
@@ -322,17 +314,24 @@ struct HostDBInfo
   int *heap_offset_ptr();
 
 HostDBInfo()
-  : srv_weight(0), srv_priority(0), srv_port(0), srv_count(0), is_srv(0),
-    ip_timestamp(0),
-    ip_timeout_interval(0), full(0), backed(0), deleted(0), hits(0), round_robin(0), reverse_dns(0), md5_low_low(0),
-    md5_low(0), md5_high(0) {
-#ifdef PURIFY
-    memset(&app, 0, sizeof(app));
-#else
+  : srv_weight(0)
+  , srv_priority(0)
+  , srv_port(0)
+  , srv_count(0)
+  , is_srv(0)
+  , ip_timestamp(0)
+  , ip_timeout_interval(0)
+  , full(0)
+  , backed(0)
+  , deleted(0)
+  , hits(0)
+  , round_robin(0)
+  , reverse_dns(0)
+  , md5_low_low(0)
+  , md5_low(0), md5_high(0) {
     app.allotment.application1 = 0;
     app.allotment.application2 = 0;
-#endif
-    ip() = 0;
+    ats_ip_invalidate(ip());
 
     return;
   }
@@ -348,6 +347,7 @@ struct HostDBRoundRobin
   short good;
 
   unsigned short current;
+  ink_time_t timed_rr_ctime;
 
   HostDBInfo info[HOST_DB_MAX_ROUND_ROBIN_INFO];
   char rr_srv_hosts[HOST_DB_MAX_ROUND_ROBIN_INFO][MAXDNAME];
@@ -371,10 +371,9 @@ struct HostDBRoundRobin
     }
   }
 
-  HostDBInfo *find_ip(unsigned int ip);
-  HostDBInfo *select_best(unsigned int client_ip, HostDBInfo * r = NULL);
-
-  HostDBInfo *select_best_http(unsigned int client_ip, time_t now, int32_t fail_window);
+  HostDBInfo *find_ip(sockaddr const* addr);
+  HostDBInfo *select_best(sockaddr const* client_ip, HostDBInfo * r = NULL);
+  HostDBInfo *select_best_http(sockaddr const* client_ip, ink_time_t now, int32_t fail_window);
 
   HostDBInfo *increment_round_robin()
   {
@@ -389,7 +388,7 @@ struct HostDBRoundRobin
   }
 
   HostDBRoundRobin()
-  : n(0), good(0), current(0)
+    : n(0), good(0), current(0), timed_rr_ctime(0)
   { }
 
 };
@@ -426,18 +425,25 @@ struct HostDBProcessor: public Processor
   HostDBProcessor()
   { }
 
-  inkcoreapi Action *getbyname_re(Continuation * cont, char *hostname, int len = 0, int port = 0,
+  inkcoreapi Action *getbyname_re(Continuation * cont, const char *hostname, int len = 0, int port = 0,
                                   int flags = HOSTDB_DO_NOT_FORCE_DNS);
 
-  Action *getSRVbyname_imm(Continuation * cont, process_srv_info_pfn process_srv_info, char *hostname, int len = 0,
+  Action *getSRVbyname_imm(Continuation * cont, process_srv_info_pfn process_srv_info, const char *hostname, int len = 0,
                            int port = 0, int flags = HOSTDB_DO_NOT_FORCE_DNS, int timeout = 0);
 
-  Action *getbyname_imm(Continuation * cont, process_hostdb_info_pfn process_hostdb_info, char *hostname, int len = 0,
-                        int port = 0, int flags = HOSTDB_DO_NOT_FORCE_DNS, int timeout = 0);
+  Action *getbyname_imm(
+    Continuation * cont,
+    process_hostdb_info_pfn process_hostdb_info,
+    const char *hostname,
+    int len = 0,
+    int port = 0,
+    int flags = HOSTDB_DO_NOT_FORCE_DNS,
+    int timeout = 0
+  );
 
 
   /** Lookup Hostinfo by addr */
-  Action *getbyaddr_re(Continuation * cont, unsigned int aip)
+  Action *getbyaddr_re(Continuation * cont, sockaddr const* aip)
   {
     return getby(cont, NULL, 0, 0, aip, false);
   }
@@ -450,22 +456,34 @@ struct HostDBProcessor: public Processor
     you will get a different IP address.
 
   */
-  Action *failed_connect_on_ip_for_name(Continuation * cont,
-                                        unsigned int aip, char *hostname, int len = 0, int port = 0);
+  Action *failed_connect_on_ip_for_name(
+    Continuation * cont,
+    sockaddr const* aip,
+    const char *hostname, int len = 0
+  );
 
   /** Set the application information (fire-and-forget). */
   void setbyname_appinfo(char *hostname, int len, int port, HostDBApplicationInfo * app)
   {
-    setby(hostname, len, port, 0, app);
+    sockaddr_in addr;
+    ats_ip4_set(&addr, INADDR_ANY, port);
+    setby(hostname, len, ats_ip_sa_cast(&addr), app);
   }
 
-  void setbyaddr_appinfo(unsigned int ip, HostDBApplicationInfo * app)
+  void setbyaddr_appinfo(sockaddr const* addr, HostDBApplicationInfo * app) {
+    this->setby(0, 0, addr, app);
+  }
+
+  void setbyaddr_appinfo(in_addr_t ip, HostDBApplicationInfo * app)
   {
-    setby(0, 0, 0, ip, app);
+    sockaddr_in addr;
+    ats_ip4_set(&addr, ip);
+    this->setby(0, 0, ats_ip_sa_cast(&addr), app);
   }
 
   /** Configuration. */
   static int hostdb_strict_round_robin;
+  static int hostdb_timed_round_robin;
 
   // Processor Interface
   /* hostdb does not use any dedicated event threads
@@ -475,9 +493,23 @@ struct HostDBProcessor: public Processor
 
   // Private
   HostDBCache *cache();
-  Action *getby(Continuation * cont, char *hostname, int len, int port,
-                unsigned int ip, bool aforce_dns, int timeout = 0);
-  void setby(char *hostname, int len, int port, unsigned int aip, HostDBApplicationInfo * app);
+  Action *getby(
+    Continuation * cont,
+    const char *hostname, int len,
+    sockaddr const* ip,
+    bool aforce_dns, int timeout = 0
+  );
+  /** Set something.
+      @a aip can carry address and / or port information. If setting just
+      by a port value, the address should be set to INADDR_ANY which is of
+      type IPv4.
+   */
+  void setby(
+    const char *hostname, ///< Hostname.
+    int len, ///< Length of hostname.
+    sockaddr const* aip, ///< Address and/or port.
+    HostDBApplicationInfo * app ///< I don't know.
+  );
 
 };
 
