@@ -65,8 +65,8 @@ public:
   inkcoreapi void append_block(IOBufferBlock * block);
   virtual void UDPPacket_is_abstract() = 0;
 
-  struct sockaddr_in from;    // what address came from
-  struct sockaddr_in to;      // what address to send to
+  IpEndpoint from;    // what address came from
+  IpEndpoint to;      // what address to send to
 
   int from_size;
 
@@ -83,7 +83,7 @@ public:
    @param buf if !NULL, then len bytes copied from buf and made into packet.
    @param len # of bytes to copy from buf
  */
-TS_INLINE UDPPacket *new_UDPPacket(struct sockaddr_in *to, ink_hrtime when = 0, char *buf = NULL, int len = 0);
+TS_INLINE UDPPacket *new_UDPPacket(struct sockaddr const* to, ink_hrtime when = 0, char *buf = NULL, int len = 0);
 /**
    Create a new packet to be sent over UDPConnection. This clones and
    makes a reference to an existing IOBufferBlock chain.
@@ -95,7 +95,7 @@ TS_INLINE UDPPacket *new_UDPPacket(struct sockaddr_in *to, ink_hrtime when = 0,
    for packet
    @param len # of bytes to reference from block
  */
-TS_INLINE UDPPacket *new_UDPPacket(struct sockaddr_in *to,
+TS_INLINE UDPPacket *new_UDPPacket(struct sockaddr const* to,
                                     ink_hrtime when = 0, IOBufferBlock * block = NULL, int len = 0);
 /**
    Create a new packet to be sent over UDPConnection.  Packet has no
@@ -107,7 +107,7 @@ TS_INLINE UDPPacket *new_UDPPacket();
    Create a new packet to be delivered to application.
    Internal function only
 */
-TS_INLINE UDPPacket *new_incoming_UDPPacket(struct sockaddr_in *from, char *buf, int len);
+TS_INLINE UDPPacket *new_incoming_UDPPacket(struct sockaddr* from, char *buf, int len);
 
 //@}
 #endif //__I_UDPPACKET_H_
