@@ -45,6 +45,61 @@ typedef const SSL_METHOD * ink_ssl_method_t;
 typedef SSL_METHOD * ink_ssl_method_t;
 #endif
 
+#if TS_USE_TLS_SNI
+
+static int
+ssl_servername_callback(SSL * ssl, int * ad, void * arg)
+{
+  SSL_CTX *       ctx = NULL;
+  SSLCertLookup * lookup = (SSLCertLookup *) arg;
+  const char *    servername = SSL_get_servername(ssl, TLSEXT_NAMETYPE_host_name);
+
+  Debug("ssl", "ssl=%p ad=%d lookup=%p server=%s", ssl, *ad, lookup, servername);
+
+  // The incoming SSL_CTX is either the one mapped from the inbound IP address or the default one. If we don't find a
+  // name-based match at this point, we *do not* want to mess with the context because we've already made a best effort
+  // to find the best match.
+  if (likely(servername)) {
+    ctx = lookup->findInfoInHash((char *)servername);
+  }
+
+  if (ctx != NULL) {
+    SSL_set_SSL_CTX(ssl, ctx);
+  }
+
+  ctx = SSL_get_SSL_CTX(ssl);
+  Debug("ssl", "found SSL context %p for requested name '%s'", ctx, servername);
+
+  if (ctx == NULL) {
+    return SSL_TLSEXT_ERR_NOACK;
+  }
+
+  // We need to return one of the SSL_TLSEXT_ERR constants. If we return an
+  // error, we can fill in *ad with an alert code to propgate to the
+  // client, see SSL_AD_*.
+  return SSL_TLSEXT_ERR_OK;
+}
+
+#endif /* TS_USE_TLS_SNI */
+
+static SSL_CTX *
+make_ssl_context(void * arg)
+{
+  SSL_CTX *         ctx = NULL;
+  ink_ssl_method_t  meth = NULL;
+
+  meth = SSLv23_server_method();
+  ctx = SSL_CTX_new(meth);
+
+#if TS_USE_TLS_SNI
+  Debug("ssl", "setting SNI callbacks with for ctx %p", ctx);
+  SSL_CTX_set_tlsext_servername_callback(ctx, ssl_servername_callback);
+  SSL_CTX_set_tlsext_servername_arg(ctx, arg);
+#endif /* TS_USE_TLS_SNI */
+
+  return ctx;
+}
+
 class SSLContextStorage
 {
 
@@ -52,7 +107,7 @@ class SSLContextStorage
   {
     explicit SslEntry(SSL_CTX * c) : ctx(c) {}
 
-    void Print() const { printf("%p/%p", this, ctx); }
+    void Print() const { Debug("ssl", "SslEntry=%p SSL_CTX=%p", this, ctx); }
 
     SSL_CTX * ctx;
     LINK(SslEntry, link);
@@ -112,13 +167,13 @@ void
 SSLCertLookup::init(SslConfigParams * p)
 {
   param = p;
-  multipleCerts = buildTable();
 
-  // If there wasn't a default SSL context, make a default one. We need this to bootstrap
-  // the SNI process and also to avoid crashing (which is generaly frowned upon).
-  if (!this->ssl_default) {
-    // XXX this leaks, but we're a singleton, so ....
-    this->ssl_default = SSL_CTX_new(SSLv23_server_method());
+  this->multipleCerts = buildTable();
+
+  // We *must* have a default context even if it can't possibly work. The default context is used to bootstrap the SSL
+  // handshake so that we can subsequently do the SNI lookup to switch to the real context.
+  if (this->ssl_default == NULL) {
+    this->ssl_default = make_ssl_context(this);
   }
 }
 
@@ -199,17 +254,6 @@ SSLCertLookup::buildTable()
     line = tokLine(NULL, &tok_state);
   }                             //  while(line != NULL)
 
-/*  if(num_el == 0)
-  {
-    Warning("%s No entries in %s. Using default server cert for all connections",
-	    moduleName, configFilePath);
-  }
-
-  if(is_debug_tag_set("ssl"))
-  {
-    Print();
-  }
-*/
   ats_free(file_buf);
   return ret;
 }
@@ -279,10 +323,9 @@ SSLCertLookup::addInfoToHash(
     const char *strAddr, const char *cert,
     const char *caCert, const char *serverPrivateKey)
 {
-  ink_ssl_method_t meth = NULL;
+  SSL_CTX * ctx;
 
-  meth = SSLv23_server_method();
-  SSL_CTX *ctx = SSL_CTX_new(meth);
+  ctx = make_ssl_context(this);
   if (!ctx) {
     SSLNetProcessor::logSSLError("Cannot create new server contex.");
     return (false);
@@ -500,9 +543,10 @@ SSLContextStorage::insert(SSL_CTX * ctx, const char * name)
       return false;
     }
 
-    Debug("indexed wildcard certificate for '%s' as '%s'", name, reversed);
-    this->wildcards.Insert(reversed, new SslEntry(ctx), 0 /* rank */, -1 /* keylen */);
+    Debug("ssl", "indexed wildcard certificate for '%s' as '%s' with SSL_CTX %p", name, reversed, ctx);
+    return this->wildcards.Insert(reversed, new SslEntry(ctx), 0 /* rank */, -1 /* keylen */);
   } else {
+    Debug("ssl", "indexed '%s' with SSL_CTX %p", name, ctx);
     ink_hash_table_insert(this->hostnames, name, (void *)ctx);
   }
 
@@ -529,6 +573,7 @@ SSLContextStorage::lookup(const char * name) const
       return NULL;
     }
 
+    Debug("ssl", "attempting wildcard match for %s", reversed);
     entry = this->wildcards.Search(reversed);
     if (entry) {
       return entry->ctx;
@@ -544,17 +589,18 @@ REGRESSION_TEST(SslHostLookup)(RegressionTest* t, int atype, int * pstatus)
 {
   TestBox           tb(t, pstatus);
   SSLContextStorage storage;
-  ink_ssl_method_t  methods = SSLv23_server_method();
 
-  SSL_CTX * wild = SSL_CTX_new(methods);
-  SSL_CTX * notwild = SSL_CTX_new(methods);
-  SSL_CTX * foo = SSL_CTX_new(methods);
+  SSL_CTX * wild = make_ssl_context(NULL);
+  SSL_CTX * notwild = make_ssl_context(NULL);
+  SSL_CTX * b_notwild = make_ssl_context(NULL);
+  SSL_CTX * foo = make_ssl_context(NULL);
 
   *pstatus = REGRESSION_TEST_PASSED;
 
   tb.check(storage.insert(foo, "www.foo.com"), "insert host context");
   tb.check(storage.insert(wild, "*.wild.com"), "insert wildcard context");
   tb.check(storage.insert(notwild, "*.notwild.com"), "insert wildcard context");
+  tb.check(storage.insert(b_notwild, "*.b.notwild.com"), "insert wildcard context");
 
   // Basic wildcard cases.
   tb.check(storage.lookup("a.wild.com") == wild, "wildcard lookup for a.wild.com");
@@ -564,6 +610,7 @@ REGRESSION_TEST(SslHostLookup)(RegressionTest* t, int atype, int * pstatus)
   // Varify that wildcard does longest match.
   tb.check(storage.lookup("a.notwild.com") == notwild, "wildcard lookup for a.notwild.com");
   tb.check(storage.lookup("notwild.com") == notwild, "wildcard lookup for notwild.com");
+  tb.check(storage.lookup("c.b.notwild.com") == b_notwild, "wildcard lookup for c.b.notwild.com");
 
   // Basic hostname cases.
   tb.check(storage.lookup("www.foo.com") == foo, "host lookup for www.foo.com");
