@@ -30,6 +30,7 @@
  ****************************************************************************/
 
 #include "P_Net.h"
+#include "P_UDPNet.h"
 
 typedef int (UDPNetHandler::*UDPNetContHandler) (int, void *);
 
@@ -66,7 +67,7 @@ void *G_bulkIOState = NULL;
 InkPipeInfo G_inkPipeInfo;
 
 int G_bwGrapherFd;
-struct sockaddr_in G_bwGrapherLoc;
+sockaddr_in6 G_bwGrapherLoc;
 
 void
 initialize_thread_for_udp_net(EThread * thread)
@@ -74,11 +75,6 @@ initialize_thread_for_udp_net(EThread * thread)
   new((ink_dummy_for_new *) get_UDPPollCont(thread)) PollCont(thread->mutex);
   new((ink_dummy_for_new *) get_UDPNetHandler(thread)) UDPNetHandler;
 
-#if TS_USE_LIBEV
-  PollCont *pc = get_UDPPollCont(thread);
-  PollDescriptor *pd = pc->pollDescriptor;
-  pd->eio = ev_loop_new(LIBEV_BACKEND_LIST);
-#endif
   // These are hidden variables that control the amount of memory used by UDP
   // packets.  As usual, defaults are in RecordsConfig.cc
 
@@ -132,7 +128,7 @@ UDPNetProcessorInternal::udp_read_from_net(UDPNetHandler * nh,
   int r;
   int iters = 0;
   do {
-    struct sockaddr_in fromaddr;
+    sockaddr_in6 fromaddr;
     socklen_t fromlen = sizeof(fromaddr);
     // XXX: want to be 0 copy.
     // XXX: really should read into next contiguous region of an IOBufferData
@@ -145,7 +141,7 @@ UDPNetProcessorInternal::udp_read_from_net(UDPNetHandler * nh,
       break;
     }
     // create packet
-    UDPPacket *p = new_incoming_UDPPacket(&fromaddr, buf, r);
+    UDPPacket *p = new_incoming_UDPPacket(ats_ip_sa_cast(&fromaddr), buf, r);
     p->setConnection(uc);
     // XXX: is this expensive?  I] really want to know this information
     p->setArrivalTime(ink_get_hrtime_internal());
@@ -219,7 +215,7 @@ private:
   // on behalf of the client
   Ptr<IOBufferBlock> readbuf;
   int readlen;
-  struct sockaddr *fromaddr;
+  struct sockaddr_in6 *fromaddr;
   socklen_t *fromaddrlen;
   int fd;                       // fd we are reading from
   int ifd;                      // poll fd index
@@ -230,6 +226,8 @@ private:
 
 ClassAllocator<UDPReadContinuation> udpReadContAllocator("udpReadContAllocator");
 
+#define UNINITIALIZED_EVENT_PTR (Event *)0xdeadbeef
+
 UDPReadContinuation::UDPReadContinuation(Event * completionToken)
 : Continuation(NULL), event(completionToken), readbuf(NULL),
   readlen(0), fromaddrlen(0), fd(-1), ifd(-1), period(0), elapsed_time(0), timeout_interval(0)
@@ -241,7 +239,7 @@ UDPReadContinuation::UDPReadContinuation(Event * completionToken)
 }
 
 UDPReadContinuation::UDPReadContinuation()
-: Continuation(NULL), event(0), readbuf(NULL),
+: Continuation(NULL), event(UNINITIALIZED_EVENT_PTR), readbuf(NULL),
   readlen(0), fromaddrlen(0), fd(-1), ifd(-1), period(0), elapsed_time(0), timeout_interval(0)
 {
 }
@@ -282,7 +280,7 @@ UDPReadContinuation::init_read(int rfd, IOBufferBlock * buf, int len, struct soc
   fd = rfd;
   readbuf = buf;
   readlen = len;
-  fromaddr = fromaddr_;
+  fromaddr = ats_ip6_cast(fromaddr_);
   fromaddrlen = fromaddrlen_;
   SET_HANDLER(&UDPReadContinuation::readPollEvent);
   period = NET_PERIOD;
@@ -292,9 +290,12 @@ UDPReadContinuation::init_read(int rfd, IOBufferBlock * buf, int len, struct soc
 
 UDPReadContinuation::~UDPReadContinuation()
 {
-  ink_assert(event != NULL);
-  completionUtil::destroy(event);
-  event = NULL;
+  if (event != UNINITIALIZED_EVENT_PTR)
+  {
+    ink_assert(event != NULL);
+    completionUtil::destroy(event);
+    event = NULL;
+  }
 }
 
 void
@@ -356,7 +357,7 @@ UDPReadContinuation::readPollEvent(int event_, Event * e)
     socklen_t tmp_fromlen = *fromaddrlen;
     int rlen = socketManager.recvfrom(fd, readbuf->end(), readlen,
                                       0,        // default flags
-                                      fromaddr, &tmp_fromlen);
+                                      ats_ip_sa_cast(fromaddr), &tmp_fromlen);
     completionUtil::setThread(event, e->ethread);
     // call back user with their event
     if (rlen > 0) {
@@ -496,7 +497,7 @@ UDPNetProcessor::sendmsg_re(Continuation * cont, void *token, int fd, struct msg
  */
 Action *
 UDPNetProcessor::sendto_re(Continuation * cont,
-                           void *token, int fd, struct sockaddr * toaddr, int toaddrlen, IOBufferBlock * buf, int len)
+                           void *token, int fd, struct sockaddr const* toaddr, int toaddrlen, IOBufferBlock * buf, int len)
 {
   (void) token;
   ink_assert(buf->read_avail() >= len);
@@ -514,43 +515,45 @@ UDPNetProcessor::sendto_re(Continuation * cont,
 }
 
 Action *
-UDPNetProcessor::UDPCreatePortPairs(Continuation * cont,
-                                    int nPairs,
-                                    unsigned int myIP, unsigned int destIP, int send_bufsize, int recv_bufsize)
-{
+UDPNetProcessor::UDPCreatePortPairs(
+  Continuation * cont,
+  int nPairs,
+  sockaddr const* local_addr,
+  sockaddr const* remote_addr,
+  int send_bufsize, int recv_bufsize
+) {
 
   UDPWorkContinuation *worker = udpWorkContinuationAllocator.alloc();
   // UDPWorkContinuation *worker = NEW(new UDPWorkContinuation);
 
-  worker->init(cont, nPairs, myIP, destIP, send_bufsize, recv_bufsize);
+  worker->init(cont, nPairs, local_addr, remote_addr, send_bufsize, recv_bufsize);
   eventProcessor.schedule_imm(worker, ET_UDP);
   return &(worker->action);
 }
 
 
 bool
-UDPNetProcessor::CreateUDPSocket(int *resfd,
-                                 struct sockaddr_in * addr,
-                                 Action ** status, int my_port, unsigned int my_ip, int send_bufsize, int recv_bufsize)
-{
+UDPNetProcessor::CreateUDPSocket(
+  int *resfd,
+  sockaddr const* remote_addr,
+  sockaddr* local_addr,
+  int *local_addr_len,
+  Action ** status, 
+  int send_bufsize, int recv_bufsize
+) {
   int res = 0, fd = -1;
-  struct sockaddr_in bind_sa;
-  struct sockaddr_in myaddr;
-  int myaddr_len = sizeof(myaddr);
+
+  ink_assert(ats_ip_are_compatible(remote_addr, local_addr));
 
   *resfd = -1;
-  if ((res = socketManager.socket(AF_INET, SOCK_DGRAM, 0)) < 0)
+  if ((res = socketManager.socket(remote_addr->sa_family, SOCK_DGRAM, 0)) < 0)
     goto HardError;
   fd = res;
   if ((res = safe_fcntl(fd, F_SETFL, O_NONBLOCK)) < 0)
     goto HardError;
-  memset(&bind_sa, 0, sizeof(bind_sa));
-  bind_sa.sin_family = AF_INET;
-  bind_sa.sin_port = htons(my_port);
-  bind_sa.sin_addr.s_addr = my_ip;
-  if ((res = socketManager.ink_bind(fd, (struct sockaddr *) &bind_sa, sizeof(bind_sa), IPPROTO_UDP)) < 0) {
-    unsigned char *pt = (unsigned char *) &my_ip;
-    Debug("udpnet", "ink bind failed --- my_ip = %d.%d.%d.%d", pt[0], pt[1], pt[2], pt[3]);
+  if ((res = socketManager.ink_bind(fd, remote_addr, ats_ip_size(remote_addr), IPPROTO_UDP)) < 0) {
+    char buff[INET6_ADDRPORTSTRLEN];
+    Debug("udpnet", "ink bind failed on %s", ats_ip_nptop(remote_addr, buff, sizeof(buff)));
     goto SoftError;
   }
 
@@ -562,34 +565,29 @@ UDPNetProcessor::CreateUDPSocket(int *resfd,
     if (unlikely(socketManager.set_sndbuf_size(fd, send_bufsize)))
       Debug("udpnet", "set_dnsbuf_size(%d) failed", send_bufsize);
   }
-  if ((res = safe_getsockname(fd, (struct sockaddr *) &myaddr, &myaddr_len)) < 0) {
+  if ((res = safe_getsockname(fd, local_addr, local_addr_len)) < 0) {
     Debug("udpnet", "CreateUdpsocket: getsockname didnt' work");
     goto HardError;
   }
-  if (!myaddr.sin_addr.s_addr) {
-    // set to default IP address for machine
-    /** netfixme ... this_machine() is in proxy.
-    if (this_machine()) {
-      myaddr.sin_addr.s_addr = this_machine()->ip;
-    } else {
-      Debug("udpnet","CreateUdpSocket -- machine not initialized");
-    }
-    */
-  }
   *resfd = fd;
-  memcpy(addr, &myaddr, myaddr_len);
   *status = NULL;
-  Debug("udpnet", "creating a udp socket port = %d, %d---success", my_port, addr->sin_port);
+  Debug("udpnet", "creating a udp socket port = %d, %d---success",
+    ats_ip_port_host_order(remote_addr), ats_ip_port_host_order(local_addr)
+  );
   return true;
 SoftError:
-  Debug("udpnet", "creating a udp socket port = %d---soft failure", my_port);
+  Debug("udpnet", "creating a udp socket port = %d---soft failure",
+    ats_ip_port_host_order(local_addr)
+  );
   if (fd != -1)
     socketManager.close(fd);
   *resfd = -1;
   *status = NULL;
   return false;
 HardError:
-  Debug("udpnet", "creating a udp socket port = %d---hard failure", my_port);
+  Debug("udpnet", "creating a udp socket port = %d---hard failure",
+    ats_ip_port_host_order(local_addr)
+  );
   if (fd != -1)
     socketManager.close(fd);
   *resfd = -1;
@@ -598,10 +596,12 @@ HardError:
 }
 
 void
-UDPNetProcessor::UDPClassifyConnection(Continuation * udpConn, int destIP)
-{
+UDPNetProcessor::UDPClassifyConnection(
+  Continuation * udpConn,
+  IpAddr const& destIP
+) {
   int i;
-  UDPConnectionInternal *p = (UDPConnectionInternal *) udpConn;
+  UDPConnectionInternal *p = static_cast<UDPConnectionInternal *>(udpConn);
 
   if (G_inkPipeInfo.numPipes == 0) {
     p->pipe_class = 0;
@@ -612,10 +612,11 @@ UDPNetProcessor::UDPClassifyConnection(Continuation * udpConn, int destIP)
   for (i = 0; i < G_inkPipeInfo.numPipes + 1; i++)
     if (G_inkPipeInfo.perPipeInfo[i].destIP == destIP)
       p->pipe_class = i;
-  // no match; set it to the destIP=0 class
+  // no match; set it to the null class
   if (p->pipe_class == -1) {
-    for (i = 0; i < G_inkPipeInfo.numPipes + 1; i++)
-      if (G_inkPipeInfo.perPipeInfo[i].destIP == 0) {
+    IpAddr null; // default constructed -> invalid value.
+    for (i = 0; i < G_inkPipeInfo.numPipes + 1; ++i)
+      if (G_inkPipeInfo.perPipeInfo[i].destIP == null) {
         p->pipe_class = i;
         break;
       }
@@ -628,23 +629,22 @@ UDPNetProcessor::UDPClassifyConnection(Continuation * udpConn, int destIP)
 }
 
 Action *
-UDPNetProcessor::UDPBind(Continuation * cont, int my_port, int my_ip, int send_bufsize, int recv_bufsize)
+UDPNetProcessor::UDPBind(Continuation * cont, sockaddr const* addr, int send_bufsize, int recv_bufsize)
 {
   int res = 0;
   int fd = -1;
   UnixUDPConnection *n = NULL;
-  struct sockaddr_in bind_sa;
-  struct sockaddr_in myaddr;
+  IpEndpoint myaddr;
   int myaddr_len = sizeof(myaddr);
 
-  if ((res = socketManager.socket(AF_INET, SOCK_DGRAM, 0)) < 0)
+  if ((res = socketManager.socket(addr->sa_family, SOCK_DGRAM, 0)) < 0)
     goto Lerror;
   fd = res;
   if ((res = fcntl(fd, F_SETFL, O_NONBLOCK) < 0))
     goto Lerror;
 
   // If this is a class D address (i.e. multicast address), use REUSEADDR.
-  if ((((unsigned int) (ntohl(my_ip))) & 0xf0000000) == 0xe0000000) {
+  if (ats_is_ip_multicast(addr)) {
     int enable_reuseaddr = 1;
 
     if ((res = safe_setsockopt(fd, SOL_SOCKET, SO_REUSEADDR, (char *) &enable_reuseaddr, sizeof(enable_reuseaddr)) < 0)) {
@@ -652,11 +652,7 @@ UDPNetProcessor::UDPBind(Continuation * cont, int my_port, int my_ip, int send_b
     }
   }
 
-  memset(&bind_sa, 0, sizeof(bind_sa));
-  bind_sa.sin_family = AF_INET;
-  bind_sa.sin_port = htons(my_port);
-  bind_sa.sin_addr.s_addr = my_ip;
-  if ((res = socketManager.ink_bind(fd, (struct sockaddr *) &bind_sa, sizeof(bind_sa))) < 0) {
+  if ((res = socketManager.ink_bind(fd, addr, ats_ip_size(addr))) < 0) {
     goto Lerror;
   }
 
@@ -668,23 +664,13 @@ UDPNetProcessor::UDPBind(Continuation * cont, int my_port, int my_ip, int send_b
     if (unlikely(socketManager.set_sndbuf_size(fd, send_bufsize)))
       Debug("udpnet", "set_dnsbuf_size(%d) failed", send_bufsize);
   }
-  if ((res = safe_getsockname(fd, (struct sockaddr *) &myaddr, &myaddr_len)) < 0) {
+  if ((res = safe_getsockname(fd, &myaddr.sa, &myaddr_len)) < 0) {
     goto Lerror;
   }
-  if (!myaddr.sin_addr.s_addr) {
-    // set to default IP address for machine
-    /** netfixme this_machine is in proxy/
-    if (this_machine()) {
-      myaddr.sin_addr.s_addr = this_machine()->ip;
-    } else {
-      Debug("udpnet","UDPNetProcessor::UDPBind -- machine not initialized");
-    }
-    */
-  }
   n = NEW(new UnixUDPConnection(fd));
 
-  Debug("udpnet", "UDPNetProcessor::UDPBind: %x fd=%d", n, fd);
-  n->setBinding(&myaddr);
+  Debug("udpnet", "UDPNetProcessor::UDPBind: %p fd=%d", n, fd);
+  n->setBinding(&myaddr.sa);
   n->bindToThread(cont);
 
   cont->handleEvent(NET_EVENT_DATAGRAM_OPEN, n);
@@ -859,7 +845,7 @@ UDPQueue::service(UDPNetHandler * nh)
         continue;
       }
       // insert into our queue.
-      Debug("udp-send", "Adding 0x%x", p);
+      Debug("udp-send", "Adding %p", p);
       addToGuaranteedQ = ((p->conn->pipe_class > 0) && (p->conn->flowRateBps > 10.0));
       pktLen = p->getPktLength();
       if (p->conn->lastPktStartTime == 0) {
@@ -1062,8 +1048,8 @@ sendPackets:
       G_inkPipeInfo.perPipeInfo[i].queue->FreeCancelledPackets(g_udp_periodicCleanupSlots);
     }
     endTime = ink_get_hrtime_internal();
-    Debug("udp-pending-packets", "Did cleanup of %d buckets: %" PRId64 " bytes in %d m.sec",
-          g_udp_periodicCleanupSlots, nbytes - g_udp_bytesPending, ink_hrtime_to_msec(endTime - startTime));
+    Debug("udp-pending-packets", "Did cleanup of %d buckets: %" PRId64 " bytes in %" PRId64 " m.sec",
+          g_udp_periodicCleanupSlots, nbytes - g_udp_bytesPending, (int64_t)ink_hrtime_to_msec(endTime - startTime));
     lastCleanupTime = now;
   }
 }
@@ -1081,7 +1067,7 @@ UDPQueue::SendUDPPacket(UDPPacketInternal * p, int32_t pktLen)
     p->conn->lastSentPktStartTime = p->delivery_time;
   }
 
-  Debug("udp-send", "Sending 0x%x", p);
+  Debug("udp-send", "Sending %p", p);
 #if !defined(solaris)
   msg.msg_control = 0;
   msg.msg_controllen = 0;
@@ -1261,14 +1247,16 @@ UDPNetHandler::mainNetEvent(int event, Event * e)
 
 void
 UDPWorkContinuation::init(Continuation * c, int num_pairs,
-                          unsigned int my_ip, unsigned int dest_ip, int s_bufsize, int r_bufsize)
-{
+  sockaddr const* local_addr,
+  sockaddr const* remote_addr,
+  int s_bufsize, int r_bufsize
+) {
   mutex = c->mutex;
   cont = c;
   action = c;
   numPairs = num_pairs;
-  myIP = my_ip;
-  destIP = dest_ip;
+  ats_ip_copy(&local_ip, local_addr);
+  ats_ip_copy(&remote_ip, remote_addr);
   sendbufsize = s_bufsize;
   recvbufsize = r_bufsize;
   udpConns = NULL;
@@ -1283,8 +1271,8 @@ UDPWorkContinuation::StateCreatePortPairs(int event, void *data)
 //  int res = 0;
   int numUdpPorts = 2 * numPairs;
   int fd1 = -1, fd2 = -1;
-//  struct sockaddr_in bind_sa;
-  struct sockaddr_in myaddr1, myaddr2;
+  IpEndpoint target;
+  IpEndpoint myaddr1, myaddr2;
   int portNum, i;
 //  int myaddr_len = sizeof(myaddr1);
   static int lastAllocPort = 10000;
@@ -1304,6 +1292,7 @@ UDPWorkContinuation::StateCreatePortPairs(int event, void *data)
   }
 
   startTime = ink_get_hrtime_internal();
+  ats_ip_copy(&target, &remote_ip);
 
   udpConns = NEW(new UnixUDPConnection *[numUdpPorts]);
   for (i = 0; i < numUdpPorts; i++)
@@ -1316,13 +1305,25 @@ UDPWorkContinuation::StateCreatePortPairs(int event, void *data)
   i = 0;
   while (i < numUdpPorts) {
 
-    if (udpNet.CreateUDPSocket(&fd1, &myaddr1, &status, portNum, myIP, sendbufsize, recvbufsize)) {
-      if (udpNet.CreateUDPSocket(&fd2, &myaddr2, &status, portNum + 1, myIP, sendbufsize, recvbufsize)) {
+    int myaddr1_len = sizeof(myaddr1);
+    int myaddr2_len = sizeof(myaddr2);
+    ats_ip_port_cast(&target) = htons(portNum);
+    if (udpNet.CreateUDPSocket(&fd1, 
+        &target.sa, 
+        &myaddr1.sa,
+        &myaddr1_len,
+        &status, sendbufsize, recvbufsize)) {
+      ats_ip_port_cast(&target) = htons(portNum + 1);
+      if (udpNet.CreateUDPSocket(&fd2, 
+        &target.sa, 
+        &myaddr2.sa,
+        &myaddr2_len,
+        &status, sendbufsize, recvbufsize)) {
         udpConns[i] = NEW(new UnixUDPConnection(fd1));        // new_UnixUDPConnection(fd1);
-        udpConns[i]->setBinding(&myaddr1);
+        udpConns[i]->setBinding(&myaddr1.sa);
         i++;
         udpConns[i] = NEW(new UnixUDPConnection(fd2));        // new_UnixUDPConnection(fd2);
-        udpConns[i]->setBinding(&myaddr2);
+        udpConns[i]->setBinding(&myaddr2.sa);
         i++;
         // remember the last alloc'ed port
         ink_atomic_swap(&lastAllocPort, portNum + 2);
@@ -1351,7 +1352,7 @@ UDPWorkContinuation::StateCreatePortPairs(int event, void *data)
   }
 
   for (i = 0; i < numUdpPorts; i++) {
-    udpNet.UDPClassifyConnection(udpConns[i], destIP);
+    udpNet.UDPClassifyConnection(udpConns[i], IpAddr(target));
     Debug("udpnet-pipe", "Adding (port = %d) to Pipe class: %d",
           udpConns[i]->getPortNum(), udpConns[i]->pipe_class);
   }
