@@ -22,9 +22,7 @@
  */
 
 #include "P_Net.h"
-#ifndef _WIN32
 #include <netdb.h>
-#endif
 
 Diags *diags;
 #define DIAGS_LOG_FILE "diags.log"
@@ -83,7 +81,7 @@ reconfigure_diags()
   ////////////////////////////////////
   // change the diags config values //
   ////////////////////////////////////
-#if !defined (_WIN32) && !defined(__GNUC__) && !defined(hpux)
+#if !defined(__GNUC__) && !defined(hpux)
   diags->config = c;
 #else
   memcpy(((void *) &diags->config), ((void *) &c), sizeof(DiagsConfigState));
@@ -98,7 +96,7 @@ init_diags(char *bdt, char *bat)
 {
   FILE *diags_log_fp;
   char diags_logpath[500];
-  strcpy(diags_logpath, DIAGS_LOG_FILE);
+  ink_strlcpy(diags_logpath, DIAGS_LOG_FILE, sizeof(diags_logpath));
 
   diags_log_fp = fopen(diags_logpath, "w");
   if (diags_log_fp) {
@@ -150,9 +148,7 @@ main()
   /*
    * ignore broken pipe
    */
-#ifndef _WIN32
   signal(SIGPIPE, SIG_IGN);
-#endif
 
   /*
    * start processors
