@@ -85,7 +85,7 @@ reconfigure_diags()
   ////////////////////////////////////
   // change the diags config values //
   ////////////////////////////////////
-#if !defined (_IOCORE_WIN32) && !defined(__GNUC__) && !defined(hpux)
+#if !defined(__GNUC__) && !defined(hpux)
   diags->config = c;
 #else
   memcpy(((void *) &diags->config), ((void *) &c), sizeof(DiagsConfigState));
@@ -100,7 +100,7 @@ init_diags(char *bdt, char *bat)
 {
   FILE *diags_log_fp;
   char diags_logpath[500];
-  strcpy(diags_logpath, DIAGS_LOG_FILE);
+  ink_strlcpy(diags_logpath, DIAGS_LOG_FILE, sizeof(diags_logpath));
 
   diags_log_fp = fopen(diags_logpath, "a+");
   if (diags_log_fp) {
