@@ -33,16 +33,16 @@ Process arguments
 //  Global variables
 //
 
-char *file_arguments[MAX_FILE_ARGUMENTS] = { 0 };
-char *program_name = (char *) "Traffic Server";
-int n_file_arguments = 0;
+const char *file_arguments[MAX_FILE_ARGUMENTS] = { 0 };
+const char *program_name = (char *) "Traffic Server";
+unsigned n_file_arguments = 0;
 
 //
 //  Local variables
 //
 
-static char *argument_types_keys = (char *) "ISDfFTL";
-static char *argument_types_descriptions[] = {
+static const char *argument_types_keys = (char *) "ISDfFTL";
+static const char *argument_types_descriptions[] = {
   (char *) "int  ",
   (char *) "str  ",
   (char *) "dbl  ",
@@ -58,8 +58,8 @@ static char *argument_types_descriptions[] = {
 //
 
 static void
-process_arg(ArgumentDescription * argument_descriptions,
-            int n_argument_descriptions, int i, char ***argv, const char *usage_string)
+process_arg(const ArgumentDescription * argument_descriptions,
+            unsigned n_argument_descriptions, int i, char ***argv, const char *usage_string)
 {
   char *arg = NULL;
   if (argument_descriptions[i].type) {
@@ -83,7 +83,12 @@ process_arg(ArgumentDescription * argument_descriptions,
         *(int64_t *) argument_descriptions[i].location = ink_atoi64(arg);
         break;
       case 'S':
-        ink_strlcpy((char *) argument_descriptions[i].location, arg, atoi(argument_descriptions[i].type + 1));
+        if (argument_descriptions[i].type[1] == '*') {
+          char ** out = (char **)argument_descriptions[i].location;
+          *out = ats_strdup(arg);
+        } else {
+          ink_strlcpy((char *) argument_descriptions[i].location, arg, atoi(argument_descriptions[i].type + 1));
+        }
         break;
       default:
         ink_fatal(1, (char *) "bad argument description");
@@ -98,11 +103,10 @@ process_arg(ArgumentDescription * argument_descriptions,
 
 
 void
-show_argument_configuration(ArgumentDescription * argument_descriptions, int n_argument_descriptions)
+show_argument_configuration(const ArgumentDescription * argument_descriptions, unsigned n_argument_descriptions)
 {
-  int i = 0;
   printf("Argument Configuration\n");
-  for (i = 0; i < n_argument_descriptions; i++) {
+  for (unsigned i = 0; i < n_argument_descriptions; i++) {
     if (argument_descriptions[i].type) {
       printf("  %-34s ", argument_descriptions[i].description);
       switch (argument_descriptions[i].type[0]) {
@@ -133,9 +137,9 @@ show_argument_configuration(ArgumentDescription * argument_descriptions, int n_a
 }
 
 void
-process_args(ArgumentDescription * argument_descriptions, int n_argument_descriptions, char **argv, const char *usage_string)
+process_args(const ArgumentDescription * argument_descriptions, unsigned n_argument_descriptions, char **argv, const char *usage_string)
 {
-  int i = 0;
+  unsigned i = 0;
   //
   // Grab Environment Variables
   //
@@ -197,7 +201,7 @@ process_args(ArgumentDescription * argument_descriptions, int n_argument_descrip
 }
 
 void
-usage(ArgumentDescription * argument_descriptions, int n_argument_descriptions, const char *usage_string)
+usage(const ArgumentDescription * argument_descriptions, unsigned n_argument_descriptions, const char *usage_string)
 {
   (void) argument_descriptions;
   (void) n_argument_descriptions;
@@ -207,7 +211,7 @@ usage(ArgumentDescription * argument_descriptions, int n_argument_descriptions,
   else
     fprintf(stderr, "Usage: %s [--SWITCH [ARG]]\n", program_name);
   fprintf(stderr, "  switch__________________type__default___description\n");
-  for (int i = 0; i < n_argument_descriptions; i++) {
+  for (unsigned i = 0; i < n_argument_descriptions; i++) {
     if (!argument_descriptions[i].description)
       continue;
     fprintf(stderr, "  -%c, --%-17s %s",
