@@ -43,15 +43,15 @@ extern "C" {
 #endif                          /* __cplusplus */
 
 inkcoreapi void ink_fatal_va(int return_code, const char *message_format, va_list ap);
-void ink_fatal(int return_code, const char *message_format, ...) PRINTFLIKE(2, 3);
-void ink_pfatal(int return_code, const char *message_format, ...) PRINTFLIKE(2, 3);
-void ink_warning(const char *message_format, ...) PRINTFLIKE(1, 2);
-void ink_pwarning(const char *message_format, ...) PRINTFLIKE(1, 2);
-void ink_notice(const char *message_format, ...) PRINTFLIKE(1, 2);
-void ink_eprintf(const char *message_format, ...) PRINTFLIKE(1, 2);
-void ink_error(const char *message_format, ...) PRINTFLIKE(1, 2);
-void ink_dprintf(int debug_level, const char *message_format, ...) PRINTFLIKE(2, 3);
-void ink_fatal_die(const char *message_format, ...) PRINTFLIKE(1, 2);
+void ink_fatal(int return_code, const char *message_format, ...) TS_PRINTFLIKE(2, 3);
+void ink_pfatal(int return_code, const char *message_format, ...) TS_PRINTFLIKE(2, 3);
+void ink_warning(const char *message_format, ...) TS_PRINTFLIKE(1, 2);
+void ink_pwarning(const char *message_format, ...) TS_PRINTFLIKE(1, 2);
+void ink_notice(const char *message_format, ...) TS_PRINTFLIKE(1, 2);
+void ink_eprintf(const char *message_format, ...) TS_PRINTFLIKE(1, 2);
+void ink_error(const char *message_format, ...) TS_PRINTFLIKE(1, 2);
+void ink_dprintf(int debug_level, const char *message_format, ...) TS_PRINTFLIKE(2, 3);
+void ink_fatal_die(const char *message_format, ...) TS_PRINTFLIKE(1, 2);
 
 void ink_die_die_die(int retval);
 void ink_segv();
