@@ -40,7 +40,7 @@
 
 int
 ink_bsprintf(char *buffer, const char *format, ...)
-PRINTFLIKE(2, 3);
+TS_PRINTFLIKE(2, 3);
 int
 ink_bvsprintf(char *buffer, const char *format, va_list ap);
 
