@@ -57,6 +57,7 @@
 #if TS_USE_POSIX_CAP
 #include <sys/capability.h>
 #endif
+#include <grp.h>
 
 #define FD_THROTTLE_HEADROOM (128 + 64) // TODO: consolidate with THROTTLE_FD_HEADROOM
 
@@ -1236,6 +1237,11 @@ runAsUser(char *userName)
       _exit(1);
     }
 
+    // setup supplementary groups if it is not set.
+    if (0 == getgroups(0, NULL)) {
+      initgroups(&userName[0],result->pw_gid);
+    }
+
 #if TS_USE_POSIX_CAP
     if (0 != restoreCapabilities()) {
       mgmt_elog(stderr, "[runAsUser] Error: Failed to restore capabilities after switch to user %s.\n", userName);
