@@ -52,41 +52,25 @@ void init_pdss_format(TSPdSsFormat * info);
 void *
 _TSmalloc(unsigned int size, const char *path)
 {
-#ifdef PURIFY
-  return xmalloc(size);
-#else
-  return _xmalloc(size, path);
-#endif
+  return ats_malloc(size);
 }
 
 void *
 _TSrealloc(void *ptr, unsigned int size, const char *path)
 {
-#ifdef PURIFY
-  return xrealloc(ptr, size);
-#else
-  return _xrealloc(ptr, size, path);
-#endif
+  return ats_realloc(ptr, size);
 }
 
 char *
 _TSstrdup(const char *str, int length, const char *path)
 {
-#ifdef PURIFY
-  return xstrndup(str, length);
-#else
-  return _xstrdup(str, length, path);
-#endif
+  return ats_strndup(str, length);
 }
 
 void
 _TSfree(void *ptr)
 {
-#ifdef PURIFY
-  xfree(ptr);
-#else
-  _xfree(ptr);
-#endif
+  ats_free(ptr);
 }
 
 
@@ -515,11 +499,7 @@ TSStringListDestroy(TSStringList strl)
   /* dequeue each element and free it */
   while (!queue_is_empty((LLQ *) strl)) {
     str = (char *) dequeue((LLQ *) strl);
-
-    if (!str)
-      continue;
-
-    xfree(str);
+    ats_free(str);
   }
 
   delete_queue((LLQ *) strl);
@@ -611,11 +591,7 @@ TSIntListDestroy(TSIntList intl)
   /* dequeue each element and free it */
   while (!queue_is_empty((LLQ *) intl)) {
     iPtr = (int *) dequeue((LLQ *) intl);
-
-    if (!iPtr)
-      continue;
-
-    xfree(iPtr);
+    ats_free(iPtr);
   }
 
   delete_queue((LLQ *) intl);
@@ -712,10 +688,7 @@ init_pdss_format(TSPdSsFormat * info)
 tsapi TSEvent *
 TSEventCreate(void)
 {
-  TSEvent *event;
-  event = (TSEvent *) xmalloc(sizeof(TSEvent));
-  if (!event)
-    return NULL;
+  TSEvent *event = (TSEvent *)ats_malloc(sizeof(TSEvent));
 
   event->id = -1;
   event->name = NULL;
@@ -729,11 +702,9 @@ tsapi void
 TSEventDestroy(TSEvent * event)
 {
   if (event) {
-    if (event->name)
-      xfree(event->name);
-    if (event->description)
-      xfree(event->description);
-    xfree(event);
+    ats_free(event->name);
+    ats_free(event->description);
+    ats_free(event);
   }
   return;
 }
@@ -741,11 +712,7 @@ TSEventDestroy(TSEvent * event)
 tsapi TSRecordEle *
 TSRecordEleCreate(void)
 {
-  TSRecordEle *ele;
-
-  ele = (TSRecordEle *) xmalloc(sizeof(TSRecordEle));
-  if (!ele)
-    return NULL;
+  TSRecordEle *ele = (TSRecordEle *)ats_malloc(sizeof(TSRecordEle));
 
   ele->rec_name = NULL;
   ele->rec_type = TS_REC_UNDEFINED;
@@ -757,11 +724,10 @@ tsapi void
 TSRecordEleDestroy(TSRecordEle * ele)
 {
   if (ele) {
-    if (ele->rec_name)
-      xfree(ele->rec_name);
+    ats_free(ele->rec_name);
     if (ele->rec_type == TS_REC_STRING && ele->string_val)
-      xfree(ele->string_val);
-    xfree(ele);
+      ats_free(ele->string_val);
+    ats_free(ele);
   }
   return;
 }
@@ -769,11 +735,7 @@ TSRecordEleDestroy(TSRecordEle * ele)
 tsapi TSIpAddrEle *
 TSIpAddrEleCreate(void)
 {
-  TSIpAddrEle *ele;
-
-  ele = (TSIpAddrEle *) xmalloc(sizeof(TSIpAddrEle));
-  if (!ele)
-    return NULL;
+  TSIpAddrEle *ele = (TSIpAddrEle *)ats_malloc(sizeof(TSIpAddrEle));
 
   /* set default values */
   ele->type = TS_IP_UNDEFINED;
@@ -790,11 +752,9 @@ tsapi void
 TSIpAddrEleDestroy(TSIpAddrEle * ele)
 {
   if (ele) {
-    if (ele->ip_a)
-      xfree(ele->ip_a);
-    if (ele->ip_b)
-      xfree(ele->ip_b);
-    xfree(ele);
+    ats_free(ele->ip_a);
+    ats_free(ele->ip_b);
+    ats_free(ele);
   }
 
   return;
@@ -803,11 +763,7 @@ TSIpAddrEleDestroy(TSIpAddrEle * ele)
 tsapi TSPortEle *
 TSPortEleCreate(void)
 {
-  TSPortEle *ele;
-
-  ele = (TSPortEle *) xmalloc(sizeof(TSPortEle));
-  if (!ele)
-    return NULL;
+  TSPortEle *ele = (TSPortEle *)ats_malloc(sizeof(TSPortEle));
 
   ele->port_a = TS_INVALID_PORT;
   ele->port_b = TS_INVALID_PORT;
@@ -818,19 +774,14 @@ TSPortEleCreate(void)
 tsapi void
 TSPortEleDestroy(TSPortEle * ele)
 {
-  if (ele)
-    xfree(ele);
+  ats_free(ele);
   return;
 }
 
 tsapi TSDomain *
 TSDomainCreate()
 {
-  TSDomain *ele;
-
-  ele = (TSDomain *) xmalloc(sizeof(TSDomain));
-  if (!ele)
-    return NULL;
+  TSDomain *ele = (TSDomain *)ats_malloc(sizeof(TSDomain));
 
   ele->domain_val = NULL;
   ele->port = TS_INVALID_PORT;
@@ -842,21 +793,15 @@ tsapi void
 TSDomainDestroy(TSDomain * ele)
 {
   if (ele) {
-    // this is okay because TSIpAddr is also a char*
-    if (ele->domain_val)
-      xfree(ele->domain_val);
-    xfree(ele);
+    ats_free(ele->domain_val);
+    ats_free(ele);
   }
 }
 
 tsapi TSSspec *
 TSSspecCreate(void)
 {
-  TSSspec *sec_spec;
-
-  sec_spec = (TSSspec *) xmalloc(sizeof(TSSspec));
-  if (!sec_spec)
-    return NULL;
+  TSSspec *sec_spec = (TSSspec *)ats_malloc(sizeof(TSSspec));
 
   /* set defaults */
   sec_spec->active = 0;
@@ -877,13 +822,11 @@ tsapi void
 TSSspecDestroy(TSSspec * ele)
 {
   if (ele) {
-    if (ele->prefix)
-      xfree(ele->prefix);
-    if (ele->suffix)
-      xfree(ele->suffix);
+    ats_free(ele->prefix);
+    ats_free(ele->suffix);
     if (ele->port)
       TSPortEleDestroy(ele->port);
-    xfree(ele);
+    ats_free(ele);
   }
   return;
 }
@@ -891,11 +834,7 @@ TSSspecDestroy(TSSspec * ele)
 tsapi TSPdSsFormat *
 TSPdSsFormatCreate(void)
 {
-  TSPdSsFormat *ele;
-
-  ele = (TSPdSsFormat *) xmalloc(sizeof(TSPdSsFormat));
-  if (!ele)
-    return NULL;
+  TSPdSsFormat *ele = (TSPdSsFormat *)ats_malloc(sizeof(TSPdSsFormat));
 
   /* should set default values here */
   ele->pd_type = TS_PD_UNDEFINED;
@@ -920,16 +859,13 @@ tsapi void
 TSPdSsFormatDestroy(TSPdSsFormat * ele)
 {
   if (ele) {
-    if (ele->pd_val)
-      xfree(ele->pd_val);
-    if (ele->sec_spec.src_ip)
-      xfree(ele->sec_spec.src_ip);
-    if (ele->sec_spec.prefix)
-      xfree(ele->sec_spec.prefix);
-    if (ele->sec_spec.suffix)
-      xfree(ele->sec_spec.suffix);
+    ats_free(ele->pd_val);
+    ats_free(ele->sec_spec.src_ip);
+    ats_free(ele->sec_spec.prefix);
+    ats_free(ele->sec_spec.suffix);
     if (ele->sec_spec.port)
       TSPortEleDestroy(ele->sec_spec.port);
+    ats_free(ele);
   }
   return;
 }
@@ -951,9 +887,7 @@ TSCacheEleCreate(TSRuleTypeT type)
       type != TS_CACHE_TTL_IN_CACHE && type != TS_CACHE_AUTH_CONTENT && type != TS_TYPE_UNDEFINED)
     return NULL;                // invalid type
 
-  ele = (TSCacheEle *) xmalloc(sizeof(TSCacheEle));
-  if (!ele)
-    return NULL;
+  ele = (TSCacheEle *)ats_malloc(sizeof(TSCacheEle));
 
   /* set defaults */
   ele->cfg_ele.type = type;
@@ -972,7 +906,7 @@ TSCacheEleDestroy(TSCacheEle * ele)
 {
   if (ele) {
     TSPdSsFormatDestroy(&(ele->cache_info));
-    xfree(ele);
+    ats_free(ele);
   }
   return;
 }
@@ -986,11 +920,7 @@ TSCacheEleDestroy(TSCacheEle * ele)
 tsapi TSCongestionEle *
 TSCongestionEleCreate()
 {
-  TSCongestionEle *ele;
-
-  ele = (TSCongestionEle *) xmalloc(sizeof(TSCongestionEle));
-  if (!ele)
-    return NULL;
+  TSCongestionEle *ele = (TSCongestionEle *)ats_malloc(sizeof(TSCongestionEle));
 
   /* set defaults */
   ele->cfg_ele.type = TS_CONGESTION;
@@ -1011,7 +941,7 @@ TSCongestionEleCreate()
   ele->dead_os_conn_timeout = 15;
   ele->dead_os_conn_retries = 1;
   ele->max_connection = -1;
-  ele->error_page_uri = xstrdup("congestion#retryAfter");
+  ele->error_page_uri = ats_strdup("congestion#retryAfter");
 
   return ele;
 }
@@ -1020,13 +950,10 @@ tsapi void
 TSCongestionEleDestroy(TSCongestionEle * ele)
 {
   if (ele) {
-    if (ele->pd_val)
-      xfree(ele->pd_val);
-    if (ele->prefix)
-      xfree(ele->prefix);
-    if (ele->error_page_uri)
-      xfree(ele->error_page_uri);
-    xfree(ele);
+    ats_free(ele->pd_val);
+    ats_free(ele->prefix);
+    ats_free(ele->error_page_uri);
+    ats_free(ele);
   }
   return;
 }
@@ -1038,11 +965,7 @@ TSCongestionEleDestroy(TSCongestionEle * ele)
 tsapi TSHostingEle *
 TSHostingEleCreate()
 {
-  TSHostingEle *ele;
-
-  ele = (TSHostingEle *) xmalloc(sizeof(TSHostingEle));
-  if (!ele)
-    return NULL;
+  TSHostingEle *ele = (TSHostingEle *)ats_malloc(sizeof(TSHostingEle));
 
   ele->cfg_ele.type = TS_HOSTING;
   ele->cfg_ele.error = TS_ERR_OKAY;
@@ -1057,11 +980,10 @@ tsapi void
 TSHostingEleDestroy(TSHostingEle * ele)
 {
   if (ele) {
-    if (ele->pd_val)
-      xfree(ele->pd_val);
+    ats_free(ele->pd_val);
     if (ele->volumes)
       TSIntListDestroy(ele->volumes);
-    xfree(ele);
+    ats_free(ele);
   }
   return;
 }
@@ -1072,11 +994,7 @@ TSHostingEleDestroy(TSHostingEle * ele)
 tsapi TSIcpEle *
 TSIcpEleCreate()
 {
-  TSIcpEle *ele;
-
-  ele = (TSIcpEle *) xmalloc(sizeof(TSIcpEle));
-  if (!ele)
-    return NULL;
+  TSIcpEle *ele = (TSIcpEle *)ats_malloc(sizeof(TSIcpEle));
 
   /* set defaults */
   ele->cfg_ele.type = TS_ICP;
@@ -1098,13 +1016,10 @@ tsapi void
 TSIcpEleDestroy(TSIcpEle * ele)
 {
   if (ele) {
-    if (ele->peer_hostname)
-      xfree(ele->peer_hostname);
-    if (ele->peer_host_ip_addr)
-      xfree(ele->peer_host_ip_addr);
-    if (ele->mc_ip_addr)
-      xfree(ele->mc_ip_addr);
-    xfree(ele);
+    ats_free(ele->peer_hostname);
+    ats_free(ele->peer_host_ip_addr);
+    ats_free(ele->mc_ip_addr);
+    ats_free(ele);
   }
   return;
 }
@@ -1116,11 +1031,7 @@ tsapi TSIpAllowEle *
 TSIpAllowEleCreate()
 {
 
-  TSIpAllowEle *ele;
-
-  ele = (TSIpAllowEle *) xmalloc(sizeof(TSIpAllowEle));
-  if (!ele)
-    return NULL;
+  TSIpAllowEle *ele = (TSIpAllowEle *)ats_malloc(sizeof(TSIpAllowEle));
 
   ele->cfg_ele.type = TS_IP_ALLOW;
   ele->cfg_ele.error = TS_ERR_OKAY;
@@ -1136,7 +1047,7 @@ TSIpAllowEleDestroy(TSIpAllowEle * ele)
   if (ele) {
     if (ele->src_ip_addr)
       TSIpAddrEleDestroy(ele->src_ip_addr);
-    xfree(ele);
+    ats_free(ele);
   }
   return;
 
@@ -1149,11 +1060,7 @@ TSIpAllowEleDestroy(TSIpAllowEle * ele)
 tsapi TSLogFilterEle *
 TSLogFilterEleCreate()
 {
-  TSLogFilterEle *ele;
-
-  ele = (TSLogFilterEle *) xmalloc(sizeof(TSLogFilterEle));
-  if (!ele)
-    return NULL;
+  TSLogFilterEle *ele = (TSLogFilterEle *)ats_malloc(sizeof(TSLogFilterEle));
 
   ele->cfg_ele.type = TS_LOG_FILTER;
   ele->cfg_ele.error = TS_ERR_OKAY;
@@ -1170,13 +1077,10 @@ tsapi void
 TSLogFilterEleDestroy(TSLogFilterEle * ele)
 {
   if (ele) {
-    if (ele->filter_name)
-      xfree(ele->filter_name);
-    if (ele->log_field)
-      xfree(ele->log_field);
-    if (ele->compare_str)
-      xfree(ele->compare_str);
-    xfree(ele);
+    ats_free(ele->filter_name);
+    ats_free(ele->log_field);
+    ats_free(ele->compare_str);
+    ats_free(ele);
   }
   return;
 }
@@ -1187,11 +1091,7 @@ TSLogFilterEleDestroy(TSLogFilterEle * ele)
 tsapi TSLogFormatEle *
 TSLogFormatEleCreate()
 {
-  TSLogFormatEle *ele;
-
-  ele = (TSLogFormatEle *) xmalloc(sizeof(TSLogFormatEle));
-  if (!ele)
-    return NULL;
+  TSLogFormatEle *ele = (TSLogFormatEle *)ats_malloc(sizeof(TSLogFormatEle));
 
   ele->cfg_ele.type = TS_LOG_FORMAT;
   ele->cfg_ele.error = TS_ERR_OKAY;
@@ -1206,11 +1106,9 @@ tsapi void
 TSLogFormatEleDestroy(TSLogFormatEle * ele)
 {
   if (ele) {
-    if (ele->name)
-      xfree(ele->name);
-    if (ele->format)
-      xfree(ele->format);
-    xfree(ele);
+    ats_free(ele->name);
+    ats_free(ele->format);
+    ats_free(ele);
   }
   return;
 }
@@ -1221,11 +1119,7 @@ TSLogFormatEleDestroy(TSLogFormatEle * ele)
 tsapi TSLogObjectEle *
 TSLogObjectEleCreate()
 {
-  TSLogObjectEle *ele;
-
-  ele = (TSLogObjectEle *) xmalloc(sizeof(TSLogObjectEle));
-  if (!ele)
-    return NULL;
+  TSLogObjectEle *ele = (TSLogObjectEle *)ats_malloc(sizeof(TSLogObjectEle));
 
   ele->cfg_ele.type = TS_LOG_OBJECT;
   ele->cfg_ele.error = TS_ERR_OKAY;
@@ -1244,10 +1138,8 @@ tsapi void
 TSLogObjectEleDestroy(TSLogObjectEle * ele)
 {
   if (ele) {
-    if (ele->format_name)
-      xfree(ele->format_name);
-    if (ele->file_name)
-      xfree(ele->file_name);
+    ats_free(ele->format_name);
+    ats_free(ele->file_name);
     if (ele->collation_hosts)
       TSDomainListDestroy(ele->collation_hosts);
     if (ele->filters)
@@ -1256,7 +1148,7 @@ TSLogObjectEleDestroy(TSLogObjectEle * ele)
       TSStringListDestroy(ele->protocols);
     if (ele->server_hosts)
       TSStringListDestroy(ele->server_hosts);
-    xfree(ele);
+    ats_free(ele);
   }
   return;
 }
@@ -1272,9 +1164,7 @@ TSParentProxyEleCreate(TSRuleTypeT type)
   if (type != TS_PP_PARENT && type != TS_PP_GO_DIRECT && type != TS_TYPE_UNDEFINED)
     return NULL;
 
-  ele = (TSParentProxyEle *) xmalloc(sizeof(TSParentProxyEle));
-  if (!ele)
-    return NULL;
+  ele = (TSParentProxyEle *)ats_malloc(sizeof(TSParentProxyEle));
 
   ele->cfg_ele.type = type;
   ele->cfg_ele.error = TS_ERR_OKAY;
@@ -1293,7 +1183,7 @@ TSParentProxyEleDestroy(TSParentProxyEle * ele)
     TSPdSsFormatDestroy(&(ele->parent_info));
     if (ele->proxy_list)
       TSDomainListDestroy(ele->proxy_list);
-    xfree(ele);
+    ats_free(ele);
   }
 
   return;
@@ -1305,11 +1195,7 @@ TSParentProxyEleDestroy(TSParentProxyEle * ele)
 tsapi TSVolumeEle *
 TSVolumeEleCreate()
 {
-  TSVolumeEle *ele;
-
-  ele = (TSVolumeEle *) xmalloc(sizeof(TSVolumeEle));
-  if (!ele)
-    return NULL;
+  TSVolumeEle *ele = (TSVolumeEle *)ats_malloc(sizeof(TSVolumeEle));
 
   ele->cfg_ele.type = TS_VOLUME;
   ele->cfg_ele.error = TS_ERR_OKAY;
@@ -1324,9 +1210,7 @@ TSVolumeEleCreate()
 tsapi void
 TSVolumeEleDestroy(TSVolumeEle * ele)
 {
-  if (ele) {
-    xfree(ele);
-  }
+  ats_free(ele);
   return;
 }
 
@@ -1336,11 +1220,7 @@ TSVolumeEleDestroy(TSVolumeEle * ele)
 tsapi TSPluginEle *
 TSPluginEleCreate()
 {
-  TSPluginEle *ele;
-
-  ele = (TSPluginEle *) xmalloc(sizeof(TSPluginEle));
-  if (!ele)
-    return NULL;
+  TSPluginEle *ele = (TSPluginEle *)ats_malloc(sizeof(TSPluginEle));
 
   ele->cfg_ele.type = TS_PLUGIN;
   ele->cfg_ele.error = TS_ERR_OKAY;
@@ -1354,11 +1234,10 @@ tsapi void
 TSPluginEleDestroy(TSPluginEle * ele)
 {
   if (ele) {
-    if (ele->name)
-      xfree(ele->name);
+    ats_free(ele->name);
     if (ele->args)
       TSStringListDestroy(ele->args);
-    xfree(ele);
+    ats_free(ele);
   }
   return;
 }
@@ -1376,10 +1255,7 @@ TSRemapEleCreate(TSRuleTypeT type)
       type != TS_REMAP_REDIRECT && type != TS_REMAP_REDIRECT_TEMP && type != TS_TYPE_UNDEFINED)
     return NULL;
 
-  ele = (TSRemapEle *) xmalloc(sizeof(TSRemapEle));
-  if (!ele)
-    return NULL;
-
+  ele = (TSRemapEle *)ats_malloc(sizeof(TSRemapEle));
   ele->cfg_ele.type = type;
   ele->cfg_ele.error = TS_ERR_OKAY;
   ele->map = true;
@@ -1399,15 +1275,11 @@ void
 TSRemapEleDestroy(TSRemapEle * ele)
 {
   if (ele) {
-    if (ele->from_host)
-      xfree(ele->from_host);
-    if (ele->from_path_prefix)
-      xfree(ele->from_path_prefix);
-    if (ele->to_host)
-      xfree(ele->to_host);
-    if (ele->to_path_prefix)
-      xfree(ele->to_path_prefix);
-    xfree(ele);
+    ats_free(ele->from_host);
+    ats_free(ele->from_path_prefix);
+    ats_free(ele->to_host);
+    ats_free(ele->to_path_prefix);
+    ats_free(ele);
   }
 }
 
@@ -1417,10 +1289,7 @@ TSRemapEleDestroy(TSRemapEle * ele)
 TSSocksEle *
 TSSocksEleCreate(TSRuleTypeT type)
 {
-  TSSocksEle *ele;
-  ele = (TSSocksEle *) xmalloc(sizeof(TSSocksEle));
-  if (!ele)
-    return NULL;
+  TSSocksEle *ele = (TSSocksEle *)ats_malloc(sizeof(TSSocksEle));
 
   ele->cfg_ele.type = type;
   ele->cfg_ele.error = TS_ERR_OKAY;
@@ -1444,11 +1313,9 @@ TSSocksEleDestroy(TSSocksEle * ele)
       TSIpAddrEleDestroy(ele->dest_ip_addr);
     if (ele->socks_servers)
       TSDomainListDestroy(ele->socks_servers);
-    if (ele->username)
-      xfree(ele->username);
-    if (ele->password)
-      xfree(ele->password);
-    xfree(ele);
+    ats_free(ele->username);
+    ats_free(ele->password);
+    ats_free(ele);
   }
 }
 
@@ -1458,10 +1325,7 @@ TSSocksEleDestroy(TSSocksEle * ele)
 TSSplitDnsEle *
 TSSplitDnsEleCreate()
 {
-  TSSplitDnsEle *ele;
-  ele = (TSSplitDnsEle *) xmalloc(sizeof(TSSplitDnsEle));
-  if (!ele)
-    return NULL;
+  TSSplitDnsEle *ele = (TSSplitDnsEle *)ats_malloc(sizeof(TSSplitDnsEle));
 
   ele->cfg_ele.type = TS_SPLIT_DNS;
   ele->cfg_ele.error = TS_ERR_OKAY;
@@ -1478,15 +1342,13 @@ void
 TSSplitDnsEleDestroy(TSSplitDnsEle * ele)
 {
   if (ele) {
-    if (ele->pd_val)
-      xfree(ele->pd_val);
+    ats_free(ele->pd_val);
     if (ele->dns_servers_addrs)
       TSDomainListDestroy(ele->dns_servers_addrs);
-    if (ele->def_domain)
-      xfree(ele->def_domain);
+    ats_free(ele->def_domain);
     if (ele->search_list)
       TSDomainListDestroy(ele->search_list);
-    xfree(ele);
+    ats_free(ele);
   }
   return;
 }
@@ -1497,10 +1359,7 @@ TSSplitDnsEleDestroy(TSSplitDnsEle * ele)
 TSStorageEle *
 TSStorageEleCreate()
 {
-  TSStorageEle *ele;
-  ele = (TSStorageEle *) xmalloc(sizeof(TSStorageEle));
-  if (!ele)
-    return NULL;
+  TSStorageEle *ele = (TSStorageEle *)ats_malloc(sizeof(TSStorageEle));
 
   ele->cfg_ele.type = TS_STORAGE;
   ele->cfg_ele.error = TS_ERR_OKAY;
@@ -1514,9 +1373,8 @@ void
 TSStorageEleDestroy(TSStorageEle * ele)
 {
   if (ele) {
-    if (ele->pathname)
-      xfree(ele->pathname);
-    xfree(ele);
+    ats_free(ele->pathname);
+    ats_free(ele);
   }
   return;
 }
@@ -1527,10 +1385,7 @@ TSStorageEleDestroy(TSStorageEle * ele)
 TSUpdateEle *
 TSUpdateEleCreate()
 {
-  TSUpdateEle *ele;
-  ele = (TSUpdateEle *) xmalloc(sizeof(TSUpdateEle));
-  if (!ele)
-    return NULL;
+  TSUpdateEle *ele = (TSUpdateEle *)ats_malloc(sizeof(TSUpdateEle));
 
   ele->cfg_ele.type = TS_UPDATE_URL;
   ele->cfg_ele.error = TS_ERR_OKAY;
@@ -1547,11 +1402,10 @@ void
 TSUpdateEleDestroy(TSUpdateEle * ele)
 {
   if (ele) {
-    if (ele->url)
-      xfree(ele->url);
+    ats_free(ele->url);
     if (ele->headers)
       TSStringListDestroy(ele->headers);
-    xfree(ele);
+    ats_free(ele);
   }
   return;
 }
@@ -1562,10 +1416,7 @@ TSUpdateEleDestroy(TSUpdateEle * ele)
 TSVirtIpAddrEle *
 TSVirtIpAddrEleCreate()
 {
-  TSVirtIpAddrEle *ele;
-  ele = (TSVirtIpAddrEle *) xmalloc(sizeof(TSVirtIpAddrEle));
-  if (!ele)
-    return NULL;
+  TSVirtIpAddrEle *ele = (TSVirtIpAddrEle *)ats_malloc(sizeof(TSVirtIpAddrEle));
 
   ele->cfg_ele.type = TS_VADDRS;
   ele->cfg_ele.error = TS_ERR_OKAY;
@@ -1580,11 +1431,9 @@ void
 TSVirtIpAddrEleDestroy(TSVirtIpAddrEle * ele)
 {
   if (ele) {
-    if (ele->intr)
-      xfree(ele->intr);
-    if (ele->ip_addr)
-      xfree(ele->ip_addr);
-    xfree(ele);
+    ats_free(ele->intr);
+    ats_free(ele->ip_addr);
+    ats_free(ele);
   }
 }
 
@@ -1661,20 +1510,13 @@ TSError
 TSRecordGetString(const char *rec_name, TSString *string_val)
 {
   TSError ret;
-  char *str;
-  size_t str_size = 0;
 
   TSRecordEle *ele = TSRecordEleCreate();
   ret = MgmtRecordGet(rec_name, ele);
   if (ret != TS_ERR_OKAY)
     goto END;
 
-  str_size = strlen(ele->string_val) + 1;
-  str = (char *) xmalloc(sizeof(char) * str_size);
-  if (!str)
-    return TS_ERR_SYS_CALL;
-  ink_strncpy(str, ele->string_val, str_size);
-  *string_val = str;
+  *string_val = ats_strdup(ele->string_val);
 
 END:
   TSRecordEleDestroy(ele);
@@ -2023,7 +1865,7 @@ TSGetErrorMessage(TSError err_id)
     snprintf(msg, sizeof(msg), "[%d] Invalid configuration rule reached.", err_id);
     break;
   case TS_ERR_NET_ESTABLISH:
-    snprintf(msg, sizeof(msg), "[%d] Error establishing socket conenction.", err_id);
+    snprintf(msg, sizeof(msg), "[%d] Error establishing socket connection.", err_id);
     break;
   case TS_ERR_NET_READ:       /* Error reading from socket */
     snprintf(msg, sizeof(msg), "[%d] Error reading from socket.", err_id);
@@ -2052,7 +1894,7 @@ TSGetErrorMessage(TSError err_id)
     break;
   }
 
-  err_msg = xstrdup(msg);
+  err_msg = ats_strdup(msg);
   return err_msg;
 }
 
@@ -2071,9 +1913,7 @@ TSEncryptPassword(char *passwd, char **e_passwd)
   ink_debug_assert(TS_ENCRYPT_PASSWD_LEN <= passwd_md5_str_len);
 
   const size_t md5StringSize = (passwd_md5_str_len + 1) * sizeof(char);
-  passwd_md5_str = (char *) xmalloc(md5StringSize);
-  if (!passwd_md5_str)
-    return TS_ERR_FAIL;
+  passwd_md5_str = (char *)ats_malloc(md5StringSize);
 
   ink_code_incr_md5_init(&md5_context);
   ink_code_incr_md5_update(&md5_context, passwd, strlen(passwd));
@@ -2161,26 +2001,26 @@ TSReadFromUrlEx(const char *url, char **header, int *headerSize, char **body, in
   const char *tempPath = strstr(url, "/");
   char *host_and_port;
   if (tempPath) {
-    host_and_port = xstrndup(url, strlen(url) - strlen(tempPath));
+    host_and_port = ats_strndup(url, strlen(url) - strlen(tempPath));
     tempPath += 1;              // advance one position to get rid of leading '/'
-    httpPath = xstrdup(tempPath);
+    httpPath = ats_strdup(tempPath);
   } else {
-    host_and_port = xstrdup(url);
-    httpPath = xstrdup("");
+    host_and_port = ats_strdup(url);
+    httpPath = ats_strdup("");
   }
 
   // the port proceed by a ":", if it exists
   char *colon = strstr(host_and_port, ":");
   if (colon) {
-    httpHost = xstrndup(host_and_port, strlen(host_and_port) - strlen(colon));
+    httpHost = ats_strndup(host_and_port, strlen(host_and_port) - strlen(colon));
     colon += 1;                 // advance one position to get rid of leading ':'
     httpPort = ink_atoi(colon);
     if (httpPort <= 0)
       httpPort = HTTP_PORT;
   } else {
-    httpHost = xstrdup(host_and_port);
+    httpHost = ats_strdup(host_and_port);
   }
-  xfree(host_and_port);
+  ats_free(host_and_port);
 
   hFD = connectDirect(httpHost, httpPort, timeout);
   if (hFD == -1) {
@@ -2202,14 +2042,13 @@ TSReadFromUrlEx(const char *url, char **header, int *headerSize, char **body, in
     goto END;
 
   if (header && headerSize)
-    *header = xstrndup(hdr_temp, *headerSize);
-  *body = xstrndup(bdy_temp, *bodySize);
+    *header = ats_strndup(hdr_temp, *headerSize);
+  *body = ats_strndup(bdy_temp, *bodySize);
 
 END:
-  if (httpHost)
-    xfree(httpHost);
-  if (httpPath)
-    xfree(httpPath);
+  ats_free(httpHost);
+  ats_free(httpPath);
+
   return status;
 }
 
@@ -2247,7 +2086,7 @@ TSLookupFromCacheUrl(TSString url, TSString * info)
   if ((err = parseHTTPResponse(response, &header, &hdr_size, &body, &bdy_size)) != TS_ERR_OKAY)
     goto END;
 
-  *info = xstrndup(body, bdy_size);
+  *info = ats_strndup(body, bdy_size);
 
 END:
   return err;
@@ -2285,7 +2124,7 @@ TSLookupFromCacheUrlRegex(TSString url_regex, TSString * list)
   if ((err = parseHTTPResponse(response, &header, &hdr_size, &body, &bdy_size)) != TS_ERR_OKAY)
     goto END;
 
-  *list = xstrndup(body, bdy_size);
+  *list = ats_strndup(body, bdy_size);
 END:
   return err;
 }
@@ -2322,7 +2161,7 @@ TSDeleteFromCacheUrl(TSString url, TSString * info)
   if ((err = parseHTTPResponse(response, &header, &hdr_size, &body, &bdy_size)) != TS_ERR_OKAY)
     goto END;
 
-  *info = xstrndup(body, bdy_size);
+  *info = ats_strndup(body, bdy_size);
 
 END:
   return err;
@@ -2360,7 +2199,7 @@ TSDeleteFromCacheUrlRegex(TSString url_regex, TSString * list)
   if ((err = parseHTTPResponse(response, &header, &hdr_size, &body, &bdy_size)) != TS_ERR_OKAY)
     goto END;
 
-  *list = xstrndup(body, bdy_size);
+  *list = ats_strndup(body, bdy_size);
 END:
   return err;
 }
@@ -2397,7 +2236,7 @@ TSInvalidateFromCacheUrlRegex(TSString url_regex, TSString * list)
   if ((err = parseHTTPResponse(response, &header, &hdr_size, &body, &bdy_size)) != TS_ERR_OKAY)
     goto END;
 
-  *list = xstrndup(body, bdy_size);
+  *list = ats_strndup(body, bdy_size);
 END:
   return err;
 }
@@ -2762,8 +2601,8 @@ resetHostName(TSRmServerEle * ele, const char *hostname, const char *tail)
 {
   char buff[MAX_RULE_SIZE];
 
-  xfree(ele->str_val);
+  ats_free(ele->str_val);
   snprintf(buff, sizeof(buff), "%s.%s", hostname, tail);
-  ele->str_val = xstrdup(buff);
+  ele->str_val = ats_strdup(buff);
   return;
 }
