@@ -699,7 +699,6 @@ int IsValidHostname(char *str);
 int IsValidFQHostname(char *str);
 int IsValidDomainname(char *str);
 int IsValidIpAddress(char *str);
-int getnetparms(char *ipaddr, char *netmask);
 #if defined(solaris)
 int getnetmask(char *mask);
 #endif
@@ -765,6 +764,20 @@ int cliCheckIfEnabled(char *command);
 //
 int Cmd_Disable(ClientData clientData, Tcl_Interp * interp, int argc, const char *argv[]);
 
+////////////////////////////////////////////////////////////////
+// Cmd_Help
+//
+// This is the callback function for the "help" command. It displays the
+// online help for the requested topic(s).
+//
+// Parameters:
+//    clientData -- information about parsed arguments
+//    interp -- the Tcl interpreter
+//    argc -- number of command arguments
+//    argv -- the command arguments
+//
+int Cmd_Help(ClientData clientData, Tcl_Interp * interp, int argc, const char *argv[]);
+
 
 ////////////////////////////////////////////////////////////////
 //
