@@ -38,6 +38,16 @@
 #include "I_Tasks.h"
 
 
+typedef enum
+  {
+    OVERRIDABLE_TYPE_NULL = 0,
+    OVERRIDABLE_TYPE_INT,
+    OVERRIDABLE_TYPE_FLOAT,
+    OVERRIDABLE_TYPE_STRING,
+    OVERRIDABLE_TYPE_BYTE
+  } OverridableDataType;
+typedef int8_t TSMgmtByte; // Not for external use
+
 /* ****** Cache Structure ********* */
 
 // For memory corruption detection
@@ -106,8 +116,6 @@ struct INKConfigImpl:public ConfigInfo
   }
 };
 
-
-
 struct HttpAltInfo
 {
   HTTPHdr m_client_req;
