@@ -1,6 +1,6 @@
 /** @file
 
-  A brief file description
+  Implementation of Parent Proxy routing
 
   @section license License
 
@@ -21,11 +21,6 @@
   limitations under the License.
  */
 
-/*****************************************************************************
- *
- *  ParentSelection.cc - Implementation of Parent Proxy routing
- *
- ****************************************************************************/
 
 #include "ink_unused.h"  /* MAGIC_EDITING_TAG */
 
@@ -149,9 +144,8 @@ ParentConfig::reconfigure()
   // Handle default parent
   PARENT_ReadConfigStringAlloc(default_val, default_var);
   params->DefaultParent = createDefaultParent(default_val);
-  if (default_val) {
-    xfree(default_val);
-  }
+  ats_free(default_val);
+
   // Handle parent timeout
   PARENT_ReadConfigInteger(retry_time, retry_var);
   params->ParentRetryTime = retry_time;
@@ -312,7 +306,7 @@ ParentConfigParams::findParent(HttpRequestData * rdata, ParentResult * result)
       Debug("parent_select", "Result for %s was %s", host, ParentResultStr[result->r]);
       break;
     case PARENT_SPECIFIED:
-      Debug("parent_select", "sizeof ParentResult = %d", sizeof(ParentResult));
+      Debug("parent_select", "sizeof ParentResult = %zu", sizeof(ParentResult));
       Debug("parent_select", "Result for %s was parent %s:%d", host, result->hostname, result->port);
       break;
     default:
@@ -515,7 +509,15 @@ ParentRecord::FindParent(bool first_call, ParentResult * result, RD * rdata, Par
         break;
       case P_HASH_ROUND_ROBIN:
         // INKqa12817 - make sure to convert to host byte order
-        cur_index = ntohl(rdata->get_client_ip()) % num_parents;
+        // Why was it important to do host order here?  And does this have any
+        // impact with the transition to IPv6?  The IPv4 functionality is 
+        // preserved for now anyway as ats_ip_hash returns the 32-bit address in
+        // that case.
+        if (rdata->get_client_ip() != NULL) {
+            cur_index = ntohl(ats_ip_hash(rdata->get_client_ip())) % num_parents;
+        } else {
+            cur_index = 0;
+        }
         break;
       case P_NO_ROUND_ROBIN:
         cur_index = result->start_parent = 0;
@@ -553,8 +555,8 @@ ParentRecord::FindParent(bool first_call, ParentResult * result, RD * rdata, Par
       parentUp = true;
     } else {
       if ((result->wrap_around) || ((parents[cur_index].failedAt + config->ParentRetryTime) < request_info->xact_start)) {
-        Debug("parent_select", "Parent[%d].failedAt = %u, retry = %u,xact_start = %u but wrap = %d", cur_index,
-              parents[cur_index].failedAt, config->ParentRetryTime, request_info->xact_start, result->wrap_around);
+        Debug("parent_select", "Parent[%d].failedAt = %u, retry = %u,xact_start = %" PRId64 " but wrap = %d", cur_index,
+              parents[cur_index].failedAt, config->ParentRetryTime, (int64_t)request_info->xact_start, result->wrap_around);
         // Reuse the parent
         parentUp = true;
         parentRetry = true;
@@ -628,7 +630,7 @@ ParentRecord::ProcessParents(char *val)
     return "No parents specified";
   }
   // Allocate the parents array
-  this->parents = (pRecord *) xmalloc(sizeof(pRecord) * numTok);
+  this->parents = (pRecord *)ats_malloc(sizeof(pRecord) * numTok);
 
   // Loop through the set of parents specified
   //
@@ -678,10 +680,9 @@ ParentRecord::ProcessParents(char *val)
   return NULL;
 
 MERROR:
-  if (parents != NULL) {
-    xfree(parents);
-    parents = NULL;
-  }
+  ats_free(parents);
+  parents = NULL;
+
   return errPtr;
 }
 
@@ -708,10 +709,10 @@ ParentRecord::DefaultInit(char *val)
   errPtr = ProcessParents(val);
 
   if (errPtr != NULL) {
-    errBuf = (char *) xmalloc(1024);
+    errBuf = (char *)ats_malloc(1024);
     snprintf(errBuf, 1024, "%s %s for default parent proxy", modulePrefix, errPtr);
     SignalError(errBuf, alarmAlready);
-    xfree(errBuf);
+    ats_free(errBuf);
     return false;
   } else {
     return true;
@@ -726,7 +727,7 @@ ParentRecord::DefaultInit(char *val)
 //
 //    Returns NULL if everything is OK
 //      Otherwise, returns an error string that the caller MUST
-//        DEALLOCATE with free()
+//        DEALLOCATE with ats_free()
 //
 char *
 ParentRecord::Init(matcher_line * line_info)
@@ -778,7 +779,7 @@ ParentRecord::Init(matcher_line * line_info)
     }
     // Report errors generated by ProcessParents();
     if (errPtr != NULL) {
-      errBuf = (char *) xmalloc(errBufLen * sizeof(char));
+      errBuf = (char *)ats_malloc(errBufLen * sizeof(char));
       snprintf(errBuf, errBufLen, "%s %s at line %d", modulePrefix, errPtr, line_num);
       return errBuf;
     }
@@ -791,7 +792,7 @@ ParentRecord::Init(matcher_line * line_info)
   }
 
   if (this->parents == NULL && go_direct == false) {
-    errBuf = (char *) xmalloc(errBufLen * sizeof(char));
+    errBuf = (char *)ats_malloc(errBufLen * sizeof(char));
     snprintf(errBuf, errBufLen, "%s No parent specified in parent.config at line %d", modulePrefix, line_num);
     return errBuf;
   }
@@ -800,7 +801,7 @@ ParentRecord::Init(matcher_line * line_info)
     tmp = ProcessModifiers(line_info);
 
     if (tmp != NULL) {
-      errBuf = (char *) xmalloc(errBufLen * sizeof(char));
+      errBuf = (char *)ats_malloc(errBufLen * sizeof(char));
       snprintf(errBuf, errBufLen, "%s %s at line %d in parent.config", modulePrefix, tmp, line_num);
       return errBuf;
     }
@@ -830,13 +831,13 @@ ParentRecord::UpdateMatch(ParentResult * result, RD * rdata)
     result->rec = this;
     result->line_number = this->line_num;
 
-    Debug("parent_select", "Matched with 0x%x parent node from line %d", this, this->line_num);
+    Debug("parent_select", "Matched with %p parent node from line %d", this, this->line_num);
   }
 }
 
 ParentRecord::~ParentRecord()
 {
-  xfree(parents);
+  ats_free(parents);
 }
 
 void
@@ -957,19 +958,14 @@ setup_socks_servers(ParentRecord * rec_arr, int len)
     int n_parents = rec_arr[j].num_parents;
 
     for (int i = 0; i < n_parents; i++) {
-      uint32_t bad_ip = (uint32_t) - 1;
-      uint8_t *ip;
-
-      ink_gethostbyname_r_data data;
-      struct hostent *ent = ink_gethostbyname_r(pr[i].hostname, &data);
-
-      if (ent) {
-        ip = (uint8_t *) ent->h_addr_list[0];
+      IpEndpoint ip4, ip6;
+      if (0 == ats_ip_getbestaddrinfo(pr[i].hostname, &ip4, &ip6)) {
+        IpEndpoint* ip = ats_is_ip6(&ip6) ? &ip6 : &ip4;
+        ats_ip_ntop(ip, pr[i].hostname, MAXDNAME+1);
       } else {
         Warning("Could not resolve socks server name \"%s\". " "Please correct it", pr[i].hostname);
-        ip = (uint8_t *) & bad_ip;
+        snprintf(pr[i].hostname, MAXDNAME+1, "255.255.255.255");
       }
-      snprintf(pr[i].hostname, MAXDNAME + 1, "%hhu.%hhu.%hhu.%hhu", ip[0], ip[1], ip[2], ip[3]);
     }
   }
 
@@ -994,9 +990,7 @@ SocksServerConfig::reconfigure()
   // Handle default parent
   PARENT_ReadConfigStringAlloc(default_val, "proxy.config.socks.default_servers");
   params->DefaultParent = createDefaultParent(default_val);
-  if (default_val) {
-    xfree(default_val);
-  }
+  ats_free(default_val);
 
   if (params->DefaultParent)
     setup_socks_servers(params->DefaultParent, 1);
@@ -1052,12 +1046,15 @@ SocksServerConfig::print()
 }
 
 void
-request_to_data(HttpRequestData * req, ip_addr_t srcip, ip_addr_t dstip, const char *str)
+request_to_data(HttpRequestData * req, sockaddr const* srcip, sockaddr const* dstip, const char *str)
 {
   HTTPParser parser;
 
-  req->src_ip = srcip;
-  req->dest_ip = dstip;
+  ink_zero(req->src_ip);
+  ats_ip_copy(&req->src_ip.sa, srcip);
+  ink_zero(req->dest_ip);
+  ats_ip_copy(&req->dest_ip.sa, dstip);
+
   req->hdr = NEW(new HTTPHdr);
 
   http_parser_init(&parser);
@@ -1068,8 +1065,6 @@ request_to_data(HttpRequestData * req, ip_addr_t srcip, ip_addr_t dstip, const c
 }
 
 
-#define IP(a,b,c,d) htonl((a) << 24 | (b) << 16 | (c) << 8 | (d))
-
 static int passes;
 static int fails;
 
@@ -1088,7 +1083,7 @@ EXCLUSIVE_REGRESSION_TEST(PARENTSELECTION) (RegressionTest * t, int intensity_le
   config.startup();
   params->ParentEnable = true;
   char tbl[2048];
-#define T(x) strncat(tbl,x, sizeof(tbl));
+#define T(x) ink_strlcat(tbl,x, sizeof(tbl));
 #define REBUILD params->ParentTable = new P_table("", "ParentSelection Unit Test Table", &http_dest_tags, ALLOW_HOST_TABLE | ALLOW_REGEX_TABLE | ALLOW_IP_TABLE | DONT_BUILD_TABLE); params->ParentTable->BuildTableFromString(tbl);
   HttpRequestData *request = NULL;
   ParentResult *result = NULL;
@@ -1125,7 +1120,10 @@ EXCLUSIVE_REGRESSION_TEST(PARENTSELECTION) (RegressionTest * t, int intensity_le
       (!g && !b && !i && (v == 17))), 2)
     // Test 3 - 6 Parenting Table
     tbl[0] = '\0';
-  T("dest_ip=209.131.62.14 parent=cat:37,dog:24 round_robin=strict\n")  /* L1 */
+# define TEST_IP4_ADDR "209.131.62.14"
+# define TEST_IP6_ADDR "BEEF:DEAD:ABBA:CAFE:1337:1E1F:5EED:C0FF"
+  T("dest_ip=" TEST_IP4_ADDR " parent=cat:37,dog:24 round_robin=strict\n")  /* L1 */
+  T("dest_ip=" TEST_IP6_ADDR " parent=zwoop:37,jMCg:24 round_robin=strict\n")  /* L1 */
     T("dest_host=www.pilot.net parent=pilot_net:80\n")  /* L2 */
     T("url_regex=snoopy parent=odie:80,garfield:80 round_robin=true\n") /* L3 */
     T("dest_domain=i.am parent=amy:80,katie:80,carissa:771 round_robin=false\n")        /* L4 */
@@ -1143,18 +1141,23 @@ EXCLUSIVE_REGRESSION_TEST(PARENTSELECTION) (RegressionTest * t, int intensity_le
     T("dest_host=pluto scheme=HTTP parent=strategy:80\n")       /* L16 */
     REBUILD
     // Test 3
-    ST(3) REINIT br(request, "numeric_host", IP(209, 131, 62, 14));
+    IpEndpoint ip;
+    ats_ip_pton(TEST_IP4_ADDR, &ip.sa);
+    ST(3) REINIT br(request, "numeric_host", &ip.sa);
   FP RE(verify(result, PARENT_SPECIFIED, "cat", 37) + verify(result, PARENT_SPECIFIED, "dog", 24), 3)
-    // Test 4
-    ST(4) REINIT br(request, "www.pilot.net");
-  FP RE(verify(result, PARENT_SPECIFIED, "pilot_net", 80), 4)
+    ats_ip_pton(TEST_IP6_ADDR, &ip.sa);
+    ST(4) REINIT br(request, "numeric_host", &ip.sa);
+  FP RE(verify(result, PARENT_SPECIFIED, "zwoop", 37) + verify(result, PARENT_SPECIFIED, "jMCg", 24), 4)
     // Test 5
-    ST(5) REINIT br(request, "www.snoopy.net");
+    ST(5) REINIT br(request, "www.pilot.net");
+  FP RE(verify(result, PARENT_SPECIFIED, "pilot_net", 80), 5)
+    // Test 6
+    ST(6) REINIT br(request, "www.snoopy.net");
   const char *snoopy_dog = "http://www.snoopy.com/";
   request->hdr->url_set(snoopy_dog, strlen(snoopy_dog));
   FP RE(verify(result, PARENT_SPECIFIED, "odie", 80) + verify(result, PARENT_SPECIFIED, "garfield", 80), 5)
-    // Test 6
-    ST(6) REINIT br(request, "a.rabbit.i.am");
+    // Test 7
+    ST(7) REINIT br(request, "a.rabbit.i.am");
   FP RE(verify(result, PARENT_SPECIFIED, "amy", 80) +
         verify(result, PARENT_SPECIFIED, "katie", 80) + verify(result, PARENT_SPECIFIED, "carissa", 771), 6)
     // Test 6+ BUGBUG needs to be fixed
@@ -1183,73 +1186,73 @@ EXCLUSIVE_REGRESSION_TEST(PARENTSELECTION) (RegressionTest * t, int intensity_le
     tbl[0] = '\0';
   T("dest_domain=rabbit.net parent=fuzzy:80,fluffy:80,furry:80,frisky:80 round_robin=strict go_direct=true\n")
     REBUILD
-    // Test 7
-    ST(7) REINIT br(request, "i.am.rabbit.net");
+    // Test 8
+    ST(8) REINIT br(request, "i.am.rabbit.net");
   FP RE(verify(result, PARENT_SPECIFIED, "fuzzy", 80), 7)
     params->markParentDown(result);
 
-  // Test 8
-  ST(8) REINIT br(request, "i.am.rabbit.net");
+  // Test 9
+  ST(9) REINIT br(request, "i.am.rabbit.net");
   FP RE(verify(result, PARENT_SPECIFIED, "fluffy", 80), 8)
-    // Test 9
-    ST(9) REINIT br(request, "i.am.rabbit.net");
-  FP RE(verify(result, PARENT_SPECIFIED, "furry", 80), 9)
     // Test 10
     ST(10) REINIT br(request, "i.am.rabbit.net");
-  FP RE(verify(result, PARENT_SPECIFIED, "frisky", 80), 10)
-    // restart the loop
+  FP RE(verify(result, PARENT_SPECIFIED, "furry", 80), 9)
     // Test 11
     ST(11) REINIT br(request, "i.am.rabbit.net");
-  FP RE(verify(result, PARENT_SPECIFIED, "fluffy", 80), 11)
+  FP RE(verify(result, PARENT_SPECIFIED, "frisky", 80), 10)
+    // restart the loop
     // Test 12
     ST(12) REINIT br(request, "i.am.rabbit.net");
-  FP RE(verify(result, PARENT_SPECIFIED, "fluffy", 80), 12)
+  FP RE(verify(result, PARENT_SPECIFIED, "fluffy", 80), 11)
     // Test 13
     ST(13) REINIT br(request, "i.am.rabbit.net");
-  FP RE(verify(result, PARENT_SPECIFIED, "furry", 80), 13)
+  FP RE(verify(result, PARENT_SPECIFIED, "fluffy", 80), 12)
     // Test 14
     ST(14) REINIT br(request, "i.am.rabbit.net");
+  FP RE(verify(result, PARENT_SPECIFIED, "furry", 80), 13)
+    // Test 15
+    ST(15) REINIT br(request, "i.am.rabbit.net");
   FP RE(verify(result, PARENT_SPECIFIED, "frisky", 80), 14)
     params->markParentDown(result);
 
   // restart the loop
 
-  // Test 15
-  ST(15) REINIT br(request, "i.am.rabbit.net");
+  // Test 16
+  ST(16) REINIT br(request, "i.am.rabbit.net");
   FP RE(verify(result, PARENT_SPECIFIED, "fluffy", 80), 15)
-    // Test 16
-    ST(16) REINIT br(request, "i.am.rabbit.net");
-  FP RE(verify(result, PARENT_SPECIFIED, "fluffy", 80), 16)
     // Test 17
     ST(17) REINIT br(request, "i.am.rabbit.net");
-  FP RE(verify(result, PARENT_SPECIFIED, "furry", 80), 17)
+  FP RE(verify(result, PARENT_SPECIFIED, "fluffy", 80), 16)
     // Test 18
     ST(18) REINIT br(request, "i.am.rabbit.net");
-  FP RE(verify(result, PARENT_SPECIFIED, "fluffy", 80), 18)
-    // restart the loop
+  FP RE(verify(result, PARENT_SPECIFIED, "furry", 80), 17)
     // Test 19
     ST(19) REINIT br(request, "i.am.rabbit.net");
-  FP RE(verify(result, PARENT_SPECIFIED, "fluffy", 80), 19)
+  FP RE(verify(result, PARENT_SPECIFIED, "fluffy", 80), 18)
+    // restart the loop
     // Test 20
     ST(20) REINIT br(request, "i.am.rabbit.net");
-  FP RE(verify(result, PARENT_SPECIFIED, "fluffy", 80), 20)
+  FP RE(verify(result, PARENT_SPECIFIED, "fluffy", 80), 19)
     // Test 21
     ST(21) REINIT br(request, "i.am.rabbit.net");
+  FP RE(verify(result, PARENT_SPECIFIED, "fluffy", 80), 20)
+    // Test 22
+    ST(22) REINIT br(request, "i.am.rabbit.net");
   FP RE(verify(result, PARENT_SPECIFIED, "furry", 80), 21)
     params->markParentDown(result);
 
-  // Test 22 - 31
-  for (i = 0; i < 10; i++) {
-    ST(22 + i) REINIT br(request, "i.am.rabbit.net");
-    FP RE(verify(result, PARENT_SPECIFIED, "fluffy", 80), 22 + i)
+  // Test 23 - 32
+  for (i = 23; i < 33; i++) {
+    ST(i) REINIT br(request, "i.am.rabbit.net");
+    FP RE(verify(result, PARENT_SPECIFIED, "fluffy", 80), i)
   }
 
   params->markParentDown(result);       // now they're all down
 
-  // Test 32 - 131
-  for (i = 0; i < 100; i++) {
-    ST(32 + i) REINIT br(request, "i.am.rabbit.net");
-    FP RE(verify(result, PARENT_DIRECT, 0, 0), 32 + i)
+  // Test 33 - 132
+  for (i = 33; i < 133; i++) {
+    ST(i) REINIT br(request, "i.am.rabbit.net");
+    FP RE(verify(result, PARENT_DIRECT, 0, 0), i)
   }
 
   // sleep(5); // parents should come back up; they don't
@@ -1258,19 +1261,19 @@ EXCLUSIVE_REGRESSION_TEST(PARENTSELECTION) (RegressionTest * t, int intensity_le
   // Fix: The following tests failed because
   // br() should set xact_start correctly instead of 0.
 
-  // Test 132 - 631
-  for (i = 0; i < 40; i++) {
-    ST(132 + i) REINIT br(request, "i.am.rabbit.net");
+  // Test 133 - 172
+  for (i = 133; i < 173; i++) {
+    ST(i) REINIT br(request, "i.am.rabbit.net");
     FP sleep(1);
-    switch ((i + 1) % 4) {
+    switch (i % 4) {
     case 0:
-      RE(verify(result, PARENT_SPECIFIED, "fuzzy", 80), 132 + i) break;
+      RE(verify(result, PARENT_SPECIFIED, "fuzzy", 80), i) break;
     case 1:
-      RE(verify(result, PARENT_SPECIFIED, "fluffy", 80), 132 + i) break;
+      RE(verify(result, PARENT_SPECIFIED, "fluffy", 80), i) break;
     case 2:
-      RE(verify(result, PARENT_SPECIFIED, "furry", 80), 132 + i) break;
+      RE(verify(result, PARENT_SPECIFIED, "furry", 80), i) break;
     case 3:
-      RE(verify(result, PARENT_SPECIFIED, "frisky", 80), 132 + i) break;
+      RE(verify(result, PARENT_SPECIFIED, "frisky", 80), i) break;
     default:
       ink_assert(0);
     }
@@ -1293,17 +1296,15 @@ verify(ParentResult * r, ParentResultType e, const char *h, int p)
 
 // br creates an HttpRequestData object
 void
-br(HttpRequestData * h, const char *os_hostname, int dest_ip)
+br(HttpRequestData * h, const char *os_hostname, sockaddr const* dest_ip)
 {
-  int hostname_len = strlen(os_hostname) + 1;
-
   h->hdr = new HTTPHdr();
   h->hdr->create(HTTP_TYPE_REQUEST);
-  h->hostname_str = (char *) malloc(hostname_len);
-  ink_strncpy(h->hostname_str, os_hostname, hostname_len);
+  h->hostname_str = (char *)ats_strdup(os_hostname);
   h->xact_start = time(NULL);
-  h->src_ip = 0;
-  h->dest_ip = dest_ip;
+  ink_zero(h->src_ip);
+  ink_zero(h->dest_ip);
+  ats_ip_copy(&h->dest_ip.sa, dest_ip);
   h->incoming_port = 80;
   h->api_info = new _HttpApiInfo();
 }
