@@ -240,7 +240,7 @@ PluginVC::do_io_read(Continuation * c, int64_t nbytes, MIOBuffer * buf)
   read_state.vio.vc_server = (VConnection *) this;
   read_state.vio.op = VIO::READ;
 
-  Debug("pvc", "[%u] %s: do_io_read for %d bytes", PVC_ID, PVC_TYPE, nbytes);
+  Debug("pvc", "[%u] %s: do_io_read for %"PRId64" bytes", PVC_ID, PVC_TYPE, nbytes);
 
   // Since reentrant callbacks are not allowed on from do_io
   //   functions schedule ourselves get on a different stack
@@ -273,7 +273,7 @@ PluginVC::do_io_write(Continuation * c, int64_t nbytes, IOBufferReader * abuffer
   write_state.vio.vc_server = (VConnection *) this;
   write_state.vio.op = VIO::WRITE;
 
-  Debug("pvc", "[%u] %s: do_io_write for %d bytes", PVC_ID, PVC_TYPE, nbytes);
+  Debug("pvc", "[%u] %s: do_io_write for %"PRId64" bytes", PVC_ID, PVC_TYPE, nbytes);
 
   // Since reentrant callbacks are not allowed on from do_io
   //   functions schedule ourselves get on a different stack
@@ -504,7 +504,7 @@ PluginVC::process_write_side(bool other_side_call)
   int64_t bytes_avail = reader->read_avail();
   int64_t act_on = MIN(bytes_avail, ntodo);
 
-  Debug("pvc", "[%u] %s: process_write_side; act_on %d", PVC_ID, PVC_TYPE, act_on);
+  Debug("pvc", "[%u] %s: process_write_side; act_on %"PRId64"", PVC_ID, PVC_TYPE, act_on);
 
   if (other_side->closed || other_side->read_state.shutdown) {
     write_state.vio._cont->handleEvent(VC_EVENT_ERROR, &write_state.vio);
@@ -540,7 +540,7 @@ PluginVC::process_write_side(bool other_side_call)
 
   write_state.vio.ndone += added;
 
-  Debug("pvc", "[%u] %s: process_write_side; added %d", PVC_ID, PVC_TYPE, added);
+  Debug("pvc", "[%u] %s: process_write_side; added %"PRId64"", PVC_ID, PVC_TYPE, added);
 
   if (write_state.vio.ntodo() == 0) {
     write_state.vio._cont->handleEvent(VC_EVENT_WRITE_COMPLETE, &write_state.vio);
@@ -619,7 +619,7 @@ PluginVC::process_read_side(bool other_side_call)
   int64_t bytes_avail = core_reader->read_avail();
   int64_t act_on = MIN(bytes_avail, ntodo);
 
-  Debug("pvc", "[%u] %s: process_read_side; act_on %d", PVC_ID, PVC_TYPE, act_on);
+  Debug("pvc", "[%u] %s: process_read_side; act_on %"PRId64"", PVC_ID, PVC_TYPE, act_on);
 
   if (act_on <= 0) {
     if (other_side->closed || other_side->write_state.shutdown) {
@@ -652,7 +652,7 @@ PluginVC::process_read_side(bool other_side_call)
 
   read_state.vio.ndone += added;
 
-  Debug("pvc", "[%u] %s: process_read_side; added %d", PVC_ID, PVC_TYPE, added);
+  Debug("pvc", "[%u] %s: process_read_side; added %"PRId64"", PVC_ID, PVC_TYPE, added);
 
   if (read_state.vio.ntodo() == 0) {
     read_state.vio._cont->handleEvent(VC_EVENT_READ_COMPLETE, &read_state.vio);
@@ -879,9 +879,11 @@ void
 PluginVC::set_local_addr()
 {
   if (vc_type == PLUGIN_VC_ACTIVE) {
-    local_addr = core_obj->active_addr_struct;
+    ats_ip_copy(&local_addr, &core_obj->active_addr_struct);
+//    local_addr = core_obj->active_addr_struct;
   } else {
-    local_addr = core_obj->passive_addr_struct;
+    ats_ip_copy(&local_addr, &core_obj->passive_addr_struct);
+//    local_addr = core_obj->passive_addr_struct;
   }
 }
 
@@ -889,12 +891,23 @@ void
 PluginVC::set_remote_addr()
 {
   if (vc_type == PLUGIN_VC_ACTIVE) {
-    remote_addr = core_obj->passive_addr_struct;
+    ats_ip_copy(&remote_addr, &core_obj->passive_addr_struct);
   } else {
-    remote_addr = core_obj->active_addr_struct;
+    ats_ip_copy(&remote_addr, &core_obj->active_addr_struct);
   }
 }
 
+int
+PluginVC::set_tcp_init_cwnd(int init_cwnd)
+{
+  return -1;
+}
+
+void
+PluginVC::apply_options()
+{
+  // do nothing
+}
 
 bool
 PluginVC::get_data(int id, void *data)
@@ -986,14 +999,14 @@ PluginVCCore::init()
   a_to_p_buffer = new_MIOBuffer(BUFFER_SIZE_INDEX_32K);
   a_to_p_reader = a_to_p_buffer->alloc_reader();
 
-  Debug("pvc", "[%u] Created PluginVCCore at 0x%X, active 0x%X, passive 0x%X", id, this, &active_vc, &passive_vc);
+  Debug("pvc", "[%u] Created PluginVCCore at %p, active %p, passive %p", id, this, &active_vc, &passive_vc);
 }
 
 void
 PluginVCCore::destroy()
 {
 
-  Debug("pvc", "[%u] Destroying PluginVCCore at 0x%X", id, this);
+  Debug("pvc", "[%u] Destroying PluginVCCore at %p", id, this);
 
   ink_assert(active_vc.closed == true || !connected);
   active_vc.mutex = NULL;
@@ -1138,17 +1151,27 @@ PluginVCCore::kill_no_connect()
 }
 
 void
-PluginVCCore::set_passive_addr(uint32_t ip, int port)
+PluginVCCore::set_passive_addr(in_addr_t ip, int port)
 {
-  ((struct sockaddr_in *)&(passive_addr_struct))->sin_addr.s_addr = htonl(ip);
-  ((struct sockaddr_in *)&(passive_addr_struct))->sin_port = htons(port);
+  ats_ip4_set(&passive_addr_struct, htonl(ip), htons(port));
 }
 
 void
-PluginVCCore::set_active_addr(uint32_t ip, int port)
+PluginVCCore::set_passive_addr(sockaddr const* ip)
 {
-  ((struct sockaddr_in *)&(active_addr_struct))->sin_addr.s_addr = htonl(ip);
-  ((struct sockaddr_in *)&(active_addr_struct))->sin_port = htons(port);
+  passive_addr_struct.assign(ip);
+}
+
+void
+PluginVCCore::set_active_addr(in_addr_t ip, int port)
+{
+  ats_ip4_set(&active_addr_struct, htonl(ip), htons(port));
+}
+
+void
+PluginVCCore::set_active_addr(sockaddr const* ip)
+{
+  active_addr_struct.assign(ip);
 }
 
 void
@@ -1163,6 +1186,14 @@ PluginVCCore::set_active_data(void *data)
   active_data = data;
 }
 
+void
+PluginVCCore::set_transparent(bool passive_side, bool active_side)
+{
+  passive_vc.set_is_transparent(passive_side);
+  active_vc.set_is_transparent(active_side);
+}
+
+
 /*************************************************************
  *
  *   REGRESSION TEST STUFF
