@@ -43,7 +43,6 @@
 #include "RemapProcessor.h"
 #include "UrlRewrite.h"
 #include "UrlMapping.h"
-#include "StringHash.h"
 #include "ink_unused.h"      /* MAGIC_EDITING_TAG */
 
 /** Time till we free the old stuff after a reconfiguration. */
@@ -101,13 +100,11 @@ init_reverse_proxy()
 // odd ball configs, for example if you use the "CONNECT" method, or if you set
 // set proxy.config.url_remap.url_remap_mode to "2" (which is a completely undocumented "feature").
 bool
-request_url_remap(HttpTransact::State * s, HTTPHdr * request_header, char **redirect_url, char **orig_url,
-                  unsigned int filter_mask)
+request_url_remap(HttpTransact::State * s, HTTPHdr * request_header, char **redirect_url, unsigned int filter_mask)
 {
   NOWARN_UNUSED(s);
   NOWARN_UNUSED(request_header);
   NOWARN_UNUSED(redirect_url);
-  NOWARN_UNUSED(orig_url);
   NOWARN_UNUSED(filter_mask);
   return false;
   // return rewrite_table ? rewrite_table->Remap(s, request_header, redirect_url, orig_url, tag, filter_mask) : false;
@@ -118,9 +115,9 @@ request_url_remap(HttpTransact::State * s, HTTPHdr * request_header, char **redi
    according to the rules in remap.config.
 */
 mapping_type
-request_url_remap_redirect(HTTPHdr *request_header, URL *redirect_url, char **orig_url)
+request_url_remap_redirect(HTTPHdr *request_header, URL *redirect_url)
 {
-  return rewrite_table ? rewrite_table->Remap_redirect(request_header, redirect_url, orig_url) : NONE;
+  return rewrite_table ? rewrite_table->Remap_redirect(request_header, redirect_url) : NONE;
 }
 
 bool
@@ -198,8 +195,8 @@ reloadUrlRewrite()
   } else {
     static const char* msg = "failed to reload remap.config, not replacing!";
     delete newTable;
-    Debug("url_rewrite", msg);
-    Warning(msg);
+    Debug("url_rewrite", "%s", msg);
+    Warning("%s", msg);
   }
 }
 
