@@ -21,6 +21,7 @@
   limitations under the License.
  */
 
+#include "ink_port.h"
 #include "libts.h"
 #include <assert.h>
 #include <stdio.h>
@@ -162,7 +163,7 @@ is_digit(char c)
  ***********************************************************************/
 
 void
-http_hdr_adjust(HTTPHdrImpl * hdrp, int32_t offset, int32_t length, int32_t delta)
+http_hdr_adjust(HTTPHdrImpl *hdrp, int32_t offset, int32_t length, int32_t delta)
 {
   NOWARN_UNUSED(hdrp);
   NOWARN_UNUSED(offset);
@@ -294,7 +295,7 @@ http_init()
   -------------------------------------------------------------------------*/
 
 HTTPHdrImpl *
-http_hdr_create(HdrHeap * heap, HTTPType polarity)
+http_hdr_create(HdrHeap *heap, HTTPType polarity)
 {
   HTTPHdrImpl *hh;
 
@@ -307,7 +308,7 @@ http_hdr_create(HdrHeap * heap, HTTPType polarity)
   -------------------------------------------------------------------------*/
 
 void
-http_hdr_init(HdrHeap * heap, HTTPHdrImpl * hh, HTTPType polarity)
+http_hdr_init(HdrHeap *heap, HTTPHdrImpl *hh, HTTPType polarity)
 {
   memset(&(hh->u), 0, sizeof(hh->u));
   hh->m_polarity = polarity;
@@ -323,7 +324,7 @@ http_hdr_init(HdrHeap * heap, HTTPHdrImpl * hh, HTTPType polarity)
   -------------------------------------------------------------------------*/
 
 void
-http_hdr_copy_onto(HTTPHdrImpl * s_hh, HdrHeap * s_heap, HTTPHdrImpl * d_hh, HdrHeap * d_heap, bool inherit_strs)
+http_hdr_copy_onto(HTTPHdrImpl *s_hh, HdrHeap *s_heap, HTTPHdrImpl *d_hh, HdrHeap *d_heap, bool inherit_strs)
 {
   MIMEHdrImpl *s_mh, *d_mh;
   URLImpl *s_url, *d_url;
@@ -363,7 +364,7 @@ http_hdr_copy_onto(HTTPHdrImpl * s_hh, HdrHeap * s_heap, HTTPHdrImpl * d_hh, Hdr
   -------------------------------------------------------------------------*/
 
 HTTPHdrImpl *
-http_hdr_clone(HTTPHdrImpl * s_hh, HdrHeap * s_heap, HdrHeap * d_heap)
+http_hdr_clone(HTTPHdrImpl *s_hh, HdrHeap *s_heap, HdrHeap *d_heap)
 {
   HTTPHdrImpl *d_hh;
 
@@ -418,7 +419,7 @@ http_version_print(int32_t version, char *buf, int bufsize, int *bufindex, int *
   -------------------------------------------------------------------------*/
 
 int
-http_hdr_print(HdrHeap * heap, HTTPHdrImpl * hdr, char *buf, int bufsize, int *bufindex, int *dumpoffset)
+http_hdr_print(HdrHeap *heap, HTTPHdrImpl *hdr, char *buf, int bufsize, int *bufindex, int *dumpoffset)
 {
 #define TRY(x)  if (!x) return 0
 
@@ -559,12 +560,12 @@ http_hdr_print(HdrHeap * heap, HTTPHdrImpl * hdr, char *buf, int bufsize, int *b
   -------------------------------------------------------------------------*/
 
 void
-http_hdr_describe(HdrHeapObjImpl * raw, bool recurse)
+http_hdr_describe(HdrHeapObjImpl *raw, bool recurse)
 {
   HTTPHdrImpl *obj = (HTTPHdrImpl *) raw;
 
   if (obj->m_polarity == HTTP_TYPE_REQUEST) {
-    Debug("http", "[TYPE: REQ, V: %04X, URL: 0x%04X, METHOD: \"%.*s\", METHOD_LEN: %d, FIELDS: 0x%04X]\n",
+    Debug("http", "[TYPE: REQ, V: %04X, URL: %p, METHOD: \"%.*s\", METHOD_LEN: %d, FIELDS: %p]\n",
           obj->m_version, obj->u.req.m_url_impl,
           obj->u.req.m_len_method, (obj->u.req.m_ptr_method ? obj->u.req.m_ptr_method : "NULL"),
           obj->u.req.m_len_method, obj->m_fields_impl);
@@ -575,7 +576,7 @@ http_hdr_describe(HdrHeapObjImpl * raw, bool recurse)
         obj_describe(obj->m_fields_impl, recurse);
     }
   } else {
-    Debug("http", "[TYPE: RSP, V: %04X, STATUS: %d, REASON: \"%.*s\", REASON_LEN: %d, FIELDS: 0x%04X]\n",
+    Debug("http", "[TYPE: RSP, V: %04X, STATUS: %d, REASON: \"%.*s\", REASON_LEN: %d, FIELDS: %p]\n",
           obj->m_version, obj->u.resp.m_status,
           obj->u.resp.m_len_reason, (obj->u.resp.m_ptr_reason ? obj->u.resp.m_ptr_reason : "NULL"),
           obj->u.resp.m_len_reason, obj->m_fields_impl);
@@ -590,7 +591,7 @@ http_hdr_describe(HdrHeapObjImpl * raw, bool recurse)
   -------------------------------------------------------------------------*/
 
 int
-http_hdr_length_get(HTTPHdrImpl * hdr)
+http_hdr_length_get(HTTPHdrImpl *hdr)
 {
   int length = 0;
 
@@ -641,7 +642,7 @@ http_hdr_length_get(HTTPHdrImpl * hdr)
   -------------------------------------------------------------------------*/
 
 void
-http_hdr_type_set(HTTPHdrImpl * hh, HTTPType type)
+http_hdr_type_set(HTTPHdrImpl *hh, HTTPType type)
 {
   hh->m_polarity = type;
 }
@@ -650,7 +651,7 @@ http_hdr_type_set(HTTPHdrImpl * hh, HTTPType type)
   -------------------------------------------------------------------------*/
 
 void
-http_hdr_version_set(HTTPHdrImpl * hh, int32_t ver)
+http_hdr_version_set(HTTPHdrImpl *hh, int32_t ver)
 {
   hh->m_version = ver;
 }
@@ -659,7 +660,7 @@ http_hdr_version_set(HTTPHdrImpl * hh, int32_t ver)
   -------------------------------------------------------------------------*/
 
 const char *
-http_hdr_method_get(HTTPHdrImpl * hh, int *length)
+http_hdr_method_get(HTTPHdrImpl *hh, int *length)
 {
   const char *str;
 
@@ -680,8 +681,8 @@ http_hdr_method_get(HTTPHdrImpl * hh, int *length)
   -------------------------------------------------------------------------*/
 
 void
-http_hdr_method_set(HdrHeap * heap,
-                    HTTPHdrImpl * hh, const char *method, int16_t method_wks_idx, int method_length, bool must_copy)
+http_hdr_method_set(HdrHeap *heap, HTTPHdrImpl *hh, const char *method, int16_t method_wks_idx, int method_length,
+                    bool must_copy)
 {
   ink_debug_assert(hh->m_polarity == HTTP_TYPE_REQUEST);
 
@@ -693,7 +694,7 @@ http_hdr_method_set(HdrHeap * heap,
   -------------------------------------------------------------------------*/
 
 void
-http_hdr_url_set(HdrHeap * heap, HTTPHdrImpl * hh, URLImpl * url)
+http_hdr_url_set(HdrHeap *heap, HTTPHdrImpl *hh, URLImpl *url)
 {
   ink_debug_assert(hh->m_polarity == HTTP_TYPE_REQUEST);
   if (hh->u.req.m_url_impl != url) {
@@ -709,7 +710,7 @@ http_hdr_url_set(HdrHeap * heap, HTTPHdrImpl * hh, URLImpl * url)
   -------------------------------------------------------------------------*/
 
 void
-http_hdr_status_set(HTTPHdrImpl * hh, HTTPStatus status)
+http_hdr_status_set(HTTPHdrImpl *hh, HTTPStatus status)
 {
   ink_debug_assert(hh->m_polarity == HTTP_TYPE_RESPONSE);
   hh->u.resp.m_status = status;
@@ -719,7 +720,7 @@ http_hdr_status_set(HTTPHdrImpl * hh, HTTPStatus status)
   -------------------------------------------------------------------------*/
 
 const char *
-http_hdr_reason_get(HTTPHdrImpl * hh, int *length)
+http_hdr_reason_get(HTTPHdrImpl *hh, int *length)
 {
   ink_debug_assert(hh->m_polarity == HTTP_TYPE_RESPONSE);
   *length = hh->u.resp.m_len_reason;
@@ -730,7 +731,7 @@ http_hdr_reason_get(HTTPHdrImpl * hh, int *length)
   -------------------------------------------------------------------------*/
 
 void
-http_hdr_reason_set(HdrHeap * heap, HTTPHdrImpl * hh, const char *value, int length, bool must_copy)
+http_hdr_reason_set(HdrHeap *heap, HTTPHdrImpl *hh, const char *value, int length, bool must_copy)
 {
   ink_debug_assert(hh->m_polarity == HTTP_TYPE_RESPONSE);
   mime_str_u16_set(heap, value, length, &(hh->u.resp.m_ptr_reason), &(hh->u.resp.m_len_reason), must_copy);
@@ -740,99 +741,93 @@ http_hdr_reason_set(HdrHeap * heap, HTTPHdrImpl * hh, const char *value, int len
   -------------------------------------------------------------------------*/
 
 const char *
-http_hdr_reason_lookup(HTTPStatus status)
+http_hdr_reason_lookup(unsigned status)
 {
+#define HTTP_STATUS_ENTRY(value, reason) \
+    case value: return #reason
+
   switch (status) {
-  case HTTP_STATUS_NONE:
-    return "None";
-  case HTTP_STATUS_CONTINUE:
-    return "Continue";
-  case HTTP_STATUS_SWITCHING_PROTOCOL:
-    return "Switching Protocol";
-  case HTTP_STATUS_OK:
-    return "Ok";
-  case HTTP_STATUS_CREATED:
-    return "Created";
-  case HTTP_STATUS_ACCEPTED:
-    return "Accepted";
-  case HTTP_STATUS_NON_AUTHORITATIVE_INFORMATION:
-    return "Non Authoritative Information";
-  case HTTP_STATUS_NO_CONTENT:
-    return "No Content";
-  case HTTP_STATUS_RESET_CONTENT:
-    return "Reset Content";
-  case HTTP_STATUS_PARTIAL_CONTENT:
-    return "Partial Content";
-  case HTTP_STATUS_MULTIPLE_CHOICES:
-    return "Multiple Choices";
-  case HTTP_STATUS_MOVED_PERMANENTLY:
-    return "Moved Permanently";
-  case HTTP_STATUS_MOVED_TEMPORARILY:
-    return "Moved Temporarily";
-  case HTTP_STATUS_SEE_OTHER:
-    return "See Other";
-  case HTTP_STATUS_NOT_MODIFIED:
-    return "Not Modified";
-  case HTTP_STATUS_USE_PROXY:
-    return "Use Proxy";
-  case HTTP_STATUS_TEMPORARY_REDIRECT:
-    return "Temporary Redirect";
-  case HTTP_STATUS_BAD_REQUEST:
-    return "Bad Request";
-  case HTTP_STATUS_UNAUTHORIZED:
-    return "Unauthorized";
-  case HTTP_STATUS_PAYMENT_REQUIRED:
-    return "Payment Required";
-  case HTTP_STATUS_FORBIDDEN:
-    return "Forbidden";
-  case HTTP_STATUS_NOT_FOUND:
-    return "Not Found";
-  case HTTP_STATUS_METHOD_NOT_ALLOWED:
-    return "Method Not Allowed";
-  case HTTP_STATUS_NOT_ACCEPTABLE:
-    return "Not Acceptable";
-  case HTTP_STATUS_PROXY_AUTHENTICATION_REQUIRED:
-    return "Proxy Authentication Required";
-  case HTTP_STATUS_REQUEST_TIMEOUT:
-    return "Request Timeout";
-  case HTTP_STATUS_CONFLICT:
-    return "Conflict";
-  case HTTP_STATUS_GONE:
-    return "Gone";
-  case HTTP_STATUS_LENGTH_REQUIRED:
-    return "Length Required";
-  case HTTP_STATUS_PRECONDITION_FAILED:
-    return "Precondition Failed";
-  case HTTP_STATUS_RANGE_NOT_SATISFIABLE:
-    return "Requested Range Not Satisfiable";
-  case HTTP_STATUS_REQUEST_ENTITY_TOO_LARGE:
-    return "Request Entity Too Large";
-  case HTTP_STATUS_REQUEST_URI_TOO_LONG:
-    return "Request URI Too Long";
-  case HTTP_STATUS_UNSUPPORTED_MEDIA_TYPE:
-    return "Unsupported Media Type";
-  case HTTP_STATUS_INTERNAL_SERVER_ERROR:
-    return "Internal Server Error";
-  case HTTP_STATUS_NOT_IMPLEMENTED:
-    return "Not Implemented";
-  case HTTP_STATUS_BAD_GATEWAY:
-    return "Bad Gateway";
-  case HTTP_STATUS_SERVICE_UNAVAILABLE:
-    return "Service Unavailable";
-  case HTTP_STATUS_GATEWAY_TIMEOUT:
-    return "Gateway Timeout";
-  case HTTP_STATUS_HTTPVER_NOT_SUPPORTED:
-    return "HTTP Version Not Supported";
+    HTTP_STATUS_ENTRY(0, None);                                       // TS_HTTP_STATUS_NONE
+    HTTP_STATUS_ENTRY(100, Continue);                                 // [RFC2616]
+    HTTP_STATUS_ENTRY(101, Switching Protocols);                      // [RFC2616]
+    HTTP_STATUS_ENTRY(102, Processing);                               // [RFC2518]
+    // 103-199 Unassigned
+    HTTP_STATUS_ENTRY(200, OK);                                       // [RFC2616]
+    HTTP_STATUS_ENTRY(201, Created);                                  // [RFC2616]
+    HTTP_STATUS_ENTRY(202, Accepted);                                 // [RFC2616]
+    HTTP_STATUS_ENTRY(203, Non-Authoritative Information);            // [RFC2616]
+    HTTP_STATUS_ENTRY(204, No Content);                               // [RFC2616]
+    HTTP_STATUS_ENTRY(205, Reset Content);                            // [RFC2616]
+    HTTP_STATUS_ENTRY(206, Partial Content);                          // [RFC2616]
+    HTTP_STATUS_ENTRY(207, Multi-Status);                             // [RFC4918]
+    HTTP_STATUS_ENTRY(208, Already Reported);                         // [RFC5842]
+    // 209-225 Unassigned
+    HTTP_STATUS_ENTRY(226, IM Used);                                  // [RFC3229]
+    // 227-299 Unassigned
+    HTTP_STATUS_ENTRY(300, Multiple Choices);                         // [RFC2616]
+    HTTP_STATUS_ENTRY(301, Moved Permanently);                        // [RFC2616]
+    HTTP_STATUS_ENTRY(302, Found);                                    // [RFC2616]
+    HTTP_STATUS_ENTRY(303, See Other);                                // [RFC2616]
+    HTTP_STATUS_ENTRY(304, Not Modified);                             // [RFC2616]
+    HTTP_STATUS_ENTRY(305, Use Proxy);                                // [RFC2616]
+    // 306 Reserved                                                   // [RFC2616]
+    HTTP_STATUS_ENTRY(307, Temporary Redirect);                       // [RFC2616]
+    HTTP_STATUS_ENTRY(308, Permanent Redirect);                       // [RFC-reschke-http-status-308-07]
+    // 309-399 Unassigned
+    HTTP_STATUS_ENTRY(400, Bad Request);                              // [RFC2616]
+    HTTP_STATUS_ENTRY(401, Unauthorized);                             // [RFC2616]
+    HTTP_STATUS_ENTRY(402, Payment Required);                         // [RFC2616]
+    HTTP_STATUS_ENTRY(403, Forbidden);                                // [RFC2616]
+    HTTP_STATUS_ENTRY(404, Not Found);                                // [RFC2616]
+    HTTP_STATUS_ENTRY(405, Method Not Allowed);                       // [RFC2616]
+    HTTP_STATUS_ENTRY(406, Not Acceptable);                           // [RFC2616]
+    HTTP_STATUS_ENTRY(407, Proxy Authentication Required);            // [RFC2616]
+    HTTP_STATUS_ENTRY(408, Request Timeout);                          // [RFC2616]
+    HTTP_STATUS_ENTRY(409, Conflict);                                 // [RFC2616]
+    HTTP_STATUS_ENTRY(410, Gone);                                     // [RFC2616]
+    HTTP_STATUS_ENTRY(411, Length Required);                          // [RFC2616]
+    HTTP_STATUS_ENTRY(412, Precondition Failed);                      // [RFC2616]
+    HTTP_STATUS_ENTRY(413, Request Entity Too Large);                 // [RFC2616]
+    HTTP_STATUS_ENTRY(414, Request-URI Too Long);                     // [RFC2616]
+    HTTP_STATUS_ENTRY(415, Unsupported Media Type);                   // [RFC2616]
+    HTTP_STATUS_ENTRY(416, Requested Range Not Satisfiable);          // [RFC2616]
+    HTTP_STATUS_ENTRY(417, Expectation Failed);                       // [RFC2616]
+    HTTP_STATUS_ENTRY(422, Unprocessable Entity);                     // [RFC4918]
+    HTTP_STATUS_ENTRY(423, Locked);                                   // [RFC4918]
+    HTTP_STATUS_ENTRY(424, Failed Dependency);                        // [RFC4918]
+    // 425 Reserved                                                   // [RFC2817]
+    HTTP_STATUS_ENTRY(426, Upgrade Required);                         // [RFC2817]
+    // 427 Unassigned
+    HTTP_STATUS_ENTRY(428, Precondition Required);                    // [RFC6585]
+    HTTP_STATUS_ENTRY(429, Too Many Requests);                        // [RFC6585]
+    // 430 Unassigned
+    HTTP_STATUS_ENTRY(431, Request Header Fields Too Large);          // [RFC6585]
+    // 432-499 Unassigned
+    HTTP_STATUS_ENTRY(500, Internal Server Error);                    // [RFC2616]
+    HTTP_STATUS_ENTRY(501, Not Implemented);                          // [RFC2616]
+    HTTP_STATUS_ENTRY(502, Bad Gateway);                              // [RFC2616]
+    HTTP_STATUS_ENTRY(503, Service Unavailable);                      // [RFC2616]
+    HTTP_STATUS_ENTRY(504, Gateway Timeout);                          // [RFC2616]
+    HTTP_STATUS_ENTRY(505, HTTP Version Not Supported);               // [RFC2616]
+    HTTP_STATUS_ENTRY(506, Variant Also Negotiates);                  // [RFC2295]
+    HTTP_STATUS_ENTRY(507, Insufficient Storage);                     // [RFC4918]
+    HTTP_STATUS_ENTRY(508, Loop Detected);                            // [RFC5842]
+    // 509 Unassigned
+    HTTP_STATUS_ENTRY(510, Not Extended);                             // [RFC2774]
+    HTTP_STATUS_ENTRY(511, Network Authentication Required);          // [RFC6585]
+    // 512-599 Unassigned
   }
 
-  return (NULL);
+#undef HTTP_STATUS_ENTRY
+
+  return NULL;
 }
 
 /*-------------------------------------------------------------------------
   -------------------------------------------------------------------------*/
 
 void
-_http_parser_init(HTTPParser * parser)
+_http_parser_init(HTTPParser *parser)
 {
   parser->m_parsing_http = true;
 }
@@ -843,14 +838,14 @@ _http_parser_init(HTTPParser * parser)
 //////////////////////////////////////////////////////
 
 void
-http_parser_init(HTTPParser * parser)
+http_parser_init(HTTPParser *parser)
 {
   _http_parser_init(parser);
   mime_parser_init(&parser->m_mime_parser);
 }
 
 void
-http_parser_clear(HTTPParser * parser)
+http_parser_clear(HTTPParser *parser)
 {
   _http_parser_init(parser);
   mime_parser_clear(&parser->m_mime_parser);
@@ -876,9 +871,8 @@ http_parser_clear(HTTPParser * parser)
 // NOTE: end is ONE CHARACTER PAST end of string!
 
 MIMEParseResult
-http_parser_parse_req(HTTPParser * parser,
-                      HdrHeap * heap,
-                      HTTPHdrImpl * hh, const char **start, const char *end, bool must_copy_strings, bool eof)
+http_parser_parse_req(HTTPParser *parser, HdrHeap *heap, HTTPHdrImpl *hh, const char **start, const char *end,
+                      bool must_copy_strings, bool eof)
 {
   if (parser->m_parsing_http) {
     MIMEScanner *scanner = &parser->m_mime_parser.m_scanner;
@@ -901,6 +895,10 @@ http_parser_parse_req(HTTPParser * parser,
   start:
     hh->m_polarity = HTTP_TYPE_REQUEST;
 
+    // Make sure the line is not longer than 64K
+    if (scanner->m_line_length >= UINT16_MAX)
+      return PARSE_ERROR;
+
     err = mime_scanner_get(scanner, start, real_end, &line_start, &end, &line_is_real, eof, MIME_SCANNER_TYPE_LINE);
     if (err < 0)
       return err;
@@ -912,8 +910,18 @@ http_parser_parse_req(HTTPParser * parser,
       return err;
 
     cur = line_start;
+    ink_assert((end - cur) >= 0);
+    ink_assert((end - cur) < UINT16_MAX);
+
     must_copy_strings = (must_copy_strings || (!line_is_real));
 
+#if ENABLE_SAVE_ORIGINAL_REQUEST
+    mime_str_u16_set(heap, line_start, strlen(line_start),
+                     &(hh->u.req.m_url_impl->the_request),
+                     &(hh->u.req.m_url_impl->the_request_len),
+                     must_copy_strings);
+#endif
+
 #if (ENABLE_PARSER_FAST_PATHS)
     // first try fast path
     if (end - cur >= 16) {
@@ -1086,9 +1094,8 @@ http_parser_parse_req(HTTPParser * parser,
   -------------------------------------------------------------------------*/
 
 MIMEParseResult
-http_parser_parse_resp(HTTPParser * parser,
-                       HdrHeap * heap,
-                       HTTPHdrImpl * hh, const char **start, const char *end, bool must_copy_strings, bool eof)
+http_parser_parse_resp(HTTPParser *parser, HdrHeap *heap, HTTPHdrImpl *hh, const char **start, const char *end,
+                       bool must_copy_strings, bool eof)
 {
   if (parser->m_parsing_http) {
     MIMEScanner *scanner = &parser->m_mime_parser.m_scanner;
@@ -1111,6 +1118,10 @@ http_parser_parse_resp(HTTPParser * parser,
 
     hh->m_polarity = HTTP_TYPE_RESPONSE;
 
+    // Make sure the line is not longer than 64K
+    if (scanner->m_line_length >= UINT16_MAX)
+      return PARSE_ERROR;
+
     err = mime_scanner_get(scanner, start, real_end, &line_start, &end, &line_is_real, eof, MIME_SCANNER_TYPE_LINE);
     if (err < 0)
       return err;
@@ -1118,6 +1129,9 @@ http_parser_parse_resp(HTTPParser * parser,
       return err;
 
     cur = line_start;
+    ink_assert((end - cur) >= 0);
+    ink_assert((end - cur) < UINT16_MAX);
+
     must_copy_strings = (must_copy_strings || (!line_is_real));
 
 #if (ENABLE_PARSER_FAST_PATHS)
@@ -1320,7 +1334,7 @@ http_parse_version(const char *start, const char *end)
   -------------------------------------------------------------------------*/
 
 static char *
-http_str_store(Arena * arena, const char *str, int length)
+http_str_store(Arena *arena, const char *str, int length)
 {
   const char *wks;
   int idx = hdrtoken_tokenize(str, length, &wks);
@@ -1423,7 +1437,7 @@ http_parse_qvalue(const char *&buf, int &len)
   -------------------------------------------------------------------------*/
 
 HTTPValTE *
-http_parse_te(const char *buf, int len, Arena * arena)
+http_parse_te(const char *buf, int len, Arena *arena)
 {
   HTTPValTE *val;
   const char *s;
@@ -1448,28 +1462,27 @@ void
 HTTPHdr::_fill_target_cache() const
 {
   URL* url = this->url_get();
+  char const* port_ptr;
+
   m_target_in_url = false;
   m_port_in_header = false;
+  m_host_mime = NULL;
   // Check in the URL first, then the HOST field.
-  if (0 != (m_host = url->host_get(&m_host_length))) {
+  if (0 != url->host_get(&m_host_length)) {
     m_target_in_url = true;
     m_port = url->port_get();
     m_port_in_header = 0 != url->port_get_raw();
-  } else if (0 != (m_host = const_cast<HTTPHdr*>(this)->value_get(MIME_FIELD_HOST, MIME_LEN_HOST, &m_host_length))) {
-    // Check for port in the host.
-    char const* colon = static_cast<char const*>(memchr(m_host, ':', m_host_length));
-    
-    if (colon) {
-      m_host_length = colon - m_host; // Length of just the host in the value.
+    m_host_mime = NULL;
+  } else if (0 != (m_host_mime = const_cast<HTTPHdr*>(this)->get_host_port_values(0, &m_host_length, &port_ptr, 0))) {
+    if (port_ptr) {
       m_port = 0;
-      for ( ++colon ; is_digit(*colon) ; ++colon )
-	m_port = m_port * 10 + *colon - '0';
-      m_port_in_header = 0 != m_port;
+      for ( ; is_digit(*port_ptr) ; ++port_ptr )
+        m_port = m_port * 10 + *port_ptr - '0';
+      m_port_in_header = (0 != m_port);
     }
     m_port = url_canonicalize_port(url->m_url_impl->m_url_type, m_port);
-  } else {
-    m_host_length = 0; // reset in case any earlier check corrupted it
   }
+
   m_target_cached = true;
 }
 
@@ -1480,13 +1493,16 @@ HTTPHdr::set_url_target_from_host_field(URL* url) {
   if (!url) {
     // Use local cached URL and don't copy if the target
     // is already there.
-    if (!m_target_in_url && m_host_length) {
-      m_url_cached.host_set(m_host, m_host_length);
+    if (!m_target_in_url && m_host_mime && m_host_length) {
+      m_url_cached.host_set(m_host_mime->m_ptr_value, m_host_length);
       if (m_port_in_header) m_url_cached.port_set(m_port);
       m_target_in_url = true; // it's there now.
     }
   } else {
-    url->host_set(m_host, m_host_length);
+    int host_len = 0;
+    char const *host = NULL;
+    host = host_get(&host_len);
+    url->host_set(host, host_len);
     if (m_port_in_header) url->port_set(m_port);
   }
 }
@@ -1516,9 +1532,9 @@ HTTPHdr::url_string_get(Arena* arena, int* length) {
        2) The values were in a HTTP header field.
     */
 
-    if (!m_target_in_url && m_host_length) {
+    if (!m_target_in_url && m_host_length && m_host_mime) {
       assert(0 == ui->m_ptr_host); // shouldn't be non-zero if not in URL.
-      ui->m_ptr_host = m_host;
+      ui->m_ptr_host = m_host_mime->m_ptr_value;
       ui->m_len_host = m_host_length;
       should_reset_host = true;
     }
@@ -1545,14 +1561,11 @@ HTTPHdr::url_string_get(Arena* arena, int* length) {
  ***********************************************************************/
 
 int
-HTTPHdr::unmarshal(char *buf, int len, RefCountObj * block_ref)
+HTTPHdr::unmarshal(char *buf, int len, RefCountObj *block_ref)
 {
-  m_heap = (HdrHeap *) buf;
+  m_heap = (HdrHeap *)buf;
 
-  int res = m_heap->unmarshal(len,
-                              HDR_HEAP_OBJ_HTTP_HEADER,
-                              (HdrHeapObjImpl **) & m_http,
-                              block_ref);
+  int res = m_heap->unmarshal(len, HDR_HEAP_OBJ_HTTP_HEADER, (HdrHeapObjImpl **) & m_http, block_ref);
 
   if (res > 0) {
     m_mime = m_http->m_fields_impl;
@@ -1564,7 +1577,7 @@ HTTPHdr::unmarshal(char *buf, int len, RefCountObj * block_ref)
 }
 
 int
-HTTPHdrImpl::marshal(MarshalXlate * ptr_xlate, int num_ptr, MarshalXlate * str_xlate, int num_str)
+HTTPHdrImpl::marshal(MarshalXlate *ptr_xlate, int num_ptr, MarshalXlate *str_xlate, int num_str)
 {
 
   if (m_polarity == HTTP_TYPE_REQUEST) {
@@ -1600,7 +1613,7 @@ HTTPHdrImpl::unmarshal(intptr_t offset)
 
 
 void
-HTTPHdrImpl::move_strings(HdrStrHeap * new_heap)
+HTTPHdrImpl::move_strings(HdrStrHeap *new_heap)
 {
   if (m_polarity == HTTP_TYPE_REQUEST) {
     HDR_MOVE_STR(u.req.m_ptr_method, u.req.m_len_method);
@@ -1612,7 +1625,7 @@ HTTPHdrImpl::move_strings(HdrStrHeap * new_heap)
 }
 
 void
-HTTPHdrImpl::check_strings(HeapCheck * heaps, int num_heaps)
+HTTPHdrImpl::check_strings(HeapCheck *heaps, int num_heaps)
 {
 
   if (m_polarity == HTTP_TYPE_REQUEST) {
@@ -1656,7 +1669,7 @@ HTTPCacheAlt::destroy()
 }
 
 void
-HTTPCacheAlt::copy(HTTPCacheAlt * to_copy)
+HTTPCacheAlt::copy(HTTPCacheAlt *to_copy)
 {
 
   m_magic = to_copy->m_magic;
@@ -1692,7 +1705,7 @@ HTTPInfo::create()
 }
 
 void
-HTTPInfo::copy(HTTPInfo * hi)
+HTTPInfo::copy(HTTPInfo *hi)
 {
 
   if (m_alt && m_alt->m_writeable) {
@@ -1777,7 +1790,7 @@ HTTPInfo::marshal(char *buf, int len)
 }
 
 int
-HTTPInfo::unmarshal(char *buf, int len, RefCountObj * block_ref)
+HTTPInfo::unmarshal(char *buf, int len, RefCountObj *block_ref)
 {
   HTTPCacheAlt *alt = (HTTPCacheAlt *) buf;
   int orig_len = len;
@@ -1899,7 +1912,7 @@ HTTPInfo::check_marshalled(char *buf, int len)
 //     to set up the reference
 //
 void
-HTTPInfo::set_buffer_reference(RefCountObj * block_ref)
+HTTPInfo::set_buffer_reference(RefCountObj *block_ref)
 {
   ink_assert(m_alt->m_magic == CACHE_ALT_MAGIC_ALIVE);
 
