@@ -36,9 +36,6 @@
 #include "MIME.h"
 #include "HTTP.h"
 
-#define HDR_MAX_ALLOC_SIZE (HDR_HEAP_DEFAULT_SIZE - sizeof(HdrHeap))
-#define HDR_HEAP_HDR_SIZE ROUND(sizeof(HdrHeap), HDR_PTR_SIZE)
-#define STR_HEAP_HDR_SIZE sizeof(HdrStrHeap)
 #define MAX_LOST_STR_SPACE 1024
 
 Allocator hdrHeapAllocator("hdrHeap", HDR_HEAP_DEFAULT_SIZE);
@@ -55,7 +52,7 @@ obj_describe(HdrHeapObjImpl * obj, bool recurse)
 {
   static const char *obj_names[] = { "EMPTY", "RAW", "URL", "HTTP_HEADER", "MIME_HEADER", "FIELD_BLOCK" };
 
-  Debug("http", "%s 0x%X: [T: %d, L: %4d, OBJFLAGS: %X]  ",
+  Debug("http", "%s %p: [T: %d, L: %4d, OBJFLAGS: %X]  ",
         obj_names[obj->m_type], obj, obj->m_type, obj->m_length, obj->m_obj_flags);
 
   extern void url_describe(HdrHeapObjImpl * obj, bool recurse);
@@ -103,16 +100,11 @@ HdrHeap::init()
   //  garbage it is pointing to
   m_read_write_heap.m_ptr = NULL;
 
-  m_ronly_heap[0].m_heap_start = NULL;
-  m_ronly_heap[0].m_ref_count_ptr.m_ptr = NULL;
-  m_ronly_heap[0].m_locked = false;
-  m_ronly_heap[1].m_heap_start = NULL;
-  m_ronly_heap[1].m_ref_count_ptr.m_ptr = NULL;
-  m_ronly_heap[1].m_locked = false;
-  m_ronly_heap[2].m_heap_start = NULL;
-  m_ronly_heap[2].m_ref_count_ptr.m_ptr = NULL;
-  m_ronly_heap[2].m_locked = false;
-
+  for (int i = 0; i < HDR_BUF_RONLY_HEAPS; i++) {
+    m_ronly_heap[i].m_heap_start = NULL;
+    m_ronly_heap[i].m_ref_count_ptr.m_ptr = NULL;
+    m_ronly_heap[i].m_locked = false;
+  }
   m_lost_string_space = 0;
 
   ink_assert(m_free_size > 0);
@@ -126,7 +118,7 @@ new_HdrHeap(int size)
     size = HDR_HEAP_DEFAULT_SIZE;
     h = (HdrHeap *) hdrHeapAllocator.alloc_void();
   } else {
-    h = (HdrHeap *) xmalloc(size);
+    h = (HdrHeap *)ats_malloc(size);
   }
 
 //    Debug("hdrs", "Allocated header heap in size %d", size);
@@ -157,7 +149,7 @@ new_HdrStrHeap(int requested_size)
     sh = (HdrStrHeap *) strHeapAllocator.alloc_void();
   } else {
     alloc_size = ROUND(alloc_size, HDR_STR_HEAP_DEFAULT_SIZE*2);
-    sh = (HdrStrHeap *) xmalloc(alloc_size);
+    sh = (HdrStrHeap *)ats_malloc(alloc_size);
   }
 
 //    Debug("hdrs", "Allocated string heap in size %d", alloc_size);
@@ -172,10 +164,6 @@ new_HdrStrHeap(int requested_size)
 
   ink_assert(sh->m_free_size > 0);
 
-#ifdef PURIFY
-  memset(sh->m_free_start, '#', sh->m_free_size);
-#endif
-
   return sh;
 }
 
@@ -187,14 +175,13 @@ HdrHeap::destroy()
   }
 
   m_read_write_heap = NULL;
-  m_ronly_heap[0].m_ref_count_ptr = NULL;
-  m_ronly_heap[1].m_ref_count_ptr = NULL;
-  m_ronly_heap[2].m_ref_count_ptr = NULL;
+  for (int i = 0; i < HDR_BUF_RONLY_HEAPS; i++)
+    m_ronly_heap[i].m_ref_count_ptr = NULL;
 
   if (m_size == HDR_HEAP_DEFAULT_SIZE) {
     hdrHeapAllocator.free_void(this);
   } else {
-    xfree(this);
+    ats_free(this);
   }
 }
 
@@ -313,19 +300,8 @@ FAILED:
 char *
 HdrHeap::expand_str(const char *old_str, int old_len, int new_len)
 {
-  char *rw_ptr = (char *) m_read_write_heap.m_ptr;
-
-  if (rw_ptr) {
-    // First check to see the old string is in this read-write string
-    // heap
-    if (old_str >= rw_ptr + STR_HEAP_HDR_SIZE && old_str < rw_ptr + m_read_write_heap->m_heap_size) {
-      // We're in the heap.  Try to grow the string
-      char *r = m_read_write_heap->expand((char *) old_str, old_len, new_len);
-      if (r) {
-        return r;
-      }
-    }
-  }
+  if (m_read_write_heap && m_read_write_heap->contains(old_str))
+    return m_read_write_heap->expand((char *)old_str, old_len, new_len);
 
   return NULL;
 }
@@ -338,7 +314,7 @@ HdrHeap::expand_str(const char *old_str, int old_len, int new_len)
 char *
 HdrHeap::duplicate_str(const char *str, int nbytes)
 {
-  ProtectHeaps protect(this); // Don't let the source get de-allocated.
+  HeapGuard guard(this, str); // Don't let the source get de-allocated.
   char *new_str = allocate_str(nbytes);
 
   memcpy(new_str, str, nbytes);
@@ -679,8 +655,8 @@ HdrHeap::marshal(char *buf, int len)
   marshal_hdr->m_ronly_heap[0].m_heap_start = (char *)(intptr_t)marshal_hdr->m_size;     // offset
   marshal_hdr->m_ronly_heap[0].m_ref_count_ptr.m_ptr = NULL;
 
-  marshal_hdr->m_ronly_heap[1].m_heap_start = NULL;
-  marshal_hdr->m_ronly_heap[2].m_heap_start = NULL;
+  for (int i = 1; i < HDR_BUF_RONLY_HEAPS; i++)
+    marshal_hdr->m_ronly_heap[i].m_heap_start = NULL;
 
   // Next order of business is to copy over string heaps
   //   As we are copying over the string heaps, build
@@ -1129,7 +1105,7 @@ HdrStrHeap::free()
   if (m_heap_size == HDR_STR_HEAP_DEFAULT_SIZE) {
     strHeapAllocator.free_void(this);
   } else {
-    xfree(this);
+    ats_free(this);
   }
 }
 
