@@ -160,7 +160,7 @@ HdrTest::test_error_page_selection()
   for (i = 0; i < nsets; i++) {
     HttpBodySetRawData *body_set;
 
-    body_set = (HttpBodySetRawData *) xmalloc(sizeof(HttpBodySetRawData));
+    body_set = (HttpBodySetRawData *)ats_malloc(sizeof(HttpBodySetRawData));
     body_set->magic = 0;
     body_set->set_name = (char *) (sets[i].set_name);
     body_set->content_language = (char *) (sets[i].content_language);
@@ -388,10 +388,32 @@ HdrTest::test_url()
     "http://foo:bar@some.place:80",
     "http://foo:bar@some.place:80/",
 
+    // Some address stuff
+    "http://172.16.28.101",
+    "http://172.16.28.101:8080",
+    "http://[::]",
+    "http://[::1]",
+    "http://[fc01:172:16:28::101]",
+    "http://[fc01:172:16:28::101]:80",
+    "http://[fc01:172:16:28:BAAD:BEEF:DEAD:101]",
+    "http://[fc01:172:16:28:BAAD:BEEF:DEAD:101]:8080",
+    "http://172.16.28.101/some/path",
+    "http://172.16.28.101:8080/some/path",
+    "http://[::1]/some/path",
+    "http://[fc01:172:16:28::101]/some/path",
+    "http://[fc01:172:16:28::101]:80/some/path",
+    "http://[fc01:172:16:28:BAAD:BEEF:DEAD:101]/some/path",
+    "http://[fc01:172:16:28:BAAD:BEEF:DEAD:101]:8080/some/path",
+    "http://172.16.28.101/",
+    "http://[fc01:172:16:28:BAAD:BEEF:DEAD:101]:8080/",
+
+
     "foo:bar@some.place",
     "foo:bar@some.place/",
     "http://foo:bar@some.place",
     "http://foo:bar@some.place/",
+    "http://foo:bar@[::1]:8080/",
+    "http://foo@[::1]",
 
     "mms://sm02.tsqa.example.com/0102rally.asf",
     "pnm://foo:bar@some.place:80/path;params?query#fragment",
@@ -401,6 +423,19 @@ HdrTest::test_url()
   };
   static int nstrs = sizeof(strs) / sizeof(strs[0]);
 
+  static char const* bad[] = {
+    "http://[1:2:3:4:5:6:7:8:9]",
+    "http://1:2:3:4:5:6:7:8:A:B",
+    "http://bob.com[::1]",
+    "http://[::1].com"
+    "http://foo:bar:baz@bob.com/",
+    "http://foo:bar:baz@[::1]:8080/",
+    "http://]",
+    "http://:",
+    "http:/"
+  };
+  static int nbad = sizeof(bad) / sizeof(bad[0]);
+
   int err, failed;
   URL url;
   const char *start;
@@ -456,6 +491,33 @@ HdrTest::test_url()
     url.destroy();
   }
 
+  for (i = 0 ; i < nbad ; ++i) {
+    char const* x = bad[i];
+    url.create(NULL);
+    err = url.parse(x, strlen(x));
+    url.destroy();
+    if (err == PARSE_DONE) {
+      failed = 1;
+      printf("Successfully parsed invalid url '%s'", x);
+      break;
+    }
+  }
+
+#if 0
+  if (!failed) {
+    Note("URL performance test start");
+    for (int j = 0 ; j < 100000 ; ++j) {
+      for (i = 0 ; i < nstrs ; ++i) {
+        char const* x = strs[i];
+        url.create(NULL);
+        err = url.parse(x, strlen(x));
+        url.destroy();
+      }
+    }
+    Note("URL performance test end");
+  }
+#endif
+
   return (failures_to_status("test_url", failed));
 }
 
@@ -910,28 +972,28 @@ comp_http_hdr(HTTPHdr * h1, HTTPHdr * h2)
     return "length mismatch";
   }
 
-  h1_pbuf = (char *) xmalloc(h1_len + 1);
-  h2_pbuf = (char *) xmalloc(h2_len + 1);
+  h1_pbuf = (char *)ats_malloc(h1_len + 1);
+  h2_pbuf = (char *)ats_malloc(h2_len + 1);
 
   p_index = p_dumpoffset = 0;
   rval = h1->print(h1_pbuf, h1_len, &p_index, &p_dumpoffset);
   if (rval != 1) {
-    xfree(h1_pbuf);
-    xfree(h2_pbuf);
+    ats_free(h1_pbuf);
+    ats_free(h2_pbuf);
     return "hdr print failed";
   }
 
   p_index = p_dumpoffset = 0;
   rval = h2->print(h2_pbuf, h2_len, &p_index, &p_dumpoffset);
   if (rval != 1) {
-    xfree(h1_pbuf);
-    xfree(h2_pbuf);
+    ats_free(h1_pbuf);
+    ats_free(h2_pbuf);
     return "hdr print failed";
   }
 
   rval = memcmp(h1_pbuf, h2_pbuf, h1_len);
-  xfree(h1_pbuf);
-  xfree(h2_pbuf);
+  ats_free(h1_pbuf);
+  ats_free(h2_pbuf);
 
   if (rval != 0) {
     return "compare failed";
@@ -1013,6 +1075,10 @@ HdrTest::test_http_hdr_copy_over_aux(int testnum, const char *request, const cha
   if (comp_str)
     goto done;
 
+  // The APIs for copying headers uses memcpy() which can be unsafe for
+  // overlapping memory areas. It's unclear to me why these tests were
+  // created in the first place honestly, since nothing else does this.
+#if 0
     /*** (4) Copying over yourself ***/
   copy1.copy(&copy1);
   comp_str = comp_http_hdr(&req_hdr, &copy1);
@@ -1023,6 +1089,7 @@ HdrTest::test_http_hdr_copy_over_aux(int testnum, const char *request, const cha
   comp_str = comp_http_hdr(&resp_hdr, &copy2);
   if (comp_str)
     goto done;
+#endif
 
     /*** (5) Gender bending copying ***/
   copy1.copy(&resp_hdr);
@@ -1074,7 +1141,7 @@ HdrTest::test_http_hdr_print_and_copy_aux(int testnum,
   int cpy_bufsize = sizeof(cpy_buf);
   int cpy_bufindex, cpy_dumpoffset, cpy_ret;
 
-  char marshal_buf[2048];
+  char *marshal_buf = (char*)ats_malloc(2048);
   int marshal_bufsize = sizeof(cpy_buf);
 
     /*** (1) parse the request string into hdr ***/
@@ -1106,6 +1173,7 @@ HdrTest::test_http_hdr_print_and_copy_aux(int testnum,
   marshal_hdr.unmarshal(marshal_buf, marshal_len, &ref);
   new_hdr.create(HTTP_TYPE_REQUEST);
   new_hdr.copy(&marshal_hdr);
+  ats_free(marshal_buf);
 
     /*** (3) print the request header and copy to buffers ***/
 
@@ -1559,9 +1627,6 @@ HdrTest::test_regex()
 
 /*-------------------------------------------------------------------------
   -------------------------------------------------------------------------*/
-#if defined (_WIN32)
-#pragma warning (once:4305)
-#endif
 
 int
 HdrTest::test_accept_language_match()
@@ -2213,6 +2278,3 @@ HdrTest::failures_to_status(const char *testname, int nfail)
   return ((nfail > 0) ? 0 : 1);
 }
 
-#if defined (_WIN32)
-#pragma warning (default:4305)
-#endif
