@@ -112,6 +112,14 @@
     return len; \
 }
 
+#define DEFAULT_IP_FIELD {\
+    int len = sizeof(LogFieldIp); \
+    if (buf) { \
+      len = marshal_ip (buf, NULL); \
+    } \
+    return len; \
+}
+
 // should be at least 22 bytes to always accomodate a converted
 // MgmtInt, MgmtIntCounter or MgmtFloat. 22 bytes is enough for 64 bit
 // ints + sign + eos, and enough for %e floating point representation
@@ -172,6 +180,7 @@ public:
   inkcoreapi virtual int marshal_client_req_url_canon(char *);  // STR
   inkcoreapi virtual int marshal_client_req_unmapped_url_canon(char *); // STR
   inkcoreapi virtual int marshal_client_req_unmapped_url_path(char *);  // STR
+  inkcoreapi virtual int marshal_client_req_unmapped_url_host(char *);  // STR
   inkcoreapi virtual int marshal_client_req_url_path(char *);   // STR
   inkcoreapi virtual int marshal_client_req_url_scheme(char *); // STR
   inkcoreapi virtual int marshal_client_req_http_version(char *);       // INT
@@ -215,6 +224,15 @@ public:
   inkcoreapi virtual int marshal_server_resp_header_len(char *);        // INT
   inkcoreapi virtual int marshal_server_resp_http_version(char *);      // INT
 
+  //
+  // cache -> client fields
+  //
+  inkcoreapi virtual int marshal_cache_resp_status_code(char *);  // INT
+  inkcoreapi virtual int marshal_cache_resp_content_len(char *);  // INT
+  inkcoreapi virtual int marshal_cache_resp_header_len(char *);   // INT
+  inkcoreapi virtual int marshal_cache_resp_http_version(char *); // INT
+
+
   //
   // congestion control -- client_retry_after_time
   //
@@ -270,7 +288,9 @@ public:
   static int unmarshal_http_version(char **buf, char *dest, int len);
   static int unmarshal_http_text(char **buf, char *dest, int len);
   static int unmarshal_http_status(char **buf, char *dest, int len);
-  static int unmarshal_ip(char **buf, char *dest, int len, Ptr<LogFieldAliasMap> map);
+  static int unmarshal_ip(char** buf, IpEndpoint* dest);
+  static int unmarshal_ip_to_str(char **buf, char *dest, int len);
+  static int unmarshal_ip_to_hex(char** buf, char* dest, int len);
   static int unmarshal_hierarchy(char **buf, char *dest, int len, Ptr<LogFieldAliasMap> map);
   static int unmarshal_finish_status(char **buf, char *dest, int len, Ptr<LogFieldAliasMap> map);
   static int unmarshal_cache_code(char **buf, char *dest, int len, Ptr<LogFieldAliasMap> map);
@@ -286,12 +306,13 @@ public:
   // so that there are no alignment problems with the int values.
   //
   static int round_strlen(int len);
-  static int strlen(char *str);
+  static int strlen(char const* str);
 
 public:
   inkcoreapi static void marshal_int(char *dest, int64_t source);
   inkcoreapi static void marshal_str(char *dest, const char *source, int padded_len);
   inkcoreapi static void marshal_mem(char *dest, const char *source, int actual_len, int padded_len);
+  inkcoreapi static int marshal_ip(char *dest, sockaddr const* ip);
 
   bool initialized;
 
@@ -315,7 +336,7 @@ LogAccess::round_strlen(int len)
   -------------------------------------------------------------------------*/
 
 inline int
-LogAccess::strlen(char *str)
+LogAccess::strlen(char const* str)
 {
   if (str == NULL || str[0] == 0) {
     return round_strlen(sizeof(DEFAULT_STR));
@@ -327,7 +348,7 @@ LogAccess::strlen(char *str)
 inline void
 LogAccess::marshal_int(char *dest, int64_t source)
 {
-  // TODO: This used to do htonl on the source
+  // TODO: This used to do htonl on the source. TS-1156
   *((int64_t *)dest) = source;
 }
 
