@@ -88,25 +88,13 @@ static const int FILESIZE_SAFE_THRESHOLD_FACTOR = 10;
 
 LogFile::LogFile(const char *name, const char *header, LogFileFormat format,
                  uint64_t signature, size_t ascii_buffer_size, size_t max_line_size, size_t overspill_report_count)
-  :
-m_file_format(format),
-m_name(xstrdup(name)),
-m_header(xstrdup(header)),
-m_signature(signature),
-m_meta_info(NULL),
-m_max_line_size(max_line_size),
-m_overspill_report_count(overspill_report_count)
-{
-  init();
-  m_ascii_buffer_size = (ascii_buffer_size < max_line_size ? max_line_size : ascii_buffer_size);
-  m_ascii_buffer = NEW(new char[m_ascii_buffer_size]);
-  m_overspill_buffer = NEW(new char[m_max_line_size]);
-
-  Debug("log-file", "exiting LogFile constructor, m_name=%s, this=%p", m_name, this);
-}
-
-void
-LogFile::init()
+  : m_file_format(format),
+    m_name(ats_strdup(name)),
+    m_header(ats_strdup(header)),
+    m_signature(signature),
+    m_meta_info(NULL),
+    m_max_line_size(max_line_size),
+    m_overspill_report_count(overspill_report_count)
 {
   delete m_meta_info;
   m_meta_info = NULL;
@@ -118,11 +106,43 @@ LogFile::init()
   m_end_time = 0L;
   m_bytes_written = 0;
   m_size_bytes = 0;
-  m_has_size_limit = false;
-  m_size_limit_bytes = 0;
-  m_filesystem_checks_done = false;
+  m_ascii_buffer_size = (ascii_buffer_size < max_line_size ? max_line_size : ascii_buffer_size);
+  m_ascii_buffer = NEW(new char[m_ascii_buffer_size]);
+  m_overspill_buffer = NEW(new char[m_max_line_size]);
+
+  Debug("log-file", "exiting LogFile constructor, m_name=%s, this=%p", m_name, this);
 }
 
+/*-------------------------------------------------------------------------
+  LogFile::LogFile
+
+  This (copy) contructor builds a LogFile object from another LogFile object.
+  -------------------------------------------------------------------------*/
+
+LogFile::LogFile (const LogFile& copy)
+  : m_file_format (copy.m_file_format),
+    m_name  (ats_strdup (copy.m_name)),
+    m_header  (ats_strdup (copy.m_header)),
+    m_signature (copy.m_signature),
+    m_meta_info (NULL),
+    m_ascii_buffer_size (copy.m_ascii_buffer_size),
+    m_max_line_size (copy.m_max_line_size),
+    m_overspill_bytes (0),
+    m_overspill_written (0),
+    m_attempts_to_write_overspill (0),
+    m_overspill_report_count (copy.m_overspill_report_count),
+    m_fd (-1),
+    m_start_time (0L),
+    m_end_time (0L),
+    m_bytes_written (0)
+{
+    ink_debug_assert(m_ascii_buffer_size >= m_max_line_size);
+    m_ascii_buffer = NEW (new char[m_ascii_buffer_size]);
+    m_overspill_buffer = NEW (new char[m_max_line_size]);
+
+    Debug("log-file", "exiting LogFile copy constructor, m_name=%s, this=%p",
+          m_name, this);
+}
 /*-------------------------------------------------------------------------
   LogFile::~LogFile
   -------------------------------------------------------------------------*/
@@ -132,8 +152,8 @@ LogFile::~LogFile()
   Debug("log-file", "entering LogFile destructor, this=%p", this);
   close_file();
 
-  xfree(m_name);
-  xfree(m_header);
+  ats_free(m_name);
+  ats_free(m_header);
   delete m_meta_info;
   delete[]m_ascii_buffer;
   m_ascii_buffer = 0;
@@ -160,8 +180,8 @@ bool LogFile::exists(const char *pathname)
 void
 LogFile::change_name(char *new_name)
 {
-  xfree(m_name);
-  m_name = xstrdup(new_name);
+  ats_free(m_name);
+  m_name = ats_strdup(new_name);
 }
 
 /*-------------------------------------------------------------------------
@@ -171,8 +191,8 @@ LogFile::change_name(char *new_name)
 void
 LogFile::change_header(const char *header)
 {
-  xfree(m_header);
-  m_header = xstrdup(header);
+  ats_free(m_header);
+  m_header = ats_strdup(header);
 }
 
 /*-------------------------------------------------------------------------
@@ -217,7 +237,7 @@ LogFile::open_file()
 
   if (m_file_format == ASCII_PIPE) {
 #ifdef ASCII_PIPE_FORMAT_SUPPORTED
-    if (mknod(m_name, S_IFIFO | S_IRUSR | S_IWUSR, 0) < 0) {
+    if (mkfifo(m_name, S_IRUSR | S_IWUSR) < 0) {
       if (errno != EEXIST) {
         Error("Could not create named pipe %s for logging: %s", m_name, strerror(errno));
         return LOG_FILE_COULD_NOT_CREATE_PIPE;
@@ -298,7 +318,6 @@ LogFile::close_file()
     // are running) and these won't be visible in Traffic Manager
     LOG_SUM_GLOBAL_DYN_STAT(log_stat_log_files_open_stat, -1);
   }
-  m_filesystem_checks_done = false;
 }
 
 /*-------------------------------------------------------------------------
@@ -426,7 +445,7 @@ LogFile::roll(long interval_start, long interval_end)
   snprintf(roll_name, MAXPATHLEN, "%s%s%s.%s-%s%s",
                m_name,
                LOGFILE_SEPARATOR_STRING,
-               this_machine()->hostname, start_time_ext, end_time_ext, LOGFILE_ROLLED_EXTENSION);
+               Machine::instance()->hostname, start_time_ext, end_time_ext, LOGFILE_ROLLED_EXTENSION);
 
   //
   // It may be possible that the file we want to roll into already
@@ -441,7 +460,7 @@ LogFile::roll(long interval_start, long interval_end)
     snprintf(roll_name, MAXPATHLEN, "%s%s%s.%s-%s.%d%s",
                  m_name,
                  LOGFILE_SEPARATOR_STRING,
-                 this_machine()->hostname, start_time_ext, end_time_ext, version, LOGFILE_ROLLED_EXTENSION);
+                 Machine::instance()->hostname, start_time_ext, end_time_ext, version, LOGFILE_ROLLED_EXTENSION);
     version++;
   }
 
@@ -457,6 +476,7 @@ LogFile::roll(long interval_start, long interval_end)
   // reset m_start_time
   //
   m_start_time = 0;
+  m_bytes_written = 0;
 
   Status("The logfile %s was rolled to %s.", m_name, roll_name);
 
@@ -469,16 +489,16 @@ LogFile::roll(long interval_start, long interval_end)
   Write the given LogBuffer data onto this file
   -------------------------------------------------------------------------*/
 int
-LogFile::write(LogBuffer * lb, size_t * to_disk, size_t * to_net, size_t * to_pipe)
+LogFile::write(LogBuffer * lb)
 {
-  NOWARN_UNUSED(to_net);
   if (lb == NULL) {
     Note("Cannot write LogBuffer to LogFile %s; LogBuffer is NULL", m_name);
     return -1;
   }
   LogBufferHeader *buffer_header = lb->header();
   if (buffer_header == NULL) {
-    Note("Cannot write LogBuffer to LogFile %s; LogBufferHeader is NULL", m_name);
+    Note("Cannot write LogBuffer to LogFile %s; LogBufferHeader is NULL",
+        m_name);
     return -1;
   }
   if (buffer_header->entry_count == 0) {
@@ -486,13 +506,7 @@ LogFile::write(LogBuffer * lb, size_t * to_disk, size_t * to_net, size_t * to_pi
     Note("LogBuffer with 0 entries for LogFile %s, nothing to write", m_name);
     return 0;
   }
-  // check if size limit has been exceeded and roll file if that is the case
 
-  if (size_limit_exceeded()) {
-    Warning("File %s will be rolled because its size is close to "
-            "or exceeds the operating system filesize limit", get_name());
-    roll(0, LogUtils::timestamp());
-  }
   // make sure we're open & ready to write
 
   check_fd();
@@ -510,30 +524,21 @@ LogFile::write(LogBuffer * lb, size_t * to_disk, size_t * to_net, size_t * to_pi
     // don't change between buffers), it's not worth trying to separate
     // out the buffer-dependent data from the buffer-independent data.
     //
-    if (!Log::config->logging_space_exhausted) {
-      bytes = writeln((char *) buffer_header, buffer_header->byte_count, m_fd, m_name);
-      if (bytes > 0 && to_disk) {
-        *to_disk += bytes;
-      }
+    bytes = ::write(m_fd, buffer_header, buffer_header->byte_count);
+    if (static_cast<uint32_t>(bytes) != buffer_header->byte_count) {
+      Warning("An error was encountered writing to %s: [tried %d, wrote %d, '%s']", m_name, buffer_header->byte_count, bytes, strerror(errno));
     }
-  } else if (m_file_format == ASCII_LOG) {
+  }
+  else if (m_file_format == ASCII_LOG || m_file_format == ASCII_PIPE) {
     bytes = write_ascii_logbuffer3(buffer_header);
-    if (bytes > 0 && to_disk) {
-      *to_disk += bytes;
-    }
 #if defined(LOG_BUFFER_TRACKING)
-    Debug("log-buftrak", "[%d]LogFile::write - ascii write complete", buffer_header->id);
+    Debug("log-buftrak", "[%d]LogFile::write - ascii write complete",
+        buffer_header->id);
 #endif // defined(LOG_BUFFER_TRACKING)
-  } else if (m_file_format == ASCII_PIPE) {
-    bytes = write_ascii_logbuffer3(buffer_header);
-    if (bytes > 0 && to_pipe) {
-      *to_pipe += bytes;
-    }
-#if defined(LOG_BUFFER_TRACKING)
-    Debug("log-buftrak", "[%d]LogFile::write - ascii pipe write complete", buffer_header->id);
-#endif // defined(LOG_BUFFER_TRACKING)
-  } else {
-    Error("Cannot write LogBuffer to LogFile %s; invalid file format: %d", m_name, m_file_format);
+  }
+  else {
+    Note("Cannot write LogBuffer to LogFile %s; invalid file format: %d",
+        m_name, m_file_format);
     return -1;
   }
 
@@ -546,14 +551,7 @@ LogFile::write(LogBuffer * lb, size_t * to_disk, size_t * to_net, size_t * to_pi
   if (!m_start_time)
     m_start_time = buffer_header->low_timestamp;
   m_end_time = buffer_header->high_timestamp;
-
-  // update bytes written and file size (if not a pipe)
-  //
   m_bytes_written += bytes;
-  if (m_file_format != ASCII_PIPE) {
-    m_size_bytes += bytes;
-  }
-
   return bytes;
 }
 
@@ -708,11 +706,11 @@ LogFile::write_ascii_logbuffer3(LogBufferHeader * buffer_header, char *alt_forma
       if (m_overspill_bytes) {
         ++m_attempts_to_write_overspill;
         if (m_overspill_report_count && (m_attempts_to_write_overspill % m_overspill_report_count == 0)) {
-          Warning("Have dropped %u records so far because buffer "
+          Warning("Have dropped %zu records so far because buffer "
                   "for %s is full", m_attempts_to_write_overspill, m_name);
         }
       } else if (m_attempts_to_write_overspill) {
-        Warning("Dropped %u records because buffer for %s was full", m_attempts_to_write_overspill, m_name);
+        Warning("Dropped %zu records because buffer for %s was full", m_attempts_to_write_overspill, m_name);
         m_attempts_to_write_overspill = 0;
       }
     }
@@ -831,36 +829,6 @@ LogFile::display(FILE * fd)
   fprintf(fd, "Logfile: %s, %s\n", get_name(), (is_open())? "file is open" : "file is not open");
 }
 
-int
-LogFile::do_filesystem_checks()
-{
-  int ret_val = 0;
-
-  int e = LogUtils::file_is_writeable(m_name, &m_size_bytes,
-                                      &m_has_size_limit,
-                                      &m_size_limit_bytes);
-
-  if (e == 1) {
-    Error("Log file %s is not a regular file or pipe", m_name);
-    ret_val = -1;
-  } else if (e == -1) {
-    Error("Filesystem checks for log file %s failed: %s", m_name, strerror(errno));
-    ret_val = -1;
-  } else if (m_has_size_limit) {
-    uint64_t safe_threshold =
-      (m_file_format == ASCII_PIPE ? 0 : Log::config->log_buffer_size * FILESIZE_SAFE_THRESHOLD_FACTOR);
-    if (safe_threshold > m_size_limit_bytes) {
-      Error("Filesize limit is too low for log file %s", m_name);
-      ret_val = -1;
-    } else {
-      m_size_limit_bytes = m_size_limit_bytes - safe_threshold;
-    }
-  }
-
-  m_filesystem_checks_done = true;
-  return ret_val;
-}
-
 /***************************************************************************
  LogFileList IS NOT USED
 ****************************************************************************/
@@ -886,7 +854,7 @@ MetaInfo::_build_name(const char *filename)
 
   // 7 = 1 (dot at beginning) + 5 (".meta") + 1 (null terminating)
   //
-  _filename = (char *) xmalloc(l + 7);
+  _filename = (char *)ats_malloc(l + 7);
 
   if (i < 0) {
     ink_string_concatenate_strings(_filename, ".", filename, ".meta", NULL);
