@@ -1,6 +1,6 @@
 /** @file
 
-  A brief file description
+  Standalone Collator
 
   @section license License
 
@@ -21,18 +21,12 @@
   limitations under the License.
  */
 
-/***************************************************************************
- sac.cc
-
- Standalone Collator
-
-
- ***************************************************************************/
 #include "ink_config.h"
 #include "ink_file.h"
 #include "ink_unused.h"
 #include "I_Layout.h"
 #include "I_Version.h"
+#include "P_Net.h"
 
 #define PROGRAM_NAME  "traffic_sac"
 
@@ -100,7 +94,7 @@ main(int argc, char *argv[])
   process_args(argument_descriptions, n_argument_descriptions, argv);
 
   // Get log directory
-  ink_strlcpy(system_log_dir, Layout::get()->logdir, PATH_NAME_MAX);
+  ink_strlcpy(system_log_dir, Layout::get()->logdir, sizeof(system_log_dir));
   if (access(system_log_dir, R_OK) == -1) {
     fprintf(stderr, "unable to change to log directory \"%s\" [%d '%s']\n", system_log_dir, errno, strerror(errno));
     fprintf(stderr, " please set correct path in env variable TS_ROOT \n");
@@ -108,7 +102,7 @@ main(int argc, char *argv[])
   }
 
   management_directory[0] = 0;
-  strncat(management_directory, system_config_directory, 256 - 1);
+  ink_strlcat(management_directory, system_config_directory, sizeof(management_directory));
 
   // check for the version number request
   //
@@ -140,7 +134,7 @@ main(int argc, char *argv[])
   eventProcessor.start(ink_number_of_processors());
   ink_net_init(makeModuleVersion(1, 0, PRIVATE_MODULE_HEADER));
   netProcessor.start();
-  create_this_machine(NULL, 0);
+  Machine::init();
 
   Log::init(Log::NO_REMOTE_MANAGEMENT | Log::STANDALONE_COLLATOR);
 
